@@ -9,6 +9,7 @@
 #include "arrays.h"
 #include "dei_rkck.h"
 #include "parser.h"
+#include "../math/ncm_c.h"
 
 enum spatial_curvature {flat,open,closed};
 
@@ -16,7 +17,7 @@ enum spatial_curvature {flat,open,closed};
  * All background parameters and evolution that other modules need to know.
  *
  * Once initialized by the backgound_init(), contains all necessary
- * information on the background evolution (excepted thermodynamics),
+ * information on the background evolution (except thermodynamics),
  * and in particular, a table of all background quantities as a
  * function of time and scale factor, used for interpolation in other
  * modules.
@@ -25,7 +26,7 @@ enum spatial_curvature {flat,open,closed};
 struct background
 {
   /** @name - input parameters initialized by user in input module
-   *  (all other quantitites are computed in this module, given these parameters
+   *  (all other quantities are computed in this module, given these parameters
    *   and the content of the 'precision' structure)
    *
    * The background cosmological parameters listed here form a parameter
@@ -37,38 +38,40 @@ struct background
 
   //@{
 
-  double H0; /**< \f$ H_0 \f$ : Hubble parameter (in fact, [H_0/c]) in \f$ Mpc^{-1} \f$ */
+  double H0; /**< \f$ H_0 \f$: Hubble parameter (in fact, [\f$H_0/c\f$]) in \f$ Mpc^{-1} \f$ */
 
-  double Omega0_g; /**< \f$ \Omega_{0 \gamma} \f$ : photons */
+  double Omega0_g; /**< \f$ \Omega_{0 \gamma} \f$: photons */
 
-  double T_cmb; /**< \f$ T_{cmb} \f$ : current CMB temperature in Kelvins */
+  double T_cmb; /**< \f$ T_{cmb} \f$: current CMB temperature in Kelvins */
 
-  double Omega0_b; /**< \f$ \Omega_{0 b} \f$ : baryons */
+  double Omega0_b; /**< \f$ \Omega_{0 b} \f$: baryons */
 
-  double Omega0_cdm; /**< \f$ \Omega_{0 cdm} \f$ : cold dark matter */
+  double Omega0_cdm; /**< \f$ \Omega_{0 cdm} \f$: cold dark matter */
 
-  double Omega0_lambda; /**< \f$ \Omega_{0_\Lambda} \f$ : cosmological constant */
+  double Omega0_lambda; /**< \f$ \Omega_{0_\Lambda} \f$: cosmological constant */
 
-  double Omega0_fld; /**< \f$ \Omega_{0 de} \f$ : fluid with constant
+  double Omega0_fld; /**< \f$ \Omega_{0 de} \f$: fluid with constant
 			\f$ w \f$ and \f$ c_s^2 \f$ */
-  double w0_fld; /**< \f$ w0_{DE} \f$ : current fluid equation of state parameter */
-  double wa_fld; /**< \f$ wa_{DE} \f$ : fluid equation of state parameter derivative */
+  double w0_fld; /**< \f$ w0_{DE} \f$: current fluid equation of state parameter */
+  double wa_fld; /**< \f$ wa_{DE} \f$: fluid equation of state parameter derivative */
 
-  double cs2_fld; /**< \f$ c^2_{s~DE} \f$ : sound speed of the fluid
+  double cs2_fld; /**< \f$ c^2_{s~DE} \f$: sound speed of the fluid
 		     in the frame comoving with the fluid (so, this is
 		     not [delta p/delta rho] in the synchronous or
 		     newtonian gauge!!!) */
 
-  double Omega0_ur; /**< \f$ \Omega_{0 \nu r} \f$ : ultra-relativistic neutrinos */
+  double Omega0_ur; /**< \f$ \Omega_{0 \nu r} \f$: ultra-relativistic neutrinos */
 
-  double Omega0_dcdmdr; /**< \f$ \Omega_{0 dcdm}+\Omega_{0 dr} \f$ : dcdm decaying to dr */
+  double Omega0_dcdmdr; /**< \f$ \Omega_{0 dcdm}+\Omega_{0 dr} \f$: decaying cold dark matter (dcdm) decaying to dark radiation (dr) */
 
-  double Gamma_dcdm; /**< \f$ \Gamma_{dcdm} \f$ : decay constant for decaying cold dark matter */
+  double Gamma_dcdm; /**< \f$ \Gamma_{dcdm} \f$: decay constant for decaying cold dark matter */
 
-  double Omega0_scf;        /**< \f$ \Omega_{0 scf} \f$ : scalar field */
-  short attractor_ic_scf;   /** < whether the scalar field has attractor initial conditions */
-  double phi_ini_scf;       /**< \f$ \phi(t_0) \f$ : scalar field initial value */
-  double phi_prime_ini_scf; /**< \f$ d\phi(t_0)/d\tau \f$ : scalar field initial derivative wrt conformal time */
+  double Omega_ini_dcdm;    /**< \f$ \Omega_{ini,dcdm} \f$: rescaled initial value for dcdm density (see 1407.2418 for definitions) */
+
+  double Omega0_scf;        /**< \f$ \Omega_{0 scf} \f$: scalar field */
+  short attractor_ic_scf;   /**< whether the scalar field has attractor initial conditions */
+  double phi_ini_scf;       /**< \f$ \phi(t_0) \f$: scalar field initial value */
+  double phi_prime_ini_scf; /**< \f$ d\phi(t_0)/d\tau \f$: scalar field initial derivative wrt conformal time */
   double * scf_parameters;  /**< list of parameters describing the scalar field potential */
   int scf_parameters_size;  /**< size of scf_parameters */
   int scf_tuning_index;     /**< index in scf_parameters used for tuning */
@@ -77,14 +80,13 @@ struct background
   //double scf_B; /**< \f$ \alpha \f$ : Albrecht-Skordis field shift */
   //double scf_A; /**< \f$ \alpha \f$ : Albrecht-Skordis offset */
 
+  double Omega0_k; /**< \f$ \Omega_{0_k} \f$: curvature contribution */
 
-  double Omega0_k; /**< \f$ \Omega_{0_k} \f$ : curvature contribution */
-
-  int N_ncdm;                            /**< Number of distinguishabe ncdm species */
-  double * M_ncdm;                       /**<vector of masses of non-cold relic:
+  int N_ncdm;                            /**< Number of distinguishable ncdm species */
+  double * M_ncdm;                       /**< vector of masses of non-cold relic:
                                              dimensionless ratios m_ncdm/T_ncdm */
-  double * Omega0_ncdm, Omega0_ncdm_tot; /**<Omega0_ncdm for each species and for the total Omega0_ncdm */
-  double * deg_ncdm, deg_ncdm_default;   /**<vector of degeneracy parameters in factor
+  double * Omega0_ncdm, Omega0_ncdm_tot; /**< Omega0_ncdm for each species and for the total Omega0_ncdm */
+  double * deg_ncdm, deg_ncdm_default;   /**< vector of degeneracy parameters in factor
                                              of p-s-d: 1 for one family of neutrinos
                                              (= one neutrino plus its anti-neutrino,
                                              total g*=1+1=2, so deg = 0.5 g*); and its
@@ -102,7 +104,7 @@ struct background
 					     ksi_ncdm1/T_ncdm1; and its
 					     default value */
   double * ncdm_psd_parameters;         /**< list of parameters for specifying/modifying
-                                             ncdm p.s.d.'s, to be cutomized for given model
+                                             ncdm p.s.d.'s, to be customized for given model
                                              (could be e.g. mixing angles) */
   /* end of parameters for analytical ncdm p-s-d */
 
@@ -112,24 +114,21 @@ struct background
   char * ncdm_psd_files;                /**< list of filenames for tabulated p-s-d */
   /* end of parameters for tabulated ncdm p-s-d */
 
-  /* rescaled initial value for dcdm density. */
-  double Omega_ini_dcdm;
-
   //@}
 
   /** @name - related parameters */
 
   //@{
 
-  double h; /** reduced Hubble parameter */
+  double h; /**< reduced Hubble parameter */
   double age; /**< age in Gyears */
   double conformal_age; /**< conformal age in Mpc */
-  double K; /**< \f$ K \f$ : Curvature parameter K=-Omega0_k*a_today^2*H_0^2; */
+  double K; /**< \f$ K \f$: Curvature parameter \f$ K=-\Omega0_k*a_{today}^2*H_0^2\f$; */
   int sgnK; /**< K/|K|: -1, 0 or 1 */
-  double * m_ncdm_in_eV; /**< list of ncdm masses in eV (infered from M_ncdm and other parameters above) */
+  double * m_ncdm_in_eV; /**< list of ncdm masses in eV (inferred from M_ncdm and other parameters above) */
   double Neff; /**< so-called "effective neutrino number", computed at earliest time in interpolation table */
-  double Omega0_dcdm; /**< \f$ \Omega_{0 dcdm} \f$ : decaying cold dark matter */
-  double Omega0_dr; /**< \f$ \Omega_{0 dr} \f$ : decay radiation */
+  double Omega0_dcdm; /**< \f$ \Omega_{0 dcdm} \f$: decaying cold dark matter */
+  double Omega0_dr; /**< \f$ \Omega_{0 dr} \f$: decay radiation */
 
 
   //@}
@@ -138,7 +137,7 @@ struct background
 
   //@{
 
-  double a_today; /** scale factor today (arbitrary and irrelevant for most purposes) */
+  double a_today; /**< scale factor today (arbitrary and irrelevant for most purposes) */
 
   //@}
 
@@ -147,10 +146,10 @@ struct background
   //@{
 
   int index_bg_a;             /**< scale factor */
-  int index_bg_H;             /**< Hubble parameter in Mpc^{-1} */
+  int index_bg_H;             /**< Hubble parameter in \f$Mpc^{-1}\f$ */
   int index_bg_H_prime;       /**< its derivative w.r.t. conformal time */
 
-  /* end of vector in short format, now quantitites in normal format */
+  /* end of vector in short format, now quantities in normal format */
 
   int index_bg_rho_g;         /**< photon density */
   int index_bg_rho_b;         /**< baryon density */
@@ -175,7 +174,7 @@ struct background
 
   int index_bg_Omega_r;       /**< relativistic density fraction (\f$ \Omega_{\gamma} + \Omega_{\nu r} \f$) */
 
-  /* end of vector in normal format, now quantitites in long format */
+  /* end of vector in normal format, now quantities in long format */
 
   int index_bg_rho_crit;      /**< critical density */
   int index_bg_Omega_m;       /**< non-relativistic density fraction (\f$ \Omega_b + \Omega_cdm + \Omega_{\nu nr} \f$) */
@@ -185,7 +184,7 @@ struct background
   int index_bg_time;          /**< proper (cosmological) time in Mpc */
   int index_bg_rs;            /**< comoving sound horizon in Mpc */
 
-  int index_bg_D;             /**< density growth factor in dust universe, D = H \int [da/(aH)^3] (arbitrary normalization) */
+  int index_bg_D;             /**< density growth factor in dust universe, \f$ D = H \int [da/(aH)^3] \f$ (arbitrary normalization) */
   int index_bg_f;             /**< velocity growth factor in dust universe, [dlnD]/[dln a] */
 
   int bg_size_short;  /**< size of background vector in the "short format" */
@@ -237,7 +236,7 @@ struct background
   int index_bi_time;    /**< {C} proper (cosmological) time in Mpc */
   int index_bi_rs;      /**< {C} sound horizon */
   int index_bi_tau;     /**< {C} conformal time in Mpc */
-  int index_bi_growth;  /**< {C} integral over [da/(aH)^3]=[dtau/(aH^2)], useful for growth factor */
+  int index_bi_growth;  /**< {C} integral over \f$ [da/(aH)^3]=[d\tau/(aH^2)]\f$, useful for growth factor */
 
   int bi_B_size;        /**< Number of {B} parameters */
   int bi_size;          /**< Number of {B}+{C} parameters */
@@ -273,14 +272,14 @@ struct background
 
   //@{
 
-  double ** q_ncdm_bg;  /* Pointers to vectors of background sampling in q */
-  double ** w_ncdm_bg;  /* Pointers to vectors of corresponding quadrature weights w */
-  double ** q_ncdm;     /* Pointers to vectors of perturbation sampling in q */
-  double ** w_ncdm;     /* Pointers to vectors of corresponding quadrature weights w */
-  double ** dlnf0_dlnq_ncdm; /* Pointers to vectors of logarithmic derivatives of p-s-d */
-  int * q_size_ncdm_bg; /* Size of the q_ncdm_bg arrays */
-  int * q_size_ncdm;    /* Size of the q_ncdm arrays */
-  double * factor_ncdm; /* List of normalization factors for calculating energy density etc.*/
+  double ** q_ncdm_bg;  /**< Pointers to vectors of background sampling in q */
+  double ** w_ncdm_bg;  /**< Pointers to vectors of corresponding quadrature weights w */
+  double ** q_ncdm;     /**< Pointers to vectors of perturbation sampling in q */
+  double ** w_ncdm;     /**< Pointers to vectors of corresponding quadrature weights w */
+  double ** dlnf0_dlnq_ncdm; /**< Pointers to vectors of logarithmic derivatives of p-s-d */
+  int * q_size_ncdm_bg; /**< Size of the q_ncdm_bg arrays */
+  int * q_size_ncdm;    /**< Size of the q_ncdm arrays */
+  double * factor_ncdm; /**< List of normalization factors for calculating energy density etc.*/
 
   //@}
 
@@ -303,6 +302,10 @@ struct background
 
   //@{
 
+  short shooting_failed;  /**< flag is set to true if shooting failed. */
+
+  ErrorMsg shooting_error; /**< Error message from shooting failed. */
+
   short background_verbose; /**< flag regulating the amount of information sent to standard output (none if set to zero) */
 
   ErrorMsg error_message; /**< zone for writing error messages */
@@ -348,7 +351,7 @@ struct background_parameters_for_distributions {
 };
 
 /**************************************************************/
-
+/* @cond INCLUDE_WITH_DOXYGEN */
 /*
  * Boilerplate for C++
  */
@@ -497,18 +500,18 @@ extern "C" {
 
 //@{
 
-#define _Mpc_over_m_ 3.085677581282e22  /**< conversion factor from meters to megaparsecs */
+#define _Mpc_over_m_  (ncm_c_Mpc ()) /**< 3.085677581282e22 conversion factor from meters to megaparsecs */
 /* remark: CAMB uses 3.085678e22: good to know if you want to compare  with high accuracy */
 
-#define _Gyr_over_Mpc_ 3.06601394e2 /**< conversion factor from megaparsecs to gigayears
+#define _Gyr_over_Mpc_ (ncm_c_Glightyear_Mpc ()) /**< 3.06601394e2 conversion factor from megaparsecs to gigayears
 				         (c=1 units, Julian years of 365.25 days) */
-#define _c_ 2.99792458e8            /**< c in m/s */
-#define _G_ 6.67428e-11             /**< Newton constant in m^3/Kg/s^2 */
-#define _eV_ 1.602176487e-19        /**< 1 eV expressed in J */
+#define _c_ (ncm_c_c ())           /**< 2.99792458e8 c in m/s */
+#define _G_ (ncm_c_G ())           /**< 6.67428e-11 Newton constant in m^3/Kg/s^2 */
+#define _eV_ (ncm_c_eV ())         /**< 1.602176487e-19 1 eV expressed in J */
 
 /* parameters entering in Stefan-Boltzmann constant sigma_B */
-#define _k_B_ 1.3806504e-23
-#define _h_P_ 6.62606896e-34
+#define _k_B_ (ncm_c_kb ()) /*1.3806504e-23*/
+#define _h_P_ (ncm_c_h ())  /*6.62606896e-34*/
 /* remark: sigma_B = 2 pi^5 k_B^4 / (15h^3c^2) = 5.670400e-8
                    = Stefan-Boltzmann constant in W/m^2/K^4 = Kg/K^4/s^3 */
 
@@ -537,7 +540,7 @@ extern "C" {
 
 //@{
 
-#define _SCALE_BACK_ 0.1  /**< logsrithmic step used when searching
+#define _SCALE_BACK_ 0.1  /**< logarithmic step used when searching
 			     for an initial scale factor at which ncdm
 			     are still relativistic */
 
@@ -551,3 +554,4 @@ extern "C" {
 
 
 #endif
+/* @endcond */
