@@ -25,7 +25,8 @@ enum reionization_parametrization {
   reio_none, /**< no reionization */
   reio_camb,  /**< reionization parameterized like in CAMB */
   reio_bins_tanh,  /**< binned reionization history with tanh inteprolation between bins */
-  reio_half_tanh  /**< half a tanh, intead of the full tanh */
+  reio_half_tanh,  /**< half a tanh, instead of the full tanh */
+  reio_many_tanh   /**< similar to reio_camb but with more than one tanh */
 };
 
 /**
@@ -41,8 +42,8 @@ enum reionization_z_or_tau {
  * Two useful smooth step functions, for smoothing transitions in recfast.
  */
 
-#define f1(x) (-0.75*x*(x*x/3.-1.)+0.5)  /* goes from 0 to 1 when x goes from -1 to 1 */
-#define f2(x) (x*x*(0.5-x/3.)*6.)        /* goes from 0 to 1 when x goes from  0 to 1 */
+#define f1(x) (-0.75*x*(x*x/3.-1.)+0.5)  /**< goes from 0 to 1 when x goes from -1 to 1 */
+#define f2(x) (x*x*(0.5-x/3.)*6.)        /**< goes from 0 to 1 when x goes from  0 to 1 */
 
 /**
  * All thermodynamics parameters and evolution that other modules need to know.
@@ -56,12 +57,12 @@ enum reionization_z_or_tau {
 struct thermo
 {
   /** @name - input parameters initialized by user in input module
-   *  (all other quantitites are computed in this module, given these parameters
+   *  (all other quantities are computed in this module, given these parameters
    *   and the content of the 'precision' and 'background' structures) */
 
   //@{
 
-  double YHe;  /**< \f$ Y_{He} \f$ : primordial helium fraction */
+  double YHe;  /**< \f$ Y_{He} \f$: primordial helium fraction */
 
   enum recombination_algorithm recombination; /**< recombination code */
 
@@ -75,6 +76,8 @@ struct thermo
 
   short compute_cb2_derivatives; /**< do we want to include in computation derivatives of baryon sound speed? */
 
+  short compute_damping_scale; /**< do we want to compute the simplest analytic approximation to the photon damping (or diffusion) scale? */
+
   /** parameters for reio_camb */
 
   double reionization_width; /**< width of H reionization */
@@ -87,28 +90,38 @@ struct thermo
 
   /** parameters for reio_bins_tanh */
 
-  int binned_reio_num; /**< with how many bins de we want to describe reionization? */
+  int binned_reio_num; /**< with how many bins do we want to describe reionization? */
 
   double * binned_reio_z; /**< central z value for each bin */
 
-  double * binned_reio_xe; /**< imposed x_e(z) value at center of each bin */
+  double * binned_reio_xe; /**< imposed \f$ X_e(z)\f$ value at center of each bin */
 
   double binned_reio_step_sharpness; /**< sharpness of tanh() step interpolating between binned values */
 
+    /** parameters for reio_many_tanh */
+
+  int many_tanh_num; /**< with how many jumps do we want to describe reionization? */
+
+  double * many_tanh_z; /**< central z value for each tanh jump */
+
+  double * many_tanh_xe; /**< imposed \f$ X_e(z)\f$ value at the end of each jump (ie at later times)*/
+
+  double many_tanh_width; /**< sharpness of tanh() steps */
+
   /** parameters for energy injection */
 
   double annihilation; /** parameter describing CDM annihilation (f <sigma*v> / m_cdm, see e.g. 0905.0003) */
 
   short has_on_the_spot; /** flag to specify if we want to use the on-the-spot approximation **/
 
-  double decay; /** parameter descibing CDM decay (f/tau, see e.g. 1109.6322)*/
+  double decay; /** parameter describing CDM decay (f/tau, see e.g. 1109.6322)*/
 
   double annihilation_variation; /** if this parameter is non-zero,
 				     the function F(z)=(f <sigma*v> /
 				     m_cdm)(z) will be a parabola in
 				     log-log scale between zmin and
 				     zmax, with a curvature given by
-				     annihlation_variation (must ne
+				     annihlation_variation (must be
 				     negative), and with a maximum in
 				     zmax; it will be constant outside
 				     this range */
@@ -119,11 +132,11 @@ struct thermo
 			     F(annihilation_z)=annihilation */
 
   double annihilation_zmax; /** if annihilation_variation is non-zero,
-				redhsift above which annihilation rate
+				redshift above which annihilation rate
 				is maximal */
 
   double annihilation_zmin; /** if annihilation_variation is non-zero,
-				redhsift below which annihilation rate
+				redshift below which annihilation rate
 				is constant */
 
   double annihilation_f_halo; /** takes the contribution of DM annihilation in halos into account*/
@@ -146,9 +159,10 @@ struct thermo
   int index_th_ddg;           /**< visibility function second derivative \f$ (d^2 g / d \tau^2) \f$ */
   int index_th_Tb;            /**< baryon temperature \f$ T_b \f$ */
   int index_th_cb2;           /**< squared baryon sound speed \f$ c_b^2 \f$ */
-  int index_th_dcb2;          /**< derivative wrt conformal time of squared baryon sound speed \f$ d [c_b^2] / d \tau \f$ (only computed if some non-mininmal tight-coupling schemes is requested) */
+  int index_th_dcb2;          /**< derivative wrt conformal time of squared baryon sound speed \f$ d [c_b^2] / d \tau \f$ (only computed if some non-minimal tight-coupling schemes is requested) */
   int index_th_ddcb2;         /**< second derivative wrt conformal time of squared baryon sound speed  \f$ d^2 [c_b^2] / d \tau^2 \f$ (only computed if some non0-minimal tight-coupling schemes is requested) */
-  int index_th_rate;          /**< maximum variation rate of \f$ exp^{-\kappa}, g and (d g / d \tau), used for computing integration step in perturbation module */
+  int index_th_rate;          /**< maximum variation rate of \f$ exp^{-\kappa}\f$, g and \f$ (d g / d \tau) \f$, used for computing integration step in perturbation module */
+  int index_th_r_d;           /**< simple analytic approximation to the photon comoving damping scale */
   int th_size;                /**< size of thermodynamics vector */
 
   //@}
@@ -182,6 +196,7 @@ struct thermo
   double ds_rec;  /**< physical sound horizon at recombination */
   double ra_rec;  /**< conformal angular diameter distance to recombination */
   double da_rec;  /**< physical angular diameter distance to recombination */
+  double rd_rec;  /**< comoving photon damping scale at recombination */
   double z_d;     /**< baryon drag redshift */
   double tau_d;   /**< baryon drag time */
   double ds_d;    /**< physical sound horizon at baryon drag */
@@ -203,7 +218,7 @@ struct thermo
 
   //@{
 
-  double tau_ini;
+  double tau_ini; /**< initial conformal time at which thermodynamical variables have been be integrated */
 
   //@}
 
@@ -211,7 +226,7 @@ struct thermo
 
   //@{
 
-  double n_e;
+  double n_e; /**< total number density of electrons today (free or not) */
 
   //@}
 
@@ -255,12 +270,12 @@ struct recombination {
   int index_re_xe;         /**< ionization fraction \f$ x_e \f$ */
   int index_re_Tb;         /**< baryon temperature \f$ T_b \f$ */
   int index_re_cb2;        /**< squared baryon sound speed \f$ c_b^2 \f$ */
-  int index_re_dkappadtau; /**< Thomson scattering rate \f$ d \kappa / d \tau\f$ (units 1/Mpc) */
+  int index_re_dkappadtau; /**< Thomson scattering rate \f$ d \kappa / d \tau \f$ (units 1/Mpc) */
   int re_size;             /**< size of this vector */
 
   //@}
 
-  /** @name - table of the above variables at each redshift, and number of redshits */
+  /** @name - table of the above variables at each redshift, and number of redshifts */
 
   //@{
 
@@ -274,59 +289,59 @@ struct recombination {
 
   //@{
 
-  double CDB;
-  double CR;
-  double CK;
-  double CL;
-  double CT;
-  double fHe;
-  double CDB_He;
-  double CK_He;
-  double CL_He;
-  double fu;
-  double H_frac;
-  double Tnow;
-  double Nnow;
-  double Bfact;
-  double CB1;
-  double CB1_He1;
-  double CB1_He2;
-  double H0;
-  double YHe;
+  double CDB; /**< defined as in RECFAST */
+  double CR;  /**< defined as in RECFAST */
+  double CK;  /**< defined as in RECFAST */
+  double CL;  /**< defined as in RECFAST */
+  double CT;  /**< defined as in RECFAST */
+  double fHe; /**< defined as in RECFAST */
+  double CDB_He; /**< defined as in RECFAST */
+  double CK_He;  /**< defined as in RECFAST */
+  double CL_He;  /**< defined as in RECFAST */
+  double fu; /**< defined as in RECFAST */
+  double H_frac; /**< defined as in RECFAST */
+  double Tnow;   /**< defined as in RECFAST */
+  double Nnow;   /**< defined as in RECFAST */
+  double Bfact;  /**< defined as in RECFAST */
+  double CB1;    /**< defined as in RECFAST */
+  double CB1_He1; /**< defined as in RECFAST */
+  double CB1_He2; /**< defined as in RECFAST */
+  double H0;  /**< defined as in RECFAST */
+  double YHe; /**< defined as in RECFAST */
 
-  /** parameters for energy injection */
+  /* parameters for energy injection */
 
-  double annihilation; /** parameter describing CDM annihilation (f <sigma*v> / m_cdm, see e.g. 0905.0003) */
+  double annihilation; /**< parameter describing CDM annihilation (f <sigma*v> / m_cdm, see e.g. 0905.0003) */
 
-  short has_on_the_spot; /** flag to specify if we want to use the on-the-spot approximation **/
+  short has_on_the_spot; /**< flag to specify if we want to use the on-the-spot approximation **/
 
-  double decay; /** parameter descibing CDM decay (f/tau, see e.g. 1109.6322)*/
+  double decay; /**< parameter describing CDM decay (f/tau, see e.g. 1109.6322)*/
 
-  double annihilation_variation; /** if this parameter is non-zero,
+  double annihilation_variation; /**< if this parameter is non-zero,
 				     the function F(z)=(f <sigma*v> /
 				     m_cdm)(z) will be a parabola in
 				     log-log scale between zmin and
 				     zmax, with a curvature given by
-				     annihlation_variation (must ne
+				     annihlation_variation (must be
 				     negative), and with a maximum in
 				     zmax; it will be constant outside
 				     this range */
 
-  double annihilation_z; /** if annihilation_variation is non-zero,
+  double annihilation_z; /**< if annihilation_variation is non-zero,
 			     this is the value of z at which the
 			     parameter annihilation is defined, i.e.
 			     F(annihilation_z)=annihilation */
 
-  double annihilation_zmax; /** if annihilation_variation is non-zero,
-				redhsift above which annihilation rate
+  double annihilation_zmax; /**< if annihilation_variation is non-zero,
+				redshift above which annihilation rate
 				is maximal */
 
-  double annihilation_zmin; /** if annihilation_variation is non-zero,
-				redhsift below which annihilation rate
+  double annihilation_zmin; /**< if annihilation_variation is non-zero,
+				redshift below which annihilation rate
 				is constant */
 
-  double annihilation_f_halo; /* takes the contribution of DM annihilation in halos into account*/
-  double annihilation_z_halo; /*characteristic redshift for DM annihilation in halos*/
+  double annihilation_f_halo; /**< takes the contribution of DM annihilation in halos into account*/
+  double annihilation_z_halo; /**< characteristic redshift for DM annihilation in halos*/
 
   //@}
 
@@ -356,7 +371,7 @@ struct reionization {
 
   //@}
 
-  /** @name - table of the above variables at each redshift, and number of redshits */
+  /** @name - table of the above variables at each redshift, and number of redshifts */
 
   //@{
 
@@ -365,11 +380,11 @@ struct reionization {
 
   //@}
 
-  /** @name - reionization optical depth infered from reionization history */
+  /** @name - reionization optical depth inferred from reionization history */
 
   //@{
 
-  double reionization_optical_depth;
+  double reionization_optical_depth; /**< reionization optical depth inferred from reionization history */
 
   //@}
 
@@ -377,25 +392,25 @@ struct reionization {
 
   //@{
 
-  /** parameters used by reio_camb */
+  /* parameters used by reio_camb */
 
   int index_reio_redshift;  /**< hydrogen reionization redshift */
   int index_reio_exponent;  /**< an exponent used in the function x_e(z) in the reio_camb scheme */
   int index_reio_width;     /**< a width defining the duration of hydrogen reionization in the reio_camb scheme */
   int index_reio_xe_before; /**< ionization fraction at redshift 'reio_start' */
   int index_reio_xe_after;  /**< ionization fraction after full reionization */
-  int index_helium_fullreio_fraction; /**< helium full reionization fraction infered from primordial helium fraction */
+  int index_helium_fullreio_fraction; /**< helium full reionization fraction inferred from primordial helium fraction */
   int index_helium_fullreio_redshift; /**< helium full reionization redshift */
   int index_helium_fullreio_width;    /**< a width defining the duration of helium full reionization in the reio_camb scheme */
 
-  /** parameters used by reio_bins_tanh */
+  /* parameters used by reio_bins_tanh and reio_many_tanh */
 
-  int reio_num_z;
-  int index_reio_first_z;
-  int index_reio_first_xe;
-  int index_reio_step_sharpness;
+  int reio_num_z; /**< number of reionization jumps */
+  int index_reio_first_z; /**< redshift at which we start to impose reionization function */
+  int index_reio_first_xe; /**< ionization fraction at redshift first_z (inferred from recombination code) */
+  int index_reio_step_sharpness; /**< sharpness of tanh jump */
 
-  /** parameters used by all schemes */
+  /* parameters used by all schemes */
 
   int index_reio_start;     /**< redshift above which hydrogen reionization neglected */
 
@@ -403,8 +418,8 @@ struct reionization {
 
   /** @name - vector of such parameters, and its size */
 
-  double * reionization_parameters;
-  int reio_num_params;
+  double * reionization_parameters; /**< vector containing all reionization parameters necessary to compute xe(z) */
+  int reio_num_params; /**< length of vector reionization_parameters */
 
   //@}
 
@@ -412,7 +427,7 @@ struct reionization {
 
   //@{
 
-  int index_reco_when_reio_start;
+  int index_reco_when_reio_start; /**< index of line in recombination table corresponding to first line of reionization table*/
 
   //@}
 
@@ -435,7 +450,7 @@ struct thermodynamics_parameters_and_workspace {
 };
 
 /**************************************************************/
-
+/* @cond INCLUDE_WITH_DOXYGEN */
 /*
  * Boilerplate for C++
  */
@@ -575,6 +590,12 @@ extern "C" {
                                  double *data
                                  );
 
+  int thermodynamics_tanh(double x,
+                          double center,
+                          double before,
+                          double after,
+                          double width,
+                          double * result);
 
 #ifdef __cplusplus
 }
@@ -654,7 +675,7 @@ extern "C" {
 /**
  * @name Some limits imposed on cosmological parameter values:
  */
-
+/* @endcond */
 //@{
 
 #define _YHE_BIG_ 0.5      /**< maximal \f$ Y_{He} \f$ */
