@@ -2259,6 +2259,56 @@ int array_interpolate_growing_closeby(
   return _SUCCESS_;
 }
 
+/**
+  * interpolate to get y(x), when x and y are two columns of the same array, x is arranged in growing order, and the point x is presumably close to the previous point x from the last call of this function.
+  *
+  * Called by background_at_eta(); background_eta_of_z(); background_solve(); thermodynamics_at_z().
+  */
+int array_interpolate_one_growing_closeby(
+		   double * array,
+		   int n_columns,
+		   int n_lines,
+		   int index_x,   /** from 0 to (n_columns-1) */
+		   double x,
+		   int * last_index,
+           int index_y,
+		   double * result,
+		   ErrorMsg errmsg) {
+
+  int inf,sup;
+  double weight;
+
+  inf = *last_index;
+  sup = *last_index+1;
+
+  while (x < *(array+inf*n_columns+index_x)) {
+    inf--;
+    if (inf < 0) {
+      sprintf(errmsg,"%s(L:%d) : x=%e < x_min=%e",__func__,__LINE__,
+	      x,array[index_x]);
+      return _FAILURE_;
+    }
+  }
+  sup = inf+1;
+  while (x > *(array+sup*n_columns+index_x)) {
+    sup++;
+    if (sup > (n_lines-1)) {
+      sprintf(errmsg,"%s(L:%d) : x=%e > x_max=%e",__func__,__LINE__,
+	      x,array[(n_lines-1)*n_columns+index_x]);
+      return _FAILURE_;
+    }
+  }
+  inf = sup-1;
+
+  *last_index = inf;
+
+  weight=(x-*(array+inf*n_columns+index_x))/(*(array+sup*n_columns+index_x)-*(array+inf*n_columns+index_x));
+
+  *result = *(array+inf*n_columns+index_y) * (1.-weight) + *(array+sup*n_columns+index_y) * weight;
+
+  return _SUCCESS_;
+}
+
  /**
   * interpolate to get y_i(x), when x and y_i are all columns of the same array, x is arranged in growing order, and the point x is presumably very close to the previous point x from the last call of this function.
   *
