@@ -44,6 +44,7 @@
 #include "math/ncm_serialize.h"
 #include "math/ncm_cfg.h"
 
+#ifndef NUMCOSMO_GIR_SCAN
 #include <glib/gstdio.h>
 #include <gsl/gsl_randist.h>
 #include <gsl/gsl_sort.h>
@@ -51,6 +52,7 @@
 #ifdef NUMCOSMO_HAVE_CFITSIO
 #include <fitsio.h>
 #endif /* NUMCOSMO_HAVE_CFITSIO */
+#endif /* NUMCOSMO_GIR_SCAN */
 
 enum
 {
@@ -1349,10 +1351,8 @@ _nc_data_cluster_ncount_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL)
   }
   else
   {
-    NcClusterRedshiftImpl z_impl = nc_cluster_redshift_impl (clusterz);
-    NcClusterMassImpl lnM_impl   = nc_cluster_mass_impl (clusterm);
-    gboolean z_p                 = z_impl & NC_CLUSTER_REDSHIFT_P;
-    gboolean lnM_p               = lnM_impl & NC_CLUSTER_MASS_P;
+    gboolean z_p   = ncm_model_check_impl_opt (NCM_MODEL (clusterz), NC_CLUSTER_REDSHIFT_P);
+    gboolean lnM_p = ncm_model_check_impl_opt (NCM_MODEL (clusterm), NC_CLUSTER_MASS_P);
     
     if (z_p && lnM_p)
     {
