@@ -25,7 +25,7 @@
 /**
  * SECTION:nc_cluster_abundance
  * @title: NcClusterAbundance
- * @short_description: Cluster abundance distribution.
+ * @short_description: Cluster abundance distribution
  *
  * FIXME
  */
@@ -43,7 +43,9 @@
 #include "math/memory_pool.h"
 #include "math/ncm_cfg.h"
 
+#ifndef NUMCOSMO_GIR_SCAN
 #include <gsl/gsl_histogram.h>
+#endif /* NUMCOSMO_GIR_SCAN */
 
 enum
 {
@@ -194,7 +196,7 @@ nc_cluster_abundance_class_init (NcClusterAbundanceClass *klass)
   /**
    * NcClusterAbundance:halo-mass-function:
    *
-   * FIXME
+   * Number  density of halos with mass [M, M + dM].
    */
   g_object_class_install_property (object_class,
                                    PROP_MASS_FUNCTION,
@@ -332,17 +334,17 @@ _nc_cluster_abundance_z_p_lnM_p_d2n_integrand (gdouble lnM, gdouble z, gpointer
  * @cosmo: a #NcHICosmo
  * @clusterz: a #NcClusterRedshift
  * @clusterm: a #NcClusterMass
- * @lnM_obs: logarithm base e of the observed mass
- * @lnM_obs_params: FIXME
- * @z_obs: observed redshift
- * @z_obs_params: FIXME
+ * @lnM_obs: (array) (element-type gdouble): logarithm base e of the observed mass
+ * @lnM_obs_params: (array) (element-type gdouble): FIXME
+ * @z_obs: (array) (element-type gdouble): observed redshift
+ * @z_obs_params: (array) (element-type gdouble): FIXME
  *
- * This function computes /f$ \int_0^\infty dz \int_0^\infty d\ln M \frac{d^2N(\ln M, z)}{dzd\ln M} * P(z^{phot}|z) *
- * P(\ln M^{obs}|\ln M, z) /f$. We studied the convergence of this integral to optimize this function. We verified
- * that it converges to 5 decimal places at the redshift interval /f$ [z^{phot} - 10\sigma^{phot}, z^{phot} +
-   * 10\sigma^{phot}] /f$ and the mass interval /f$ [\ln M^{obs} - 7\sigma_{\ln M}, \ln M^{obs} + 7\sigma_{\ln M}] /f$.
+ * This function computes $ \int_0^\infty dz \int_0^\infty d\ln M \frac{d^2N(\ln M, z)}{dzd\ln M} * P(z^{phot}|z) *
+ * P(\ln M^{obs}|\ln M, z) $. We studied the convergence of this integral to optimize this function. We verified
+ * that it converges to 5 decimal places at the redshift interval $ [z^{phot} - 10\sigma^{phot}, z^{phot} +
+   * 10\sigma^{phot}] $ and the mass interval $ [\ln M^{obs} - 7\sigma_{\ln M}, \ln M^{obs} + 7\sigma_{\ln M}] $.
  *
- * Returns: a gdouble which represents /f$ \frac{d^2N(\ln M^{obs}, z^{phot})}{dzd\lnM} /f$.
+ * Returns: a gdouble which represents $ \frac{d^2N(\ln M^{obs}, z^{phot})}{dzd\lnM} $.
  */
 gdouble
 nc_cluster_abundance_z_p_lnM_p_d2n (NcClusterAbundance *cad, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm, gdouble *lnM_obs, gdouble *lnM_obs_params, gdouble *z_obs, gdouble *z_obs_params)
@@ -389,15 +391,15 @@ _nc_cluster_abundance_z_p_d2n_integrand (gdouble z, gpointer params)
  * @clusterz: a #NcClusterRedshift
  * @clusterm: a #NcClusterMass
  * @lnM: the logarithm base e of the mass (gdouble)
- * @z_obs: observed redshift
- * @z_obs_params: FIXME
+ * @z_obs: (array) (element-type gdouble): observed redshift
+ * @z_obs_params: (array) (element-type gdouble): FIXME
  *
- * This function computes /f$ \int_{z_{phot} - 10\sigma_{phot}}^{z_{phot} + 10\sigma_{phot}} dz \,
- * \frac{d^2N}{dzdlnM} * P(z^{photo}|z) /f$. The integral limits were determined requiring a precision
+ * This function computes $ \int_{z_{phot} - 10\sigma_{phot}}^{z_{phot} + 10\sigma_{phot}} dz \,
+ * \frac{d^2N}{dzdlnM} * P(z^{photo}|z) $. The integral limits were determined requiring a precision
  * to five decimal places.
  *
- * Returns: a gdouble which corresponds to /f$ \int_{z_{phot} - 10\sigma_{phot}}^{z_{phot} + 10\sigma_{phot}} dz \,
- * \frac{d^2N}{dzdlnM} * P(z^{photo}|z) /f$.
+ * Returns: a gdouble which corresponds to $ \int_{z_{phot} - 10\sigma_{phot}}^{z_{phot} + 10\sigma_{phot}} dz \,
+ * \frac{d^2N}{dzdlnM} * P(z^{photo}|z) $.
  */
 gdouble
 nc_cluster_abundance_z_p_d2n (NcClusterAbundance *cad, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm, gdouble lnM, gdouble *z_obs, gdouble *z_obs_params)
@@ -455,16 +457,16 @@ _nc_cluster_abundance_lnM_p_d2n_integrand (gdouble lnM, gpointer params)
  * @cosmo: a #NcHICosmo
  * @clusterz: a #NcClusterRedshift
  * @clusterm: a #NcClusterMass 
- * @lnM_obs: logarithm base e of the observed mass
- * @lnM_obs_params: other information of the observed mass, such as its error
+ * @lnM_obs: (array) (element-type gdouble): logarithm base e of the observed mass
+ * @lnM_obs_params: (array) (element-type gdouble): other information of the observed mass, such as its error
  * @z: redshift
    *
- * This function computes /f$ \int_{\ln M^{obs} - 7\sigma_{\ln M}}^{\ln M^{obs} + 7\sigma_{\ln M}} d\ln M \,
- * \frac{d^2N}{dzdlnM} * P(\ln M^{obs}|\ln M) /f$. The integral limits were determined requiring a precision
+ * This function computes $ \int_{\ln M^{obs} - 7\sigma_{\ln M}}^{\ln M^{obs} + 7\sigma_{\ln M}} d\ln M \,
+ * \frac{d^2N}{dzdlnM} * P(\ln M^{obs}|\ln M) $. The integral limits were determined requiring a precision
  * to five decimal places.
  *
- * Returns: a gdouble which corresponds to /f$ \int_{\ln M^{obs} - 7\sigma_{\ln M}}^{\ln M^{obs} + 7\sigma_{\ln M}} d\ln M \,
- * \frac{d^2N}{dzdlnM} * P(\ln M^{obs}|\ln M) /f$.
+ * Returns: a gdouble which corresponds to $ \int_{\ln M^{obs} - 7\sigma_{\ln M}}^{\ln M^{obs} + 7\sigma_{\ln M}} d\ln M \,
+ * \frac{d^2N}{dzdlnM} * P(\ln M^{obs}|\ln M) $.
  */
 gdouble
 nc_cluster_abundance_lnM_p_d2n (NcClusterAbundance *cad, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm, gdouble *lnM_obs, gdouble *lnM_obs_params, gdouble z)
@@ -523,12 +525,12 @@ if (FALSE)
  * @lnM: true mass
  * @z: true redshift
  *
- * This function computes /f$ \int_{\ln M^{obs} - 7\sigma_{\ln M}}^{\ln M^{obs} + 7\sigma_{\ln M}} d\ln M \,
- * \frac{d^2N}{dzdlnM} * P(\ln M^{obs}|\ln M) /f$. The integral limits were determined requiring a precision
+ * This function computes $ \int_{\ln M^{obs} - 7\sigma_{\ln M}}^{\ln M^{obs} + 7\sigma_{\ln M}} d\ln M \,
+ * \frac{d^2N}{dzdlnM} * P(\ln M^{obs}|\ln M) $. The integral limits were determined requiring a precision
  * to five decimal places.
  *
- * Returns: a gdouble which corresponds to /f$ \int_{\ln M^{obs} - 7\sigma_{\ln M}}^{\ln M^{obs} + 7\sigma_{\ln M}} d\ln M \,
- * \frac{d^2N}{dzdlnM} * P(\ln M^{obs}|\ln M) /f$.
+ * Returns: a gdouble which corresponds to $ \int_{\ln M^{obs} - 7\sigma_{\ln M}}^{\ln M^{obs} + 7\sigma_{\ln M}} d\ln M \,
+ * \frac{d^2N}{dzdlnM} * P(\ln M^{obs}|\ln M) $.
  */
 gdouble
 nc_cluster_abundance_d2n (NcClusterAbundance *cad, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm, gdouble lnM, gdouble z)
@@ -658,10 +660,8 @@ _nc_cluster_abundance_lnM_intp_N (NcClusterAbundance *cad, NcHICosmo *cosmo, NcC
 static void
 _nc_cluster_abundance_funcs (NcClusterAbundance *cad, NcClusterRedshift *clusterz, NcClusterMass *clusterm)
 {
-  NcClusterRedshiftImpl z_impl = nc_cluster_redshift_impl (clusterz);
-  NcClusterMassImpl lnM_impl = nc_cluster_mass_impl (clusterm);
-  gboolean z_intp = z_impl & NC_CLUSTER_REDSHIFT_INTP;
-  gboolean lnM_intp = lnM_impl & NC_CLUSTER_MASS_INTP;
+  gboolean z_intp   = ncm_model_check_impl_opt (NCM_MODEL (clusterz), NC_CLUSTER_REDSHIFT_INTP);
+  gboolean lnM_intp = ncm_model_check_impl_opt (NCM_MODEL (clusterm), NC_CLUSTER_MASS_INTP);
 
   if (z_intp && lnM_intp)
   {
@@ -1261,4 +1261,4 @@ nc_ca_mean_bias_Mobs_denominator (NcClusterAbundance *cad, NcHICosmo *cosmo, gdo
  *
  * FIXME
  *
- */
\ No newline at end of file
+ */
