@@ -108,20 +108,6 @@ nc_cluster_mass_clear (NcClusterMass **clusterm)
   g_clear_object (clusterm);
 }
 
-/**
- * nc_cluster_mass_impl:
- * @clusterm: a #NcClusterMass
- *
- * FIXME
- *
- * Returns: FIXME
- */
-NcClusterMassImpl
-nc_cluster_mass_impl (NcClusterMass *clusterm)
-{
-  return NC_CLUSTER_MASS_GET_CLASS (clusterm)->impl;
-}
-
 /**
  * nc_cluster_mass_obs_len:
  * @clusterm: a #NcClusterMass
@@ -223,8 +209,8 @@ nc_cluster_mass_resample (NcClusterMass *clusterm, NcHICosmo *cosmo, gdouble lnM
  * nc_cluster_mass_p_limits:
  * @clusterm: a #NcClusterMass.
  * @cosmo: a #NcHICosmo.
- * @lnM_obs: observed mass.
- * @lnM_obs_params: observed mass params.
+ * @lnM_obs: (array) (element-type gdouble): observed mass.
+ * @lnM_obs_params: (array) (element-type gdouble): observed mass params.
  * @lnM_lower: (out): pointer to the lower limit of the real mass integration.
  * @lnM_upper: (out): pointer to the upper limit of the real mass integration.
  *
