@@ -43,8 +43,10 @@
 #include "math/memory_pool.h"
 #include "math/ncm_cfg.h"
 
+#ifndef NUMCOSMO_GIR_SCAN
 #include <gsl/gsl_randist.h>
 #include <gsl/gsl_roots.h>
+#endif /* NUMCOSMO_GIR_SCAN */
 
 G_DEFINE_TYPE (NcClusterMassPlCL, nc_cluster_mass_plcl, NC_TYPE_CLUSTER_MASS);
 
@@ -126,9 +128,9 @@ static void _nc_cluster_mass_plcl_n_limits (NcClusterMass *clusterm, NcHICosmo *
 static void
 nc_cluster_mass_plcl_class_init (NcClusterMassPlCLClass *klass)
 {
-  GObjectClass* object_class = G_OBJECT_CLASS (klass);
+  GObjectClass* object_class       = G_OBJECT_CLASS (klass);
   NcClusterMassClass* parent_class = NC_CLUSTER_MASS_CLASS (klass);
-  NcmModelClass *model_class = NCM_MODEL_CLASS (klass);
+  NcmModelClass *model_class       = NCM_MODEL_CLASS (klass);
 
   model_class->set_property = &_nc_cluster_mass_plcl_set_property;
   model_class->get_property = &_nc_cluster_mass_plcl_get_property;
@@ -228,15 +230,15 @@ nc_cluster_mass_plcl_class_init (NcClusterMassPlCLClass *klass)
   /* Check for errors in parameters initialization */
   ncm_model_class_check_params_info (model_class);  
 
-  parent_class->P = &_nc_cluster_mass_plcl_Msz_Ml_M500_p;
-  parent_class->intP = &_nc_cluster_mass_plcl_intp;
-  //parent_class->P_limits = &_nc_cluster_mass_plcl_p_limits;
-  parent_class->N_limits = &_nc_cluster_mass_plcl_n_limits;
-  parent_class->resample = &_nc_cluster_mass_plcl_resample;
-  parent_class->obs_len = &_nc_cluster_mass_plcl_obs_len;
+  parent_class->P              = &_nc_cluster_mass_plcl_Msz_Ml_M500_p;
+  parent_class->intP           = &_nc_cluster_mass_plcl_intp;
+  //parent_class->P_limits       = &_nc_cluster_mass_plcl_p_limits;
+  parent_class->N_limits       = &_nc_cluster_mass_plcl_n_limits;
+  parent_class->resample       = &_nc_cluster_mass_plcl_resample;
+  parent_class->obs_len        = &_nc_cluster_mass_plcl_obs_len;
   parent_class->obs_params_len = &_nc_cluster_mass_plcl_obs_params_len;
 
-  parent_class->impl = NC_CLUSTER_MASS_IMPL_ALL;
+  ncm_model_class_add_impl_flag (model_class, NC_CLUSTER_MASS_IMPL_ALL);
 }
 
 typedef struct _integrand_data
@@ -664,16 +666,19 @@ _peakfinder (const gint *ndim, const gdouble bounds[], gint *n, gdouble x[], voi
   integrand_data *data = (integrand_data *) userdata;
   NcClusterMassPlCL *mszl = data->mszl;
   gsl_multifit_function_fdf f;
-  gint status, info;
+  gint status;
 
   gdouble p0[] = {log (data->mobs[NC_CLUSTER_MASS_PLCL_MPL]/mszl->M0), log (data->mobs[NC_CLUSTER_MASS_PLCL_MCL]/mszl->M0)};
   gdouble lb[] = {bounds[0], bounds[2]};
   gdouble ub[] = {bounds[1], bounds[3]};
 
+#ifdef HAVE_GSL_2_2
+  gint info;
   const gdouble xtol = 1e-11;
   const gdouble gtol = 1e-11;
   const gdouble ftol = 0.0;
-  
+#endif /* HAVE_GSL_2_2 */
+
   p0[0] = GSL_MAX (p0[0], lb[0]);
   p0[1] = GSL_MAX (p0[1], lb[1]);
   p0[0] = GSL_MIN (p0[0], ub[0]);
