@@ -41,7 +41,9 @@
 #include "math/memory_pool.h"
 #include "math/ncm_cfg.h"
 
+#ifndef NUMCOSMO_GIR_SCAN
 #include <gsl/gsl_randist.h>
+#endif /* NUMCOSMO_GIR_SCAN */
 
 G_DEFINE_TYPE (NcClusterMassVanderlinde, nc_cluster_mass_vanderlinde, NC_TYPE_CLUSTER_MASS);
 
@@ -346,9 +348,9 @@ _nc_cluster_mass_vanderlinde_finalize (GObject *object)
 static void
 nc_cluster_mass_vanderlinde_class_init (NcClusterMassVanderlindeClass *klass)
 {
-  GObjectClass* object_class = G_OBJECT_CLASS (klass);
+  GObjectClass* object_class       = G_OBJECT_CLASS (klass);
   NcClusterMassClass* parent_class = NC_CLUSTER_MASS_CLASS (klass);
-  NcmModelClass *model_class = NCM_MODEL_CLASS (klass);
+  NcmModelClass *model_class       = NCM_MODEL_CLASS (klass);
 
   parent_class->P = &_nc_cluster_mass_vanderlinde_significance_m_p;
   parent_class->intP = &_nc_cluster_mass_vanderlinde_intp;
@@ -358,7 +360,7 @@ nc_cluster_mass_vanderlinde_class_init (NcClusterMassVanderlindeClass *klass)
   parent_class->obs_len = &_nc_cluster_mass_vanderlinde_obs_len;
   parent_class->obs_params_len = &_nc_cluster_mass_vanderlinde_obs_params_len;
 
-  parent_class->impl = NC_CLUSTER_MASS_IMPL_ALL;
+  ncm_model_class_add_impl_flag (model_class, NC_CLUSTER_MASS_IMPL_ALL);
 
   object_class->finalize = _nc_cluster_mass_vanderlinde_finalize;
 
