@@ -42,7 +42,9 @@
 #include "math/integral.h"
 #include "math/memory_pool.h"
 
+#ifndef NUMCOSMO_GIR_SCAN
 #include <gsl/gsl_roots.h>
+#endif /* NUMCOSMO_GIR_SCAN */
 
 G_DEFINE_TYPE (NcClusterPseudoCounts, nc_cluster_pseudo_counts, NCM_TYPE_MODEL);
 
@@ -56,7 +58,7 @@ enum
 {
   PROP_0,
   PROP_NCLUSTERS,
-  PROP_SIZE,
+  PROP_SIZE
 };
 
 static void
@@ -209,8 +211,7 @@ nc_cluster_pseudo_counts_class_init (NcClusterPseudoCountsClass *klass)
  * nc_cluster_pseudo_counts_new: 
  * @nclusters: total number of clusters (resample)
  *
- * This function allocates memory for a new #NcClusterPseudoCounts object and sets its properties to the values from
- * the input argument.
+ * This function instantiates a new object of type #NcClusterPseudoCounts.
  *
  * Returns: A new #NcClusterPseudoCounts.
  */
@@ -630,14 +631,17 @@ _peakfinder (gdouble lnM_M0, gdouble p0[], const gint *ndim, const gdouble bound
   NcClusterPseudoCounts *cpc = data->cpc;
   NcClusterMass *clusterm = data->clusterm;
   gsl_multifit_function_fdf f;
-  gint status, info;
+  gint status;
   
   gdouble lb[] = {bounds[0], bounds[2]};
   gdouble ub[] = {bounds[1], bounds[3]};
 
+#ifdef HAVE_GSL_2_2
+  gint info;
   const gdouble xtol = 1e-8;
   const gdouble gtol = 1e-8;
   const gdouble ftol = 0.0;
+#endif /* HAVE_GSL_2_2 */
   
   p0[0] = log (data->Mobs[NC_CLUSTER_MASS_PLCL_MPL]);
   p0[1] = log (data->Mobs[NC_CLUSTER_MASS_PLCL_MCL]);
