@@ -33,6 +33,16 @@
 #include <numcosmo/math/ncm_model_ctrl.h>
 #include <numcosmo/nc_hicosmo.h>
 
+#ifndef NUMCOSMO_GIR_SCAN
+#if HAVE_SUNDIALS_MAJOR == 3
+#include <sundials/sundials_matrix.h>
+#include <sunmatrix/sunmatrix_dense.h>
+#include <sunlinsol/sunlinsol_dense.h>
+#define SUN_DENSE_ACCESS SM_ELEMENT_D
+#endif 
+#include <nvector/nvector_serial.h>
+#endif /* NUMCOSMO_GIR_SCAN */
+
 G_BEGIN_DECLS
 
 #define NC_TYPE_GROWTH_FUNC             (nc_growth_func_get_type ())
@@ -59,6 +69,10 @@ struct _NcGrowthFunc
   gpointer cvode;
   N_Vector yv;
   N_Vector yQ;
+#if HAVE_SUNDIALS_MAJOR == 3
+  SUNMatrix A;
+  SUNLinearSolver LS;
+#endif
   gdouble zf;
   gdouble Da0;
   NcmModelCtrl *ctrl_cosmo;
