@@ -42,8 +42,10 @@
 #include "math/memory_pool.h"
 #include "math/ncm_util.h"
 
+#ifndef NUMCOSMO_GIR_SCAN
 #include <gsl/gsl_integration.h>
 #include <cuba.h>
+#endif /* NUMCOSMO_GIR_SCAN */
 
 static gpointer
 _integral_ws_alloc (gpointer userdata)
@@ -120,7 +122,7 @@ ncm_integral_locked_a_b (gsl_function *F, gdouble a, gdouble b, gdouble abstol,
  * @error: a pointer to a gdouble in which the function stores the estimated error.
  *
  * This function uses a workspace from the pool and gsl_integration_qagiu function to perform
- * the numerical integration in the \f$ [a, \infty] \f$ interval.
+ * the numerical integration in the $[a, \infty]$ interval.
  *
  * Returns: the error code returned by gsl_integration_qagiu.
  */
@@ -128,13 +130,16 @@ gint
 ncm_integral_locked_a_inf (gsl_function *F, gdouble a, gdouble abstol, gdouble reltol, gdouble *result, gdouble *error)
 {
   gsl_integration_workspace **w = ncm_integral_get_workspace ();
-  gint error_code = gsl_integration_qagiu (F, a, abstol, reltol, NCM_INTEGRAL_PARTITION, *w, result, error);
+  gint error_code               = gsl_integration_qagiu (F, a, abstol, reltol, NCM_INTEGRAL_PARTITION, *w, result, error);
+
   ncm_memory_pool_return (w);
+  
   if (error_code != GSL_SUCCESS && error_code != GSL_EROUND)
   {
     g_warning ("ncm_integral_locked_a_inf: %s", gsl_strerror (error_code));
     *result = GSL_POSINF;
   }
+
   return error_code;
 }
 
@@ -188,11 +193,11 @@ ncm_integral_cached_0_x (NcmFunctionCache *cache, gsl_function *F, gdouble x, gd
  * @x: lower integration limit.
  * @result: a pointer to a gdouble in which the function stores the result.
  * @error: a pointer to a gdouble in which the function stores the estimated error.
- *
+ * 
  * This function searchs for the nearest x_near value previously chosed as the lower integration limit
- * and perform the integration at \f$ [x, x_{near}] \f$ interval. This result is summed to that
- * obtained at \f$ [x_{near}, \infty] \f$ and then it is saved in the cache.
- *
+ * and perform the integration at $[x, x_{near}]$ interval. This result is summed to that
+ * obtained at $[x_{near}, \infty]$ and then it is saved in the cache.
+ * 
  * Returns: the error code returned by gsl_integration_qagiu.
  */
 gint
@@ -206,11 +211,14 @@ ncm_integral_cached_x_inf (NcmFunctionCache *cache, gsl_function *F, gdouble x,
   if (ncm_function_cache_get_near (cache, x, &x_found, &p_result, NC_FUNCTION_CACHE_SEARCH_BOTH))
   {
     if (x == x_found)
-      *result = gsl_vector_get(p_result, 0);
+    {
+      *result = gsl_vector_get (p_result, 0);
+    }
     else
     {
       error_code = ncm_integral_locked_a_b (F, x, x_found, cache->abstol, cache->reltol, result, error);
       *result += gsl_vector_get(p_result, 0);
+
       ncm_function_cache_insert (cache, x, *result);
     }
   }
@@ -720,11 +728,7 @@ ncm_integral_fixed_new (gulong n_nodes, gulong rule_n, gdouble xl, gdouble xu)
   intf->xl = xl;
   intf->xu = xu;
 
-#ifdef HAVE_GSL_GLF
   intf->glt = gsl_integration_glfixed_table_alloc (rule_n);
-#else
-  g_error ("NcmIntegralFixed: Needs gsl version > 1.4.");
-#endif /* HAVE_GSL_GLF */
 
   return intf;
 }
@@ -740,11 +744,7 @@ void
 ncm_integral_fixed_free (NcmIntegralFixed *intf)
 {
   g_slice_free1 (sizeof(gdouble) * intf->n_nodes * intf->rule_n, intf->int_nodes);
-#ifdef HAVE_GSL_GLF
   gsl_integration_glfixed_table_free (intf->glt);
-#else
-  g_error ("ncm_integral_fixed_free: Needs gsl version > 1.4.");
-#endif /* HAVE_GSL_GLF */
   g_slice_free (NcmIntegralFixed, intf);
 }
 
@@ -759,7 +759,6 @@ ncm_integral_fixed_free (NcmIntegralFixed *intf)
 void
 ncm_integral_fixed_calc_nodes (NcmIntegralFixed *intf, gsl_function *F)
 {
-#ifdef HAVE_GSL_GLF
   const gulong r2 = intf->rule_n / 2;
   const gboolean odd_rule = intf->rule_n & 1;
   const gdouble delta_x = (intf->xu - intf->xl) / (intf->n_nodes - 1.0);
@@ -794,9 +793,6 @@ ncm_integral_fixed_calc_nodes (NcmIntegralFixed *intf, gsl_function *F)
         intf->int_nodes[k++] = GSL_FN_EVAL (F, x1px0_2 + x1mx0_2 * intf->glt->x[j]) * intf->glt->w[j];
     }
   }
-#else
-  g_error ("ncm_integral_fixed_calc_nodes: Needs gsl version > 1.4.");
-#endif /* HAVE_GSL_GLF */
 }
 
 /**
@@ -832,7 +828,6 @@ ncm_integral_fixed_nodes_eval (NcmIntegralFixed *intf)
 gdouble
 ncm_integral_fixed_integ_mult (NcmIntegralFixed *intf, gsl_function *F)
 {
-#ifdef HAVE_GSL_GLF
   const gulong r2 = intf->rule_n / 2;
   const gboolean odd_rule = intf->rule_n & 1;
   const gdouble delta_x = (intf->xu - intf->xl) / (intf->n_nodes - 1.0);
@@ -870,10 +865,6 @@ ncm_integral_fixed_integ_mult (NcmIntegralFixed *intf, gsl_function *F)
   }
 
   return res * delta_x * 0.5;
-#else
-  g_error ("ncm_integral_fixed_integ_mult: Needs gsl version > 1.4.");
-  return 0.0;
-#endif /* HAVE_GSL_GLF */
 }
 
 /**
@@ -890,7 +881,6 @@ ncm_integral_fixed_integ_mult (NcmIntegralFixed *intf, gsl_function *F)
 gdouble
 ncm_integral_fixed_integ_posdef_mult (NcmIntegralFixed *intf, gsl_function *F, gdouble max, gdouble reltol)
 {
-#ifdef HAVE_GSL_GLF
   const gulong r2 = intf->rule_n / 2;
   const gboolean odd_rule = intf->rule_n & 1;
   const gdouble delta_x = (intf->xu - intf->xl) / (intf->n_nodes - 1.0);
@@ -983,8 +973,4 @@ ncm_integral_fixed_integ_posdef_mult (NcmIntegralFixed *intf, gsl_function *F, g
   }
 
   return res * delta_x * 0.5;
-#else
-  g_error ("ncm_integral_fixed_integ_posdef_mult: Needs gsl version > 1.4.");
-  return 0.0;
-#endif /* HAVE_GSL_GLF */
 }
