@@ -29,7 +29,10 @@
 #include <glib-object.h>
 #include <numcosmo/build_cfg.h>
 #include <numcosmo/math/function_cache.h>
+
+#ifndef NUMCOSMO_GIR_SCAN
 #include <gsl/gsl_integration.h>
+#endif /* NUMCOSMO_GIR_SCAN */
 
 G_BEGIN_DECLS
 
@@ -82,9 +85,9 @@ struct _NcmIntegralFixed
   gdouble *int_nodes;
   gulong n_nodes;
   gulong rule_n;
-#ifdef HAVE_GSL_GLF
+#ifndef NUMCOSMO_GIR_SCAN
   gsl_integration_glfixed_table *glt;
-#endif /* HAVE_GSL_GLF */
+#endif
 };
 
 gint ncm_integral_locked_a_b (gsl_function *F, gdouble a, gdouble b, gdouble abstol, gdouble reltol, gdouble *result, gdouble *error);
@@ -112,7 +115,7 @@ gdouble ncm_integral_fixed_integ_posdef_mult (NcmIntegralFixed *intf, gsl_functi
 #define NCM_INTEGRAL_PARTITION 100000
 #define NCM_INTEGRAL_ALG 6
 #define NCM_INTEGRAL_ERROR 1e-13
-#define NCM_INTEGRAL_ABS_ERROR 1e-13
+#define NCM_INTEGRAL_ABS_ERROR 0.0
 
 G_END_DECLS
 
