@@ -26,12 +26,14 @@
 #ifndef _NCM_BOOTSTRAP_H_
 #define _NCM_BOOTSTRAP_H_
 
-
 #include <glib.h>
 #include <glib-object.h>
 #include <numcosmo/build_cfg.h>
 #include <numcosmo/math/ncm_rng.h>
+
+#ifndef NUMCOSMO_GIR_SCAN
 #include <gsl/gsl_randist.h>
+#endif /* NUMCOSMO_GIR_SCAN */
 
 G_BEGIN_DECLS
 
