@@ -25,8 +25,8 @@
 
 /**
  * SECTION:ncm_data_dist1d
- * @title: One Variable Distribution Data
- * @short_description: Object representing a one variable distribution data
+ * @title: NcmDataDist1d
+ * @short_description: Abstract class for one variable distribution data.
  *
  * FIXME
  * 
@@ -38,12 +38,13 @@
 #include "build_cfg.h"
 
 #include "math/ncm_data_dist1d.h"
-#include "math/ncm_cfg.h"
+#include "math/ncm_rng.h"
 
 enum
 {
   PROP_0,
   PROP_NPOINTS,
+  PROP_VECTOR,
   PROP_SIZE,
 };
 
@@ -74,6 +75,9 @@ _ncm_data_dist1d_set_property (GObject *object, guint prop_id, const GValue *val
     case PROP_NPOINTS:
       ncm_data_dist1d_set_size (dist1d, g_value_get_uint (value));
       break;
+    case PROP_VECTOR:
+      ncm_vector_substitute (&dist1d->x, g_value_get_object (value), TRUE);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -92,6 +96,9 @@ _ncm_data_dist1d_get_property (GObject *object, guint prop_id, GValue *value, GP
     case PROP_NPOINTS:
       g_value_set_uint (value, ncm_data_dist1d_get_size (dist1d));
       break;
+    case PROP_VECTOR:
+      g_value_set_object (value, dist1d->x);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -118,9 +125,10 @@ ncm_data_dist1d_finalize (GObject *object)
 }
 
 static guint _ncm_data_dist1d_get_length (NcmData *data);
-static void _ncm_data_dist1d_copyto (NcmData *data, NcmData *data_dest);
 static void _ncm_data_dist1d_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL);
-static void _ncm_data_dist1d_resample (NcmData *data, NcmMSet *mset);
+static void _ncm_data_dist1d_resample (NcmData *data, NcmMSet *mset, NcmRNG *rng);
+static void _ncm_data_dist1d_set_size (NcmDataDist1d *dist1d, guint np);
+static guint _ncm_data_dist1d_get_size (NcmDataDist1d *dist1d);
 
 static void
 ncm_data_dist1d_class_init (NcmDataDist1dClass *klass)
@@ -142,18 +150,27 @@ ncm_data_dist1d_class_init (NcmDataDist1dClass *klass)
                                                       NULL,
                                                       "Data sample size",
                                                       0, G_MAXUINT, 0,
-                                                      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
 
+  g_object_class_install_property (object_class,
+                                   PROP_VECTOR,
+                                   g_param_spec_object ("vector",
+                                                         NULL,
+                                                         "Data vector",
+                                                         NCM_TYPE_VECTOR,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+
+  data_class->bootstrap  = TRUE;
   data_class->get_length = &_ncm_data_dist1d_get_length;
-  data_class->copyto     = &_ncm_data_dist1d_copyto;
   data_class->begin      = NULL;
 
   data_class->resample   = &_ncm_data_dist1d_resample;
   data_class->m2lnL_val  = &_ncm_data_dist1d_m2lnL_val;
 
-  dist1d_class->dist    = NULL;
-  dist1d_class->inv_pdf = NULL;
-
+  dist1d_class->m2lnL_val = NULL;
+  dist1d_class->inv_pdf   = NULL;
+  dist1d_class->set_size  = &_ncm_data_dist1d_set_size;
+  dist1d_class->get_size  = &_ncm_data_dist1d_get_size;
 }
 
 static guint 
@@ -163,86 +180,123 @@ _ncm_data_dist1d_get_length (NcmData *data)
   return dist1d->np; 
 }
 
-static void
-_ncm_data_dist1d_copyto (NcmData *data, NcmData *data_dest)
-{
-  NcmDataDist1d *dist1d = NCM_DATA_DIST1D (data);
-  NcmDataDist1d *dist1_dest = NCM_DATA_DIST1D (data_dest);
-  ncm_vector_memcpy (dist1_dest->x, dist1d->x);
-  ncm_data_set_init (data_dest);
-}
-
 static void
 _ncm_data_dist1d_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL)
 {
   NcmDataDist1d *dist1d = NCM_DATA_DIST1D (data);
   NcmDataDist1dClass *dist1d_class = NCM_DATA_DIST1D_GET_CLASS (data);
-  gint i;
+  guint i;
   
   *m2lnL = 0.0;
-
-  for (i = 0; i < dist1d->np; i++)
+  if (!ncm_data_bootstrap_enabled (data))
   {
-    const gdouble x_i = ncm_vector_get (dist1d->x, i);
-    *m2lnL += ncm_mset_func_eval1 (dist1d_class->dist, mset, x_i);
+    for (i = 0; i < dist1d->np; i++)
+    {
+      const gdouble x_i = ncm_vector_get (dist1d->x, i);
+      *m2lnL += dist1d_class->m2lnL_val (dist1d, mset, x_i);
+    }
+  }
+  else
+  {
+    const guint bsize = ncm_bootstrap_get_bsize (data->bstrap);
+    for (i = 0; i < bsize; i++)
+    {
+      guint k = ncm_bootstrap_get (data->bstrap, i);
+      const gdouble x_i = ncm_vector_get (dist1d->x, k);
+      *m2lnL += dist1d_class->m2lnL_val (dist1d, mset, x_i);
+    }    
   }
-
   return;
 }
 
 static void
-_ncm_data_dist1d_resample (NcmData *data, NcmMSet *mset)
+_ncm_data_dist1d_resample (NcmData *data, NcmMSet *mset, NcmRNG *rng)
 {
   NcmDataDist1d *dist1d = NCM_DATA_DIST1D (data);
   NcmDataDist1dClass *dist1d_class = NCM_DATA_DIST1D_GET_CLASS (data);
-  gsl_rng *rng = ncm_cfg_rng_get ();
-  gint i;
+  guint i;
 
   if (dist1d_class->inv_pdf == NULL)
     g_error ("_ncm_data_dist1d_resample: This object do not implement the inverse of the pdf.");
 
+  ncm_rng_lock (rng);
   for (i = 0; i < dist1d->np; i++)
   {
-    const gdouble u_i = gsl_rng_uniform (rng);
-    const gdouble x_i = ncm_mset_func_eval1 (dist1d_class->inv_pdf, mset, u_i);
+    const gdouble u_i = gsl_rng_uniform (rng->r);
+    const gdouble x_i = dist1d_class->inv_pdf (dist1d, mset, u_i);
     ncm_vector_set (dist1d->x, i, x_i);
   }
+  ncm_rng_unlock (rng);
+}
+
+static void 
+_ncm_data_dist1d_set_size (NcmDataDist1d *dist1d, guint np)
+{
+  NcmData *data = NCM_DATA (dist1d);
+  if ((np == 0) || (np != dist1d->np))
+  {
+    dist1d->np = 0;
+    ncm_vector_clear (&dist1d->x);
+    data->init = FALSE;
+  }
+  if ((np != 0) && (np != dist1d->np))
+  {
+    dist1d->np = np;
+    dist1d->x  = ncm_vector_new (dist1d->np);
+    if (ncm_data_bootstrap_enabled (data))
+    {
+      ncm_bootstrap_set_fsize (data->bstrap, np);
+      ncm_bootstrap_set_bsize (data->bstrap, np);
+    }
+    data->init = FALSE;
+  }
+}
+
+static guint 
+_ncm_data_dist1d_get_size (NcmDataDist1d *dist1d)
+{
+  return dist1d->np;
 }
 
 /**
- * ncm_data_dist1d_set_size:
+ * ncm_data_dist1d_set_size: (virtual set_size)
  * @dist1d: a #NcmDataDist1d
- * @np: FIXME
+ * @np: data size.
  *
- * FIXME
+ * Sets the data size to @np.
  * 
  */
 void 
 ncm_data_dist1d_set_size (NcmDataDist1d *dist1d, guint np)
 {
-  if ((np == 0) || (np != dist1d->np))
-  {
-    dist1d->np = 0;
-    ncm_vector_clear (&dist1d->x);
-  }
-  if ((np != 0) && (np != dist1d->np))
-  {
-    dist1d->np = np;
-    dist1d->x  = ncm_vector_new (dist1d->np);    
-  }
+  NCM_DATA_DIST1D_GET_CLASS (dist1d)->set_size (dist1d, np);
 }
 
 /**
- * ncm_data_dist1d_get_size:
+ * ncm_data_dist1d_get_size: (virtual get_size)
  * @dist1d: a #NcmDataDist1d
  *
- * FIXME
+ * Gets the data size.
+ * 
+ * Returns: Data size.
  * 
- * Returns: FIXME
  */
 guint 
 ncm_data_dist1d_get_size (NcmDataDist1d *dist1d)
 {
-  return dist1d->np;
+  return NCM_DATA_DIST1D_GET_CLASS (dist1d)->get_size (dist1d);
 }
 
+/**
+ * ncm_data_dist1d_get_data:
+ * @dist1d: a #NcmDataDist1d
+ *
+ * Gets the data #NcmVector.
+ * 
+ * Returns: (transfer full): Data vector. 
+ */
+NcmVector *
+ncm_data_dist1d_get_data (NcmDataDist1d *dist1d)
+{
+  return ncm_vector_ref (dist1d->x);
+}
