@@ -8,17 +8,17 @@
 /*
  * numcosmo
  * Copyright (C) 2012 Sandro Dias Pinto Vitenti <sandro@isoftware.com.br>
- * 
+ *
  * numcosmo is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
- * 
+ *
  * numcosmo is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
@@ -29,22 +29,24 @@
  * @short_description: Gaussian data -- covariance provided.
  *
  * Generic gaussian distribution which uses the covariance matrix as input.
- * 
+ *
  */
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif /* HAVE_CONFIG_H */
 #include "build_cfg.h"
 
-#include "math/ncm_data_gauss_cov.h"
-#include "math/ncm_cfg.h"
 #include "math/ncm_c.h"
+#include "math/ncm_cfg.h"
+#include "math/ncm_data_gauss_cov.h"
 #include "math/ncm_lapack.h"
 
-#include <gsl/gsl_linalg.h>
+#ifndef NUMCOSMO_GIR_SCAN
 #include <gsl/gsl_blas.h>
+#include <gsl/gsl_linalg.h>
 #include <gsl/gsl_randist.h>
+#endif /* NUMCOSMO_GIR_SCAN */
 
 enum
 {
@@ -61,13 +63,13 @@ G_DEFINE_ABSTRACT_TYPE (NcmDataGaussCov, ncm_data_gauss_cov, NCM_TYPE_DATA);
 static void
 ncm_data_gauss_cov_init (NcmDataGaussCov *gauss)
 {
-  gauss->np               = 0;
-  gauss->y                = NULL;
-  gauss->v                = NULL;
-  gauss->cov              = NULL;
-  gauss->LLT              = NULL;
-  gauss->prepared_LLT     = FALSE;
-  gauss->use_norma        = FALSE;
+  gauss->np           = 0;
+  gauss->y            = NULL;
+  gauss->v            = NULL;
+  gauss->cov          = NULL;
+  gauss->LLT          = NULL;
+  gauss->prepared_LLT = FALSE;
+  gauss->use_norma    = FALSE;
 }
 
 static void
@@ -98,7 +100,7 @@ _ncm_data_gauss_cov_set_property (GObject *object, guint prop_id, const GValue *
       NcmVector *v = g_value_get_object (value);
       ncm_vector_substitute (&gauss->y, v, TRUE);
       break;
-      }
+    }
     case PROP_COV:
       ncm_matrix_substitute (&gauss->cov, g_value_get_object (value), TRUE);
       break;
@@ -151,16 +153,19 @@ _ncm_data_gauss_cov_dispose (GObject *object)
 static void
 _ncm_data_gauss_cov_finalize (GObject *object)
 {
-  
+
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_data_gauss_cov_parent_class)->finalize (object);
 }
 
-static guint _ncm_data_gauss_cov_get_length (NcmData *data); 
+static guint _ncm_data_gauss_cov_get_length (NcmData *data);
 /* static void _ncm_data_gauss_cov_begin (NcmData *data); */
 static void _ncm_data_gauss_cov_resample (NcmData *data, NcmMSet *mset, NcmRNG *rng);
 static void _ncm_data_gauss_cov_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL);
 static void _ncm_data_gauss_cov_leastsquares_f (NcmData *data, NcmMSet *mset, NcmVector *v);
+static void _ncm_data_gauss_cov_mean_vector (NcmData *data, NcmMSet *mset, NcmVector *mu);
+static void _ncm_data_gauss_cov_inv_cov_UH (NcmData *data, NcmMSet *mset, NcmMatrix *H);
+
 static void _ncm_data_gauss_cov_set_size (NcmDataGaussCov *gauss, guint np);
 static guint _ncm_data_gauss_cov_get_size (NcmDataGaussCov *gauss);
 static void _ncm_data_gauss_cov_lnNorma2 (NcmDataGaussCov *gauss, NcmMSet *mset, gdouble *m2lnL);
@@ -169,7 +174,7 @@ static void _ncm_data_gauss_cov_lnNorma2_bs (NcmDataGaussCov *gauss, NcmMSet *ms
 static void
 ncm_data_gauss_cov_class_init (NcmDataGaussCovClass *klass)
 {
-  GObjectClass* object_class     = G_OBJECT_CLASS (klass);
+  GObjectClass *object_class     = G_OBJECT_CLASS (klass);
   NcmDataClass *data_class       = NCM_DATA_CLASS (klass);
   NcmDataGaussCovClass *gauss_cov_class = NCM_DATA_GAUSS_COV_CLASS (klass);
 
@@ -210,16 +215,20 @@ ncm_data_gauss_cov_class_init (NcmDataGaussCovClass *klass)
                                                         "Data covariance",
                                                         NCM_TYPE_MATRIX,
                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-  
+
   data_class->bootstrap          = TRUE;
-  
+
   data_class->get_length         = &_ncm_data_gauss_cov_get_length;
+  data_class->get_dof            = NULL;
   data_class->begin              = NULL;
 
   data_class->resample           = &_ncm_data_gauss_cov_resample;
   data_class->m2lnL_val          = &_ncm_data_gauss_cov_m2lnL_val;
   data_class->leastsquares_f     = &_ncm_data_gauss_cov_leastsquares_f;
 
+  data_class->mean_vector        = &_ncm_data_gauss_cov_mean_vector;
+  data_class->inv_cov_UH         = &_ncm_data_gauss_cov_inv_cov_UH;
+
   gauss_cov_class->mean_func    = NULL;
   gauss_cov_class->cov_func     = NULL;
   gauss_cov_class->lnNorma2     = &_ncm_data_gauss_cov_lnNorma2;
@@ -228,10 +237,10 @@ ncm_data_gauss_cov_class_init (NcmDataGaussCovClass *klass)
   gauss_cov_class->get_size     = &_ncm_data_gauss_cov_get_size;
 }
 
-static guint 
-_ncm_data_gauss_cov_get_length (NcmData *data) 
-{ 
-  return NCM_DATA_GAUSS_COV (data)->np; 
+static guint
+_ncm_data_gauss_cov_get_length (NcmData *data)
+{
+  return NCM_DATA_GAUSS_COV (data)->np;
 }
 
 static void
@@ -246,10 +255,14 @@ _ncm_data_gauss_cov_prepare_LLT (NcmData *data)
     ncm_matrix_memcpy (gauss->LLT, gauss->cov);
 
   ret = ncm_matrix_cholesky_decomp (gauss->LLT, 'U');
-  if (ret != 0)
-    g_error ("_ncm_data_gauss_cov_prepare_LLT[ncm_matrix_cholesky_decomp]: %d.", ret);
-  
-  gauss->prepared_LLT = TRUE;
+  if (ret != 0) /* if different from 0, something went wrong in the Cholesky decomposition */
+  {
+    // g_error ("_ncm_data_gauss_cov_prepare_LLT[ncm_matrix_cholesky_decomp]: %d.", ret);
+    g_warning ("_ncm_data_gauss_cov_prepare_LLT[ncm_matrix_cholesky_decomp]: %d.", ret);
+    gauss->prepared_LLT = FALSE;
+  }
+  else
+    gauss->prepared_LLT = TRUE;
 }
 
 static void
@@ -276,27 +289,35 @@ _ncm_data_gauss_cov_resample (NcmData *data, NcmMSet *mset, NcmRNG *rng)
   ncm_rng_unlock (rng);
 
   /* CblasLower, CblasNoTrans => CblasUpper, CblasTrans */
-  ret = gsl_blas_dtrmv (CblasUpper, CblasTrans, CblasNonUnit, 
+  ret = gsl_blas_dtrmv (CblasUpper, CblasTrans, CblasNonUnit,
                         ncm_matrix_gsl (gauss->LLT), ncm_vector_gsl (gauss->v));
   NCM_TEST_GSL_RESULT ("_ncm_data_gauss_cov_resample", ret);
 
-  gauss_cov_class->mean_func (gauss, mset, gauss->y);  
+  gauss_cov_class->mean_func (gauss, mset, gauss->y);
   ncm_vector_sub (gauss->y, gauss->v);
 }
 
-static void 
+static void
 _ncm_data_gauss_cov_lnNorma2 (NcmDataGaussCov *gauss, NcmMSet *mset, gdouble *m2lnL)
 {
-  gint exponent = 0;
+  *m2lnL += gauss->np * ncm_c_ln2pi () + ncm_matrix_cholesky_lndet (gauss->LLT);
+}
+
+static void
+_ncm_data_gauss_cov_lnNorma2_bs (NcmDataGaussCov *gauss, NcmMSet *mset, NcmBootstrap *bstrap, gdouble *m2lnL)
+{
+  const gdouble lb = 1.0e-200;
+  const gdouble ub = 1.0e+200;
+  gdouble detL     = 1.0;
+  glong exponent   = 0;
   guint i;
-  gdouble detL = 1.0;
-  //gdouble lndetL = 0.0;
 
   for (i = 0; i < gauss->np; i++)
   {
-    const gdouble Lii = ncm_matrix_get (gauss->LLT, i, i);
+    const guint k       = ncm_bootstrap_get (bstrap, i);
+    const gdouble Lii   = ncm_matrix_get (gauss->LLT, k, k);
     const gdouble ndetL = detL * Lii;
-    if (ndetL < 1.0e-300 || ndetL > 1.0e300)
+    if (G_UNLIKELY ((ndetL < lb) || (ndetL > ub)))
     {
       gint exponent_i = 0;
       detL = frexp (ndetL, &exponent_i);
@@ -304,25 +325,9 @@ _ncm_data_gauss_cov_lnNorma2 (NcmDataGaussCov *gauss, NcmMSet *mset, gdouble *m2
     }
     else
       detL = ndetL;
-//    lndetL += log (Lii);
   }
-  
+
   *m2lnL += gauss->np * ncm_c_ln2pi () + 2.0 * (log (detL) + exponent * M_LN2);
-  //return gauss->np * ncm_c_ln2pi () + 2.0 * lndetL;
-}
-
-static void 
-_ncm_data_gauss_cov_lnNorma2_bs (NcmDataGaussCov *gauss, NcmMSet *mset, NcmBootstrap *bstrap, gdouble *m2lnL)
-{
-  guint i;
-  gdouble detL = 1.0;
-
-  for (i = 0; i < gauss->np; i++)
-  {
-    guint k = ncm_bootstrap_get (bstrap, i);
-    detL *= ncm_matrix_get (gauss->LLT, k, k);
-  }
-  *m2lnL += gauss->np * ncm_c_ln2pi () + 2.0 * log (detL);  
 }
 
 static void
@@ -338,15 +343,21 @@ _ncm_data_gauss_cov_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL)
   gauss_cov_class->mean_func (gauss, mset, gauss->v);
 
   ncm_vector_sub (gauss->v, gauss->y);
-  
+
   if (gauss_cov_class->cov_func != NULL)
     cov_update = gauss_cov_class->cov_func (gauss, mset, gauss->cov);
 
   if (cov_update || !gauss->prepared_LLT)
     _ncm_data_gauss_cov_prepare_LLT (data);
 
+  if (!gauss->prepared_LLT) /* that means that the Cholesky decomposition has not worked */
+  {
+    *m2lnL = GSL_POSINF;
+    return;
+  }
+
   /* CblasLower, CblasNoTrans => CblasUpper, CblasTrans */
-  ret = gsl_blas_dtrsv (CblasUpper, CblasTrans, CblasNonUnit, 
+  ret = gsl_blas_dtrsv (CblasUpper, CblasTrans, CblasNonUnit,
                         ncm_matrix_gsl (gauss->LLT), ncm_vector_gsl (gauss->v));
   NCM_TEST_GSL_RESULT ("_ncm_data_gauss_cov_m2lnL_val", ret);
 
@@ -371,9 +382,9 @@ _ncm_data_gauss_cov_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL)
     for (i = 0; i < bsize; i++)
     {
       guint k = ncm_bootstrap_get (data->bstrap, i);
-      const gdouble u_i = ncm_vector_get (gauss->v, k); 
+      const gdouble u_i = ncm_vector_get (gauss->v, k);
       *m2lnL += u_i * u_i;
-    }    
+    }
     if (gauss->use_norma)
       gauss_cov_class->lnNorma2_bs (gauss, mset, data->bstrap, m2lnL);
   }
@@ -389,10 +400,10 @@ _ncm_data_gauss_cov_leastsquares_f (NcmData *data, NcmMSet *mset, NcmVector *v)
 
   if (ncm_data_bootstrap_enabled (data))
     g_error ("NcmDataGaussCov: does not support bootstrap with least squares");
-  
+
   gauss_cov_class->mean_func (gauss, mset, v);
   ncm_vector_sub (v, gauss->y);
-  
+
   if (gauss_cov_class->cov_func != NULL)
     cov_update = gauss_cov_class->cov_func (gauss, mset, gauss->cov);
 
@@ -400,12 +411,41 @@ _ncm_data_gauss_cov_leastsquares_f (NcmData *data, NcmMSet *mset, NcmVector *v)
     _ncm_data_gauss_cov_prepare_LLT (data);
 
   /* CblasLower, CblasNoTrans => CblasUpper, CblasTrans */
-  ret = gsl_blas_dtrsv (CblasUpper, CblasTrans, CblasNonUnit, 
+  ret = gsl_blas_dtrsv (CblasUpper, CblasTrans, CblasNonUnit,
                         ncm_matrix_gsl (gauss->LLT), ncm_vector_gsl (v));
   NCM_TEST_GSL_RESULT ("_ncm_data_gauss_cov_leastsquares_f", ret);
 }
 
+static void
+_ncm_data_gauss_cov_mean_vector (NcmData *data, NcmMSet *mset, NcmVector *mu)
+{
+  NcmDataGaussCov *gauss                = NCM_DATA_GAUSS_COV (data);
+  NcmDataGaussCovClass *gauss_cov_class = NCM_DATA_GAUSS_COV_GET_CLASS (gauss);
+
+  gauss_cov_class->mean_func (gauss, mset, mu);
+}
+
 static void 
+_ncm_data_gauss_cov_inv_cov_UH (NcmData *data, NcmMSet *mset, NcmMatrix *H)
+{
+  NcmDataGaussCov *gauss = NCM_DATA_GAUSS_COV (data);
+  NcmDataGaussCovClass *gauss_cov_class = NCM_DATA_GAUSS_COV_GET_CLASS (gauss);
+  gboolean cov_update = FALSE;
+  gint ret;
+
+  if (gauss_cov_class->cov_func != NULL)
+    cov_update = gauss_cov_class->cov_func (gauss, mset, gauss->cov);
+
+  if (cov_update || !gauss->prepared_LLT)
+    _ncm_data_gauss_cov_prepare_LLT (data);
+
+  ret = gsl_blas_dtrsm (CblasRight, CblasUpper, CblasNoTrans, CblasNonUnit, 
+                        1.0, ncm_matrix_gsl (gauss->LLT), ncm_matrix_gsl (H));
+  
+  NCM_TEST_GSL_RESULT ("_ncm_data_gauss_cov_inv_cov_UH", ret);
+}
+
+static void
 _ncm_data_gauss_cov_set_size (NcmDataGaussCov *gauss, guint np)
 {
   NcmData *data = NCM_DATA (gauss);
@@ -433,7 +473,7 @@ _ncm_data_gauss_cov_set_size (NcmDataGaussCov *gauss, guint np)
   }
 }
 
-static guint 
+static guint
 _ncm_data_gauss_cov_get_size (NcmDataGaussCov *gauss)
 {
   return gauss->np;
@@ -445,10 +485,9 @@ _ncm_data_gauss_cov_get_size (NcmDataGaussCov *gauss)
  * @np: data size.
  *
  * Sets the data size to @np.
- * 
+ *
  */
-void 
-ncm_data_gauss_cov_set_size (NcmDataGaussCov *gauss, guint np)
+void ncm_data_gauss_cov_set_size (NcmDataGaussCov *gauss, guint np)
 {
   NCM_DATA_GAUSS_COV_GET_CLASS (gauss)->set_size (gauss, np);
 }
@@ -458,12 +497,11 @@ ncm_data_gauss_cov_set_size (NcmDataGaussCov *gauss, guint np)
  * @gauss: a #NcmDataGaussCov
  *
  * Gets the data size.
- * 
+ *
  * Returns: Data size.
- * 
+ *
  */
-guint 
-ncm_data_gauss_cov_get_size (NcmDataGaussCov *gauss)
+guint ncm_data_gauss_cov_get_size (NcmDataGaussCov *gauss)
 {
   return NCM_DATA_GAUSS_COV_GET_CLASS (gauss)->get_size (gauss);
 }
