@@ -39,16 +39,37 @@
  *
  * $$F(k) = \sum_{n} c_n e^{\frac{2\pi i n}{L} \ln\left(\frac{k}{k_0}\right)}.$$
  * Substituting $F(k)$ in Eq. \eqref{eq:Gr} and changing the variable $k \rightarrow t = kr$, thus 
- * \begin{eqnarray}\label{eq:Gr_decomp}
- * r G(r) &=& \sum_n c_n \int_0^\infty \frac{k}{k_0}^{\frac{2\pi i n}{L}} K(kr)^2 d(kr) \\
- * &=& \sum_n c_n  \int_0^\infty \frac{t}{k_0 r}^{\frac{2\pi i n}{L}} K(t) dt \\
- * &=& \sum_n c_n e^{-\frac{2\pi i n}{L} \ln\left(\frac{r}{r_0}\right)} e^{-\frac{2\pi i n}{L} \ln(k_0 r_0)} Y_n, 
- * \end{eqnarray}
+ * \begin{align}\label{eq:Gr_decomp}
+ * r G(r) &= \sum_n c_n \int_0^\infty \frac{k}{k_0}^{\frac{2\pi i n}{L}} K(kr)^2 d(kr) \\
+ * &= \sum_n c_n  \int_0^\infty \frac{t}{k_0 r}^{\frac{2\pi i n}{L}} K(t) dt \\
+ * &= \sum_n c_n e^{-\frac{2\pi i n}{L} \ln\left(\frac{r}{r_0}\right)} e^{-\frac{2\pi i n}{L} \ln(k_0 r_0)} Y_n, 
+ * \end{align}
  * where 
  * $$Y_n = \int_0^\infty t^{\frac{2\pi i n}{L}} K(t) dt,$$
  * and the Fourier coefficients are
  * $$c_n = \frac{1}{N} \sum_m F(k_m) e^{- \frac{2\pi i nm}{N}}.$$
- * The total number of points $N$ corresponds to the number of knots in the fundamental interval, which is equally spaced.    
+ * The total number of points $N$ corresponds to the number of knots in the fundamental interval, which is equally spaced.
+ *
+ * The variables discretization is different depending whether $N$ is even or odd, in general
+ * $$ k_n = k_0 \mathrm{e}^{n L / N}, \qquad r_m = r_0 \mathrm{e}^{m L / N}. $$
+ * If $N$ is odd, $n$ and $m$ runs from $[-\lfloor N/2\rfloor, \lfloor N/2\rfloor]$, 
+ * where $\lfloor N/2\rfloor$  is the round-down (largest integer smaller than $N/2$) 
+ * of $N/2$. In this case 
+ * \begin{align}
+ * \ln\left(k_{-\lfloor N/2\rfloor}\right) = \ln(k_0) - \frac{(N-1)}{N} \frac{L}{2}, \\
+ * \ln\left(k_{+\lfloor N/2\rfloor}\right) = \ln(k_0) + \frac{(N-1)}{N} \frac{L}{2}.
+ * \end{align}
+ * This means that for odd $N$ the values of $k_n$ (and $r_n$) never touches the borders $\ln(k_0) \pm L/2$
+ * and $\ln(r_0) \pm L/2$. On the other hand if $N$ is even ($\lfloor N/2\rfloor = N/2$) 
+ * \begin{align}
+ * \ln\left(k_{-\lfloor N/2\rfloor}\right) = \ln(k_0) - \frac{L}{2}, \\
+ * \ln\left(k_{+\lfloor N/2\rfloor}\right) = \ln(k_0) + \frac{L}{2}.
+ * \end{align}
+ * However, since we are assuming that these functions are periodic with period $L$ these two 
+ * points refer to the same value of the functions. Thus, we do not need to include both points and in 
+ * the case of even $N$ we include the point $\ln(k_0) - \frac{L}{2}$ only. In the original [FFTLog][XHamilton2000] 
+ * paper they include both points but give them a $1/2$ weight, here we avoid this complication by using the lower 
+ * end point only.
  * 
  * The user must provide the following input values: $\ln k_0$ - ncm_fftlog_set_lnk0(), $\ln r_0$ - ncm_fftlog_set_lnr0(), 
  * $L$ - ncm_fftlog_set_length(), padding percentage - ncm_fftlog_set_padding(), $N$ - ncm_fftlog_set_size(), 
@@ -61,12 +82,12 @@
  * - $N$ knots are equally distributed in the fundamental interval and $N \times \mathrm{padding}$ knots are distributed in 
  * in the two simetric intervals as mentioned above. 
  * - For the sake of optimization, the final number of points $N_f$ is substituted by the smallest number $N_f^\prime$ (bigger than $N_f$) 
- * which can be decomposed as $N_f \leq N_f^\prime = N^\prime (1 + \mathrm{padding}) = 3^a 5^b 7^c$, where $a$, 
- * $b$ and $c$ are positive integers. 
+ * which can be decomposed as $N_f \leq N_f^\prime = N^\prime (1 + \mathrm{padding}) = 2^a 3^b 5^c 7^d$, where $a$, 
+ * $b$, $c$ and $d$ are positive integers. 
  * - The function $F(k)$ can be provided as:
- * 1. a gsl_function - ncm_fftlog_eval_by_function() - whose values are computed at the knots 
+ * 1. a gsl_function - ncm_fftlog_eval_by_gsl_function() - whose values are computed at the knots 
  * within the fundamental interval, and set to zero within the padding intervals. 
- * 2. as a vector - ncm_fftlog_eval_by_vector() - first one must get the vector of $\ln k$ knots, ncm_fftlog_get_vector_lnr(), 
+ * 2. as a vector - ncm_fftlog_eval_by_vector() - first one must get the vector of $\ln k$ knots, ncm_fftlog_get_lnk_vector(), 
  * and then pass a vector containing the values of the function computed at each knot. 
  * 
  * - Regarding $Y_n$, see the different implementations of #NcmFftlog, e.g., #NcmFftlogTophatwin2 and #NcmFftlogGausswin2.
@@ -81,12 +102,15 @@
 
 #include "math/ncm_fftlog.h"
 #include "math/ncm_cfg.h"
+#include "math/ncm_util.h"
 #include "math/ncm_spline_cubic_notaknot.h"
 
-#include <math.h>
+#ifndef NUMCOSMO_GIR_SCAN
 #ifdef NUMCOSMO_HAVE_FFTW3 
 #include <fftw3.h>
 #endif /* NUMCOSMO_HAVE_FFTW3 */
+#endif /* NUMCOSMO_GIR_SCAN */
+
 #ifndef HAVE_FFTW3_ALLOC
 #define fftw_alloc_real(n) (double *) fftw_malloc (sizeof (double) * (n))
 #define fftw_alloc_complex(n) (fftw_complex *) fftw_malloc (sizeof (fftw_complex) * (n))
@@ -101,6 +125,7 @@ enum
   PROP_LR,
   PROP_N,
   PROP_PAD,
+  PROP_NORING,
   PROP_NAME,
 };
 
@@ -120,6 +145,7 @@ ncm_fftlog_init (NcmFftlog *fftlog)
   fftlog->Nf    = 0;
   fftlog->Nf_2  = 0;
   fftlog->pad   = 0;
+  fftlog->noring    = FALSE;
   fftlog->prepared  = FALSE;
   fftlog->evaluated = FALSE;
 
@@ -170,6 +196,9 @@ _ncm_fftlog_set_property (GObject *object, guint prop_id, const GValue *value, G
     case PROP_PAD:
       ncm_fftlog_set_padding (fftlog, g_value_get_double (value));
       break;
+    case PROP_NORING:
+      ncm_fftlog_set_noring (fftlog, g_value_get_boolean (value));
+      break;
     case PROP_NAME:
       g_assert_not_reached ();
       break;
@@ -206,6 +235,9 @@ _ncm_fftlog_get_property (GObject *object, guint prop_id, GValue *value, GParamS
     case PROP_PAD:
       g_value_set_double (value, ncm_fftlog_get_padding (fftlog));
       break;
+    case PROP_NORING:
+      g_value_set_boolean (value, ncm_fftlog_get_noring (fftlog));
+      break;
     case PROP_NAME:
       g_value_set_string (value, NCM_FFTLOG_GET_CLASS (fftlog)->name);
       break;
@@ -299,10 +331,17 @@ ncm_fftlog_class_init (NcmFftlogClass *klass)
   g_object_class_install_property (object_class,
                                    PROP_PAD,
                                    g_param_spec_double ("padding",
-                                                      NULL,
-                                                      "Padding percentage",
-                                                      0, G_MAXDOUBLE, 1.0,
-                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                                        NULL,
+                                                        "Padding percentage",
+                                                        0.0, G_MAXDOUBLE, 1.0,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_NORING,
+                                   g_param_spec_boolean ("no-ringing",
+                                                         NULL,
+                                                         "No ringing",
+                                                         TRUE,
+                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_NAME,
                                    g_param_spec_string ("name",
@@ -492,43 +531,6 @@ ncm_fftlog_get_lnk0 (NcmFftlog *fftlog)
   return fftlog->lnk0;
 }
 
-static gulong
-_ncm_fftlog_fact_size (gulong n)
-{
-  if (n == 1)
-    return 0;
-  else
-  {
-    gulong r3 = n % 3;
-    gulong r5 = n % 5;
-    gulong r7 = n % 7;
-    gulong m = 1;
-
-    if (r3 == 0)
-      m *= 3;
-    if (r5 == 0)
-      m *= 5;
-    if (r7 == 0)
-      m *= 7;
-
-    if (m != 1)
-    {
-      if (n / m == 1)
-        return m;
-      else
-        return m * _ncm_fftlog_fact_size (n / m);
-    }
-    else
-    {
-      const gulong d3 = 3 - r3;
-      const gulong d5 = 5 - r5;
-      const gulong d7 = 7 - r7;
-
-      return _ncm_fftlog_fact_size (n + GSL_MIN (d3, GSL_MIN (d5, d7)));
-    }
-  }
-}
-
 /**
  * ncm_fftlog_set_size:
  * @fftlog: a #NcmFftlog
@@ -546,7 +548,7 @@ ncm_fftlog_set_size (NcmFftlog *fftlog, guint n)
 
   fftlog->Nr = n;
 
-  nt = _ncm_fftlog_fact_size (nt);
+  nt = ncm_util_fact_size (nt);
   fftlog->pad = nt * fftlog->pad_p * 0.5 / (1.0 + fftlog->pad_p);
   n = nt - 2 * fftlog->pad;
   
@@ -555,8 +557,8 @@ ncm_fftlog_set_size (NcmFftlog *fftlog, guint n)
     guint i;
     
     fftlog->N    = n;
-    fftlog->Lk_N = fftlog->Lk / (1.0 * fftlog->N - 1.0);
-    fftlog->N_2  = (fftlog->N - 1) / 2;
+    fftlog->N_2  = fftlog->N / 2;
+    fftlog->Lk_N = fftlog->Lk / (1.0 * fftlog->N);
 
     fftlog->Nf   = fftlog->N + 2 * fftlog->pad;
     fftlog->Nf_2 = fftlog->N_2 + fftlog->pad;
@@ -635,6 +637,38 @@ ncm_fftlog_get_padding (NcmFftlog *fftlog)
   return fftlog->pad_p;
 }
 
+/**
+ * ncm_fftlog_set_noring:
+ * @fftlog: a #NcmFftlog
+ * @active: whether to use the no-ringing adjustment of $\ln(r_0)$
+ * 
+ * Sets whether to use the no-ringing adjustment of $\ln(r_0)$.
+ * 
+ */
+void 
+ncm_fftlog_set_noring (NcmFftlog *fftlog, gboolean active)
+{
+  if ((!fftlog->noring && active) || (fftlog->noring && !active))
+  {
+    fftlog->noring    = active;
+    fftlog->prepared  = FALSE;
+    fftlog->evaluated = FALSE;
+  }
+}
+
+/**
+ * ncm_fftlog_get_noring:
+ * @fftlog: a #NcmFftlog
+ * 
+ * 
+ * Returns: whether no-ringing condition is activated.
+ */
+gboolean 
+ncm_fftlog_get_noring (NcmFftlog *fftlog)
+{
+  return fftlog->noring;
+}
+
 /**
  * ncm_fftlog_set_length:
  * @fftlog: a #NcmFftlog
@@ -649,7 +683,7 @@ ncm_fftlog_set_length (NcmFftlog *fftlog, gdouble Lk)
   if (fftlog->Lk != Lk)
   {
     fftlog->Lk        = Lk;
-    fftlog->Lk_N      = fftlog->Lk / (1.0 * fftlog->N - 1.0);
+    fftlog->Lk_N      = fftlog->Lk / (1.0 * fftlog->N);
     fftlog->prepared  = FALSE;
     fftlog->evaluated = FALSE;
   }
@@ -666,18 +700,36 @@ _ncm_fftlog_eval (NcmFftlog *fftlog)
   
   if (!fftlog->prepared)
   {
-    const gdouble twopi_Lt = 2.0 * M_PI / ncm_fftlog_get_full_length (fftlog);
+    const gdouble Lt       = ncm_fftlog_get_full_length (fftlog);
+    const gdouble twopi_Lt = 2.0 * M_PI / Lt;
     fftw_complex *Ym_0     = g_ptr_array_index (fftlog->Ym, 0);
+    gdouble lnr0k0         = fftlog->lnk0 + fftlog->lnr0;
     fftw_complex *Ym_ndm1;
     
     NCM_FFTLOG_GET_CLASS (fftlog)->get_Ym (fftlog, Ym_0);
 
+    if (fftlog->noring)
+    {
+      gint i;
+      for (i = 0; i < 5; i++)
+      {
+      fftw_complex YNf_2_0 = Ym_0[fftlog->Nf / 2];
+      const gdouble theta  = carg (YNf_2_0);
+      const gdouble M      = (fftlog->Nf / Lt) * lnr0k0 - theta / M_PI;
+      const glong M_round  = M;
+      const gdouble dM     = M - M_round;
+
+      lnr0k0       -= (Lt / fftlog->Nf) * dM;
+      fftlog->lnr0 -= (Lt / fftlog->Nf) * dM;
+      }
+    }
+
     for (i = 0; i < fftlog->Nf; i++)
     {
       const gint phys_i      = ncm_fftlog_get_mode_index (fftlog, i);
       const complex double a = twopi_Lt * phys_i * I;
       
-      Ym_0[i] *= cexp (- a * (fftlog->lnk0 + fftlog->lnr0));
+      Ym_0[i] *= cexp (- a * lnr0k0);
 
       Ym_ndm1 = Ym_0;
       for (nd = 1; nd <= fftlog->nderivs; nd++)
@@ -687,6 +739,18 @@ _ncm_fftlog_eval (NcmFftlog *fftlog)
         Ym_ndm1  = Ym_nd;
       }
     }
+
+    if ((fftlog->Nf % 2) == 0)
+    {
+      const gint Nf_2_index = ncm_fftlog_get_array_index (fftlog, +fftlog->Nf / 2);
+      
+      for (nd = 0; nd <= fftlog->nderivs; nd++)
+      {
+        fftw_complex *Ym_nd = g_ptr_array_index (fftlog->Ym, nd);
+        Ym_nd[Nf_2_index] = creal (Ym_nd[Nf_2_index]);
+      }
+    }
+    
     fftlog->prepared = TRUE;
   }
 
@@ -735,6 +799,29 @@ _ncm_fftlog_eval (NcmFftlog *fftlog)
 }
 #endif /* NUMCOSMO_HAVE_FFTW3 */
 
+/**
+ * ncm_fftlog_get_lnk_vector:
+ * @fftlog: a #NcmFftlog
+ * @lnk: a #NcmVector
+ * 
+ * Computes the $\ln k$ vector.
+ * 
+ */
+void 
+ncm_fftlog_get_lnk_vector (NcmFftlog *fftlog, NcmVector *lnk)
+{
+  gint i;
+  g_assert_cmpuint (fftlog->N, ==, ncm_vector_len (lnk));
+  
+  for (i = 0; i < fftlog->N; i++)
+  {
+    const gint phys_i   = i - fftlog->N_2;
+    const gdouble lnk_i = fftlog->lnk0 + fftlog->Lk_N * phys_i;
+
+    ncm_vector_set (lnk, i, lnk_i);
+  }
+}
+
 /**
  * ncm_fftlog_eval_by_vector:
  * @fftlog: a #NcmFftlog
@@ -749,6 +836,8 @@ ncm_fftlog_eval_by_vector (NcmFftlog *fftlog, NcmVector *Fk)
 #ifdef NUMCOSMO_HAVE_FFTW3
   gint i;
 
+  g_assert_cmpuint (fftlog->N, ==, ncm_vector_len (Fk));
+
   memset (fftlog->Fk, 0, sizeof (complex double) * fftlog->Nf);
   
   for (i = 0; i < fftlog->N; i++)
@@ -761,7 +850,7 @@ ncm_fftlog_eval_by_vector (NcmFftlog *fftlog, NcmVector *Fk)
 }
 
 /**
- * ncm_fftlog_eval_by_function: (skip)
+ * ncm_fftlog_eval_by_gsl_function: (skip)
  * @fftlog: a #NcmFftlog
  * @Fk: Fk function pointer
  * 
@@ -769,7 +858,7 @@ ncm_fftlog_eval_by_vector (NcmFftlog *fftlog, NcmVector *Fk)
  * 
  */
 void 
-ncm_fftlog_eval_by_function (NcmFftlog *fftlog, gsl_function *Fk)
+ncm_fftlog_eval_by_gsl_function (NcmFftlog *fftlog, gsl_function *Fk)
 {
 #ifdef NUMCOSMO_HAVE_FFTW3
   gint i;
@@ -790,6 +879,26 @@ ncm_fftlog_eval_by_function (NcmFftlog *fftlog, gsl_function *Fk)
 #endif /* NUMCOSMO_HAVE_FFTW3 */
 }
 
+/**
+ * ncm_fftlog_eval_by_function:
+ * @fftlog: a #NcmFftlog
+ * @Fk: (scope call): a #NcmFftlogFunc
+ * @user_data: @Fk user data
+ * 
+ * Evaluates the function @Fk at each knot $\ln k_m$.
+ * 
+ */
+void 
+ncm_fftlog_eval_by_function (NcmFftlog *fftlog, NcmFftlogFunc Fk, gpointer user_data)
+{
+  gsl_function F;
+
+  F.function = Fk;
+  F.params   = user_data;
+
+  ncm_fftlog_eval_by_gsl_function (fftlog, &F);
+}
+
 /**
  * ncm_fftlog_prepare_splines:
  * @fftlog: a #NcmFftlog
@@ -820,7 +929,9 @@ ncm_fftlog_prepare_splines (NcmFftlog *fftlog)
 NcmVector *
 ncm_fftlog_get_vector_lnr (NcmFftlog *fftlog)
 {
-  return ncm_vector_ref (fftlog->lnr_vec);
+  if (!fftlog->prepared)
+    g_warning ("ncm_fftlog_get_vector_lnr: returning the lnr vector without preparing evaluating, the vector may change after evaluation.");
+  return (fftlog->lnr_vec != NULL) ? ncm_vector_ref (fftlog->lnr_vec) : NULL;
 }
 
 /**
@@ -876,7 +987,7 @@ ncm_fftlog_eval_output (NcmFftlog *fftlog, guint nderiv, const gdouble lnr)
 }
 
 /**
- * ncm_fftlog_calibrate_size: (skip)
+ * ncm_fftlog_calibrate_size_gsl: (skip)
  * @fftlog: a #NcmFftlog
  * @Fk: Fk function pointer
  * @reltol: relative tolerance
@@ -886,13 +997,13 @@ ncm_fftlog_eval_output (NcmFftlog *fftlog, guint nderiv, const gdouble lnr)
  * 
  */
 void 
-ncm_fftlog_calibrate_size (NcmFftlog *fftlog, gsl_function *Fk, gdouble reltol)
+ncm_fftlog_calibrate_size_gsl (NcmFftlog *fftlog, gsl_function *Fk, const gdouble reltol)
 {
   NcmSpline **s = g_new0 (NcmSpline *, fftlog->nderivs + 1);
   guint nd, size;
   gdouble lreltol = 0.0;
 
-  ncm_fftlog_eval_by_function (fftlog, Fk);
+  ncm_fftlog_eval_by_gsl_function (fftlog, Fk);
   ncm_fftlog_prepare_splines (fftlog);
 
   for (nd = 0; nd <= fftlog->nderivs; nd++)
@@ -903,7 +1014,7 @@ ncm_fftlog_calibrate_size (NcmFftlog *fftlog, gsl_function *Fk, gdouble reltol)
   /*printf ("# Initial size %u [%u].\n", fftlog->N, fftlog->pad);*/
   ncm_fftlog_set_size (fftlog, fftlog->N * 1.2);
   /*printf ("# Trying size %u [%u].\n", fftlog->N, fftlog->pad);*/
-  ncm_fftlog_eval_by_function (fftlog, Fk);
+  ncm_fftlog_eval_by_gsl_function (fftlog, Fk);
   ncm_fftlog_prepare_splines (fftlog);
 
   size = ncm_spline_get_len (g_ptr_array_index (fftlog->Gr_s, 0));
@@ -934,7 +1045,29 @@ ncm_fftlog_calibrate_size (NcmFftlog *fftlog, gsl_function *Fk, gdouble reltol)
   g_clear_pointer (&s, g_free);
 
   if (lreltol > reltol)
-    ncm_fftlog_calibrate_size (fftlog, Fk, reltol);
+    ncm_fftlog_calibrate_size_gsl (fftlog, Fk, reltol);
+}
+
+/**
+ * ncm_fftlog_calibrate_size:
+ * @fftlog: a #NcmFftlog
+ * @Fk: (scope call): a #NcmFftlogFunc
+ * @user_data: @Fk user data
+ * @reltol: relative tolerance
+ * 
+ * Increases the original (input) number of knots until the $G(r)$ splines reach 
+ * the required precision @reltol.  
+ * 
+ */
+void 
+ncm_fftlog_calibrate_size (NcmFftlog *fftlog, NcmFftlogFunc Fk, gpointer user_data, const gdouble reltol)
+{
+  gsl_function F;
+
+  F.function = Fk;
+  F.params   = user_data;
+
+  ncm_fftlog_calibrate_size_gsl (fftlog, &F, reltol);
 }
 
 /**
@@ -985,10 +1118,20 @@ ncm_fftlog_calibrate_size (NcmFftlog *fftlog, gsl_function *Fk, gdouble reltol)
  * @i: index
  * 
  * Gets the index of the mode @i of the Fourier decomposition. This index corresponds 
- * to the lable $n$ in Eq. \eqref{eq:Gr_decomp}.
+ * to the label $n$ in Eq. \eqref{eq:Gr_decomp}.
  * 
  * Returns: the index of the mode
  */
+/**
+ * ncm_fftlog_get_array_index:
+ * @fftlog: a #NcmFftlog
+ * @phys_i: index
+ * 
+ * Gets the array index @i of the Fourier decomposition. This index corresponds the position
+ * in the fft array of the element $n$ in Eq. \eqref{eq:Gr_decomp}.
+ * 
+ * Returns: the array index corresponding to @phys_i 
+ */
 /**
  * ncm_fftlog_peek_output_vector:
  * @fftlog: a #NcmFftlog
