@@ -49,6 +49,7 @@
 #include "math/ncm_fit_nlopt.h"
 #include "ncm_enum_types.h"
 
+#ifndef NUMCOSMO_GIR_SCAN
 #include <gsl/gsl_blas.h>
 #include <gsl/gsl_rng.h>
 #include <gsl/gsl_cdf.h>
@@ -57,6 +58,7 @@
 #include <gsl/gsl_machine.h>
 #include <gsl/gsl_statistics_double.h>
 #include <gsl/gsl_sort_vector.h>
+#endif /* NUMCOSMO_GIR_SCAN */
 
 enum
 {
@@ -78,55 +80,6 @@ enum
 G_DEFINE_ABSTRACT_TYPE (NcmFit, ncm_fit, G_TYPE_OBJECT);
 G_DEFINE_BOXED_TYPE (NcmFitConstraint, ncm_fit_constraint, (GBoxedCopyFunc)&ncm_fit_constraint_dup, (GBoxedFreeFunc)&ncm_fit_constraint_free);
 
-/**
- * ncm_fit_constraint_new:
- * @fit: FIXME
- * @func: FIXME
- * @tot: FIXME
- *
- * FIXME
- *
- * Returns: (transfer full): FIXME
- */
-NcmFitConstraint *
-ncm_fit_constraint_new (NcmFit *fit, NcmMSetFunc *func, gdouble tot)
-{
-  NcmFitConstraint *fitc = g_new (NcmFitConstraint, 1);
-  g_assert (ncm_mset_func_is_scalar (func) && ncm_mset_func_is_const (func));
-  fitc->fit = ncm_fit_ref (fit);
-  fitc->func = ncm_mset_func_ref (func);
-  fitc->tot = tot;
-  return fitc;
-}
-
-/**
- * ncm_fit_constraint_dup:
- * @fitc: FIXME
- *
- * FIXME
- *
- * Returns: (transfer full): FIXME
- */
-NcmFitConstraint *
-ncm_fit_constraint_dup (NcmFitConstraint *fitc)
-{
-  return ncm_fit_constraint_new (fitc->fit, fitc->func, fitc->tot);
-}
-
-/**
- * ncm_fit_constraint_free:
- * @fitc: FIXME
- *
- * FIXME
- *
- */
-void
-ncm_fit_constraint_free (NcmFitConstraint *fitc)
-{
-  ncm_mset_func_free (fitc->func);
-  g_free (fitc);
-}
-
 static void
 ncm_fit_init (NcmFit *fit)
 {
@@ -144,49 +97,7 @@ ncm_fit_init (NcmFit *fit)
   g_ptr_array_set_free_func (fit->inequality_constraints, (GDestroyNotify) &ncm_fit_constraint_free);
 
   fit->sub_fit = NULL;
-}
-
-static void
-_ncm_fit_constructed (GObject *object)
-{
-  /* Chain up : start */
-  G_OBJECT_CLASS (ncm_fit_parent_class)->constructed (object);
-  {
-    NcmFit *fit = NCM_FIT (object);
-    gint n = ncm_dataset_get_n (fit->lh->dset);
-    gint n_priors = ncm_likelihood_priors_length_f (fit->lh) +
-      ncm_likelihood_priors_length_m2lnL (fit->lh);
-    gint data_dof = ncm_dataset_get_dof (fit->lh->dset);
-
-    g_assert (ncm_dataset_all_init (fit->lh->dset));
-
-    if (!fit->mset->valid_map)
-      ncm_mset_prepare_fparam_map (fit->mset);
-
-    /*
-     * It is no longer an error to fit 0 parameters, it just sets the value
-     * of m2lnL in the fit object.
-     *
-     * if (ncm_mset_fparam_len (fit->mset) == 0)
-     * g_warning ("ncm_fit_new: mset object has 0 free parameters");
-     *
-     */
-
-    {
-      guint data_len   = n + n_priors;
-      guint fparam_len = ncm_mset_fparam_len (fit->mset);
-      gint dof         = data_dof + n_priors - fparam_len;
-
-      if (fit->fstate == NULL)
-        fit->fstate = ncm_fit_state_new (data_len, fparam_len, dof,
-                                         NCM_FIT_GET_CLASS (fit)->is_least_squares);
-      else
-        ncm_fit_state_set_all (fit->fstate, data_len, fparam_len, dof,
-                               NCM_FIT_GET_CLASS (fit)->is_least_squares);
-
-      g_assert (data_len > 0);
-    }
-  }
+  fit->diff    = ncm_diff_new ();
 }
 
 static void
@@ -302,6 +213,49 @@ _ncm_fit_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec
   }
 }
 
+static void
+_ncm_fit_constructed (GObject *object)
+{
+  /* Chain up : start */
+  G_OBJECT_CLASS (ncm_fit_parent_class)->constructed (object);
+  {
+    NcmFit *fit = NCM_FIT (object);
+    gint n = ncm_dataset_get_n (fit->lh->dset);
+    gint n_priors = ncm_likelihood_priors_length_f (fit->lh) +
+      ncm_likelihood_priors_length_m2lnL (fit->lh);
+    gint data_dof = ncm_dataset_get_dof (fit->lh->dset);
+
+    g_assert (ncm_dataset_all_init (fit->lh->dset));
+
+    if (!fit->mset->valid_map)
+      ncm_mset_prepare_fparam_map (fit->mset);
+
+    /*
+     * It is no longer an error to fit 0 parameters, it just sets the value
+     * of m2lnL in the fit object.
+     *
+     * if (ncm_mset_fparam_len (fit->mset) == 0)
+     * g_warning ("ncm_fit_new: mset object has 0 free parameters");
+     *
+     */
+
+    {
+      guint data_len   = n + n_priors;
+      guint fparam_len = ncm_mset_fparam_len (fit->mset);
+      gint dof         = data_dof + n_priors - fparam_len;
+
+      if (fit->fstate == NULL)
+        fit->fstate = ncm_fit_state_new (data_len, fparam_len, dof,
+                                         NCM_FIT_GET_CLASS (fit)->is_least_squares);
+      else
+        ncm_fit_state_set_all (fit->fstate, data_len, fparam_len, dof,
+                               NCM_FIT_GET_CLASS (fit)->is_least_squares);
+
+      g_assert (data_len > 0);
+    }
+  }
+}
+
 static void
 ncm_fit_dispose (GObject *object)
 {
@@ -316,6 +270,8 @@ ncm_fit_dispose (GObject *object)
 
   ncm_fit_clear (&fit->sub_fit);
 
+  ncm_diff_clear (&fit->diff);
+
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_fit_parent_class)->dispose (object);
 }
@@ -450,6 +406,55 @@ _ncm_fit_reset (NcmFit *fit)
   }
 }
 
+/**
+ * ncm_fit_constraint_new:
+ * @fit: a #NcmFit
+ * @func: a #NcmMSetFunc
+ * @tot: FIXME
+ *
+ * FIXME
+ *
+ * Returns: (transfer full): FIXME
+ */
+NcmFitConstraint *
+ncm_fit_constraint_new (NcmFit *fit, NcmMSetFunc *func, gdouble tot)
+{
+  NcmFitConstraint *fitc = g_new (NcmFitConstraint, 1);
+  g_assert (ncm_mset_func_is_scalar (func) && ncm_mset_func_is_const (func));
+  fitc->fit = ncm_fit_ref (fit);
+  fitc->func = ncm_mset_func_ref (func);
+  fitc->tot = tot;
+  return fitc;
+}
+
+/**
+ * ncm_fit_constraint_dup:
+ * @fitc: a #NcmFitConstraint
+ *
+ * FIXME
+ *
+ * Returns: (transfer full): FIXME
+ */
+NcmFitConstraint *
+ncm_fit_constraint_dup (NcmFitConstraint *fitc)
+{
+  return ncm_fit_constraint_new (fitc->fit, fitc->func, fitc->tot);
+}
+
+/**
+ * ncm_fit_constraint_free:
+ * @fitc: a #NcmFitConstraint
+ *
+ * FIXME
+ *
+ */
+void
+ncm_fit_constraint_free (NcmFitConstraint *fitc)
+{
+  ncm_mset_func_free (fitc->func);
+  g_free (fitc);
+}
+
 /**
  * ncm_fit_new:
  * @ftype: a #NcmFitType
@@ -492,7 +497,7 @@ ncm_fit_new (NcmFitType ftype, gchar *algo_name, NcmLikelihood *lh, NcmMSet *mse
 
 /**
  * ncm_fit_ref:
- * @fit: a #NcmFit.
+ * @fit: a #NcmFit
  *
  * Increases the reference count of @fit.
  *
@@ -538,9 +543,10 @@ ncm_fit_dup (NcmFit *fit, NcmSerialize *ser)
 
 /**
  * ncm_fit_free:
- * @fit: a #NcmFit.
+ * @fit: a #NcmFit
  *
- * FIXME
+ * Atomically decrements the reference count of @fit by one. If the reference count drops to 0,
+ * all memory allocated by @fit is released.
  *
  */
 void
@@ -551,9 +557,9 @@ ncm_fit_free (NcmFit *fit)
 
 /**
  * ncm_fit_clear:
- * @fit: a #NcmFit.
+ * @fit: a #NcmFit
  *
- * FIXME
+ * The reference count of @fit is decreased and the pointer is set to NULL.
  *
  */
 void
@@ -644,16 +650,16 @@ static NcmFitGrad _ncm_fit_grad_numdiff_central = {
 static NcmFitGrad _ncm_fit_grad_numdiff_accurate = {
   NCM_FIT_GRAD_NUMDIFF_ACCURATE,
   "Numerical differentiantion (Richardson extrapolation)",
-  &ncm_fit_ls_J_nd_ce,
-  &ncm_fit_ls_f_J_nd_ce,
+  &ncm_fit_ls_J_nd_ac,
+  &ncm_fit_ls_f_J_nd_ac,
   &ncm_fit_m2lnL_grad_nd_ac,
   &ncm_fit_m2lnL_val_grad_nd_ac,
 };
 
 /**
  * ncm_fit_set_grad_type:
- * @fit: a #NcmLikelihood.
- * @gtype: a #NcmFitGradType.
+ * @fit: a #NcmLikelihood
+ * @gtype: a #NcmFitGradType
  *
  * FIXME
  *
@@ -688,10 +694,11 @@ ncm_fit_set_grad_type (NcmFit *fit, NcmFitGradType gtype)
 
 /**
  * ncm_fit_set_maxiter:
- * @fit: a #NcmFit.
- * @maxiter: FIXME.
+ * @fit: a #NcmFit
+ * @maxiter: maximum number of interations
  *
- * FIXME
+ * Sets the maximum number of iterations.
+ * 
  */
 void
 ncm_fit_set_maxiter (NcmFit *fit, guint maxiter)
@@ -703,9 +710,9 @@ ncm_fit_set_maxiter (NcmFit *fit, guint maxiter)
  * ncm_fit_get_maxiter:
  * @fit: a #NcmFit.
  *
- * FIXME
+ * Gets the maximum number of iterations.
  *
- * Returns: FIXME
+ * Returns: a integer (maxiter) that corresponds to the maximum number of iterations.  
  */
 guint
 ncm_fit_get_maxiter (NcmFit *fit)
@@ -715,8 +722,8 @@ ncm_fit_get_maxiter (NcmFit *fit)
 
 /**
  * ncm_fit_set_m2lnL_reltol:
- * @fit: a #NcmFit.
- * @tol: FIXME.
+ * @fit: a #NcmFit
+ * @tol: relative tolarance
  *
  * FIXME
  */
@@ -728,11 +735,11 @@ ncm_fit_set_m2lnL_reltol (NcmFit *fit, gdouble tol)
 
 /**
  * ncm_fit_get_m2lnL_reltol:
- * @fit: a #NcmFit.
+ * @fit: a #NcmFit
  *
  * FIXME
  *
- * Returns: FIXME
+ * Returns: the relative tolerance (double).
  */
 gdouble
 ncm_fit_get_m2lnL_reltol (NcmFit *fit)
@@ -742,8 +749,8 @@ ncm_fit_get_m2lnL_reltol (NcmFit *fit)
 
 /**
  * ncm_fit_set_m2lnL_abstol:
- * @fit: a #NcmFit.
- * @tol: FIXME.
+ * @fit: a #NcmFit
+ * @tol: absolute tolerance
  *
  * FIXME
  */
@@ -755,11 +762,11 @@ ncm_fit_set_m2lnL_abstol (NcmFit *fit, gdouble tol)
 
 /**
  * ncm_fit_get_m2lnL_abstol:
- * @fit: a #NcmFit.
+ * @fit: a #NcmFit
  *
  * FIXME
  *
- * Returns: FIXME
+ * Returns: the absolute tolerance (double).
  */
 gdouble
 ncm_fit_get_m2lnL_abstol (NcmFit *fit)
@@ -769,8 +776,8 @@ ncm_fit_get_m2lnL_abstol (NcmFit *fit)
 
 /**
  * ncm_fit_set_params_reltol:
- * @fit: a #NcmFit.
- * @tol: FIXME.
+ * @fit: a #NcmFit
+ * @tol: relative tolerance
  *
  * FIXME
  */
@@ -782,7 +789,7 @@ ncm_fit_set_params_reltol (NcmFit *fit, gdouble tol)
 
 /**
  * ncm_fit_get_params_reltol:
- * @fit: a #NcmFit.
+ * @fit: a #NcmFit
  *
  * FIXME
  *
@@ -796,7 +803,7 @@ ncm_fit_get_params_reltol (NcmFit *fit)
 
 /**
  * ncm_fit_params_set_vector:
- * @fit: a #NcmFit.
+ * @fit: a #NcmFit
  * @x: a #NcmVector
  *
  * FIXME
@@ -804,7 +811,7 @@ ncm_fit_get_params_reltol (NcmFit *fit)
  */
 /**
  * ncm_fit_params_set_vector_offset:
- * @fit: a #NcmFit.
+ * @fit: a #NcmFit
  * @x: a #NcmVector
  * @offset: FIXME
  *
@@ -813,7 +820,7 @@ ncm_fit_get_params_reltol (NcmFit *fit)
  */
 /**
  * ncm_fit_params_set_array:
- * @fit: a #NcmFit.
+ * @fit: a #NcmFit
  * @x: an array of gdoubles
  *
  * FIXME
@@ -821,7 +828,7 @@ ncm_fit_get_params_reltol (NcmFit *fit)
  */
 /**
  * ncm_fit_params_set_gsl_vector: (skip)
- * @fit: a #NcmFit.
+ * @fit: a #NcmFit
  * @x: a gsl_vector
  *
  * FIXME
@@ -837,8 +844,8 @@ ncm_fit_get_params_reltol (NcmFit *fit)
 
 /**
  * ncm_fit_add_equality_constraint:
- * @fit: a #NcmFit.
- * @func: FIXME
+ * @fit: a #NcmFit
+ * @func: a #NcmMSetFunc
  * @tot: FIXME
  *
  * FIXME
@@ -853,8 +860,8 @@ ncm_fit_add_equality_constraint (NcmFit *fit, NcmMSetFunc *func, gdouble tot)
 
 /**
  * ncm_fit_add_inequality_constraint:
- * @fit: a #NcmFit.
- * @func: FIXME
+ * @fit: a #NcmFit
+ * @func: a #NcmMSetFunc
  * @tot: FIXME
  *
  * FIXME
@@ -869,7 +876,7 @@ ncm_fit_add_inequality_constraint (NcmFit *fit, NcmMSetFunc *func, gdouble tot)
 
 /**
  * ncm_fit_remove_equality_constraints:
- * @fit: a #NcmFit.
+ * @fit: a #NcmFit
  *
  * FIXME
  *
@@ -882,7 +889,7 @@ ncm_fit_remove_equality_constraints (NcmFit *fit)
 
 /**
  * ncm_fit_remove_inequality_constraints:
- * @fit: a #NcmFit.
+ * @fit: a #NcmFit
  *
  * FIXME
  *
@@ -895,7 +902,7 @@ ncm_fit_remove_inequality_constraints (NcmFit *fit)
 
 /**
  * ncm_fit_has_equality_constraints:
- * @fit: a #NcmFit.
+ * @fit: a #NcmFit
  *
  * FIXME
  *
@@ -909,7 +916,7 @@ ncm_fit_has_equality_constraints (NcmFit *fit)
 
 /**
  * ncm_fit_has_inequality_constraints:
- * @fit: a #NcmFit.
+ * @fit: a #NcmFit
  *
  * FIXME
  *
@@ -923,9 +930,10 @@ ncm_fit_has_inequality_constraints (NcmFit *fit)
 
 /**
  * ncm_fit_ls_covar:
- * @fit: a #NcmFit.
+ * @fit: a #NcmFit
  *
- * FIXME
+ * Computes the covariance matrix using the least squares method, 
+ * and fills up the internal structure matrix.
  *
  */
 void
@@ -940,12 +948,16 @@ ncm_fit_ls_covar (NcmFit *fit)
 
 /**
  * ncm_fit_covar_fparam_var:
- * @fit: a #NcmFit.
- * @fpi: FIXME
+ * @fit: a #NcmFit
+ * @fpi: index of a free parameter
  *
- * FIXME
+ * Computes the variance of the fitted parameter @fpi. 
+ * This index refers to the list of all FREE parameters set in the MSet.
+ * 
+ * See also the similar function ncm_fit_covar_var() to which one has to provide 
+ * the respective model of the parameter.
  *
- * Returns: FIXME
+ * Returns: the variance of the fitted parameter @fpi
  */
 gdouble
 ncm_fit_covar_fparam_var (NcmFit *fit, guint fpi)
@@ -956,12 +968,16 @@ ncm_fit_covar_fparam_var (NcmFit *fit, guint fpi)
 
 /**
  * ncm_fit_covar_fparam_sd:
- * @fit: a #NcmFit.
- * @fpi: FIXME
+ * @fit: a #NcmFit
+ * @fpi: index of a free parameter
  *
- * FIXME
+ * Computes the standard deviation of the fitted parameter @fpi. 
+ * This index refers to the list of all FREE parameters set in the MSet.
+ * 
+ * See also the similar function ncm_fit_covar_sd() to which one has to provide 
+ * the respective model of the parameter.
  *
- * Returns: FIXME
+ * Returns: the standard deviation of the fitted parameter @fpi
  */
 gdouble
 ncm_fit_covar_fparam_sd (NcmFit *fit, guint fpi)
@@ -971,13 +987,17 @@ ncm_fit_covar_fparam_sd (NcmFit *fit, guint fpi)
 
 /**
  * ncm_fit_covar_fparam_cov:
- * @fit: a #NcmFit.
- * @fpi1: FIXME
- * @fpi2: FIXME
+ * @fit: a #NcmFit
+ * @fpi1: index of a free parameter
+ * @fpi2: index of a free parameter
  *
- * FIXME
+ * Computes the covariance between the fitted parameters @fpi1 and @fpi2. 
+ * These indices refers to the list of all FREE parameters set in the MSet.
+ * 
+ * See also the similar function ncm_fit_covar_cov() to which one has to provide 
+ * the respective models of the parameters.
  *
- * Returns: FIXME
+ * Returns: the covariance between the fitted parameters @pdi1 and @fpdi2
  */
 gdouble
 ncm_fit_covar_fparam_cov (NcmFit *fit, guint fpi1, guint fpi2)
@@ -988,13 +1008,17 @@ ncm_fit_covar_fparam_cov (NcmFit *fit, guint fpi1, guint fpi2)
 
 /**
  * ncm_fit_covar_fparam_cor:
- * @fit: a #NcmFit.
- * @fpi1: FIXME
- * @fpi2: FIXME
+ * @fit: a #NcmFit
+ * @fpi1: index of a free parameter
+ * @fpi2: index of a free parameter
  *
- * FIXME
- *
- * Returns: FIXME
+ * Computes the correlation between the fitted parameters @fpi1 and @fpi2. 
+ * These indices refers to the list of all FREE parameters set in the MSet.
+ * 
+ * See also the similar function ncm_fit_covar_cor() to which one has to provide 
+ * the respective models of the parameters. 
+ * 
+ * Returns: the correlation between the fitted parameters @pdi1 and @fpdi2
  */
 gdouble
 ncm_fit_covar_fparam_cor (NcmFit *fit, guint fpi1, guint fpi2)
@@ -1004,13 +1028,13 @@ ncm_fit_covar_fparam_cor (NcmFit *fit, guint fpi1, guint fpi2)
 
 /**
  * ncm_fit_covar_var:
- * @fit: a #NcmFit.
- * @mid: a #NcmModelID.
- * @pid: FIXME
+ * @fit: a #NcmFit
+ * @mid: a #NcmModelID
+ * @pid: the parameter's index of the model @mid (integer)
  *
- * FIXME
+ * Computes the variance of the fitted parameter @pid of the model @mid.
  *
- * Returns: FIXME
+ * Returns: the variance of the fitted parameter @pid
  */
 gdouble
 ncm_fit_covar_var (NcmFit *fit, NcmModelID mid, guint pid)
@@ -1023,13 +1047,13 @@ ncm_fit_covar_var (NcmFit *fit, NcmModelID mid, guint pid)
 
 /**
  * ncm_fit_covar_sd:
- * @fit: a #NcmFit.
- * @mid: a #NcmModelID.
- * @pid: FIXME
+ * @fit: a #NcmFit
+ * @mid: a #NcmModelID
+ * @pid: the parameter's index of the model @mid (integer)
  *
- * FIXME
+ * Computes the standard deviation of the fitted parameter @pid of the model @mid.
  *
- * Returns: FIXME
+ * Returns: the standard deviation of the fitted parameter @pid
  */
 gdouble
 ncm_fit_covar_sd (NcmFit *fit, NcmModelID mid, guint pid)
@@ -1040,14 +1064,15 @@ ncm_fit_covar_sd (NcmFit *fit, NcmModelID mid, guint pid)
 /**
  * ncm_fit_covar_cov:
  * @fit: a #NcmFit
- * @mid1: a #NcmModelID.
- * @pid1: FIXME
- * @mid2: a #NcmModelID.
- * @pid2: FIXME
+ * @mid1: a #NcmModelID
+ * @pid1: the parameter's index of the model @mid1 (integer)
+ * @mid2: a #NcmModelID
+ * @pid2: the parameter's index of the model @mid1 (integer)
  *
- * FIXME
+ * Computes the covariance between the parameters @pid1 and @pid2 of the models 
+ * @mid1 and @mid2, respectively.
  *
- * Returns: FIXME
+ * Returns: the covariance between @pid1 and @pid2
  */
 gdouble
 ncm_fit_covar_cov (NcmFit *fit, NcmModelID mid1, guint pid1, NcmModelID mid2, guint pid2)
@@ -1064,14 +1089,15 @@ ncm_fit_covar_cov (NcmFit *fit, NcmModelID mid1, guint pid1, NcmModelID mid2, gu
 /**
  * ncm_fit_covar_cor:
  * @fit: a #NcmFit
- * @mid1: a #NcmModelID.
- * @pid1: FIXME
- * @mid2: a #NcmModelID.
- * @pid2: FIXME
+ * @mid1: a #NcmModelID
+ * @pid1: the parameter's index of the model @mid1 (integer)
+ * @mid2: a #NcmModelID
+ * @pid2: the parameter's index of the model @mid1 (integer)
  *
- * FIXME
+ * Computes the correlation between the parameters @pid1 and @pid2 of the models 
+ * @mid1 and @mid2, respectively. 
  *
- * Returns: FIXME
+ * Returns: the correlation between @pid1 and @pid2
  */
 gdouble
 ncm_fit_covar_cor (NcmFit *fit, NcmModelID mid1, guint pid1, NcmModelID mid2, guint pid2)
@@ -1152,11 +1178,97 @@ ncm_fit_run (NcmFit *fit, NcmFitRunMsgs mtype)
   return run;
 }
 
+/**
+ * ncm_fit_run_restart:
+ * @fit: a #NcmFit
+ * @mtype: a #NcmFitRunMsgs
+ * @abstol: restart absolute tolerance
+ * @reltol: restart relative tolarence
+ * @save_mset: (nullable): the #NcmMSet used to save progress
+ * @mset_file: (nullable): the file name to save progress
+ * 
+ * Re-runs the fit until the difference between fits are less
+ * than the required tolerance, i.e., 
+ * $$ m2lnL_{i-1} - m2lnL_i < \mathrm{abstol} + \mathrm{reltol}\vert m2lnL_{i-1}\vert. $$
+ *
+ */
+void
+ncm_fit_run_restart (NcmFit *fit, NcmFitRunMsgs mtype, const gdouble abstol, const gdouble reltol, NcmMSet *save_mset, const gchar *mset_file)
+{
+  gboolean save_progress = (mset_file == NULL) ? FALSE : TRUE;
+  NcmMSet *mset_out      = (save_mset == NULL) ? fit->mset : save_mset;
+  NcmSerialize *ser      = ncm_serialize_new (NCM_SERIALIZE_OPT_NONE);
+
+  fit->mtype = mtype;
+
+  if (ncm_mset_fparam_len (fit->mset) == 0)
+  {    
+    ncm_fit_reset (fit);
+    ncm_fit_log_start (fit);
+
+    g_timer_start (fit->timer);
+    _ncm_fit_run_empty (fit, mtype);
+
+    fit->fstate->elapsed_time = g_timer_elapsed (fit->timer, NULL);
+    fit->fstate->is_best_fit  = TRUE;
+
+    ncm_fit_log_end (fit);
+    return;
+  }
+  else
+  {
+    gboolean restart;
+    gdouble last_m2lnL = 0.0, m2lnL = 0.0;
+    gint n = 0;
+    
+    ncm_fit_run (fit, mtype);
+    last_m2lnL = ncm_fit_state_get_m2lnL_curval (fit->fstate);
+
+    if (save_progress)
+    {
+      if (fit->mset != mset_out)
+        ncm_mset_param_set_mset (mset_out, fit->mset);
+      ncm_mset_save (mset_out, ser, mset_file, TRUE);
+    }
+    
+    do
+    {
+      ncm_fit_run (fit, mtype);
+
+      m2lnL   = ncm_fit_state_get_m2lnL_curval (fit->fstate);
+      restart = (last_m2lnL - m2lnL) >= (abstol + reltol * fabs (last_m2lnL));
+      
+      if (save_progress)
+      {
+        if (fit->mset != mset_out)
+          ncm_mset_param_set_mset (mset_out, fit->mset);
+        ncm_mset_save (mset_out, ser, mset_file, TRUE);
+      }
+
+      if (fit->mtype > NCM_FIT_RUN_MSGS_NONE)
+      {
+        ncm_cfg_msg_sepa ();
+        g_message ("# Restarting:              %s\n", restart ? "yes" : "no");
+        g_message ("#  - absolute improvement: %-22.15g\n", (last_m2lnL - m2lnL));
+        g_message ("#  - relative improvement: %-22.15g\n", (last_m2lnL - m2lnL) / last_m2lnL);
+        g_message ("#  - m2lnL_%-3d:            %-22.15g\n", n,     last_m2lnL);
+        g_message ("#  - m2lnL_%-3d:            %-22.15g\n", n + 1, m2lnL);
+      }
+
+      last_m2lnL = m2lnL;
+      
+      n++;
+    } while (restart);
+  }
+
+  ncm_serialize_free (ser);
+}
+
 /**
  * ncm_fit_is_least_squares:
  * @fit: a #NcmFit
  *
- * FIXME
+ * Indicates if the least squares fitting is being used (TRUE) or not (FALSE).
  *
  * Returns: whenever the fit object use a least squares method.
  */
@@ -1377,7 +1489,8 @@ ncm_fit_log_info (NcmFit *fit)
  * ncm_fit_log_covar:
  * @fit: a #NcmFit
  *
- * FIXME
+ * Prints to the log file the names and indices of the fitted parameters, their best-fit 
+ * values, standard deviations and correlation matrix.
  */
 void
 ncm_fit_log_covar (NcmFit *fit)
@@ -1469,6 +1582,9 @@ ncm_fit_fishermatrix_print (NcmFit *fit, FILE *out, gchar *header)
  * FIXME
  */
 
+static gdouble _ncm_fit_numdiff_m2lnL_val (NcmVector *x, gpointer user_data);
+static void _ncm_fit_numdiff_ls_f (NcmVector *x, NcmVector *y, gpointer user_data);
+
 /**
  * ncm_fit_m2lnL_grad_an:
  * @fit: a #NcmFit
@@ -1551,9 +1667,9 @@ ncm_fit_m2lnL_grad_nd_ce (NcmFit *fit, NcmVector *grad)
 void
 ncm_fit_m2lnL_hessian_nd_ce (NcmFit *fit, NcmMatrix *hessian)
 {
-  guint i;
   guint fparam_len = ncm_mset_fparam_len (fit->mset);
   NcmVector *tmp = ncm_vector_new (fparam_len);
+  guint i;
 
   for (i = 0; i < fparam_len; i++)
   {
@@ -1584,22 +1700,6 @@ ncm_fit_m2lnL_hessian_nd_ce (NcmFit *fit, NcmMatrix *hessian)
   fit->fstate->grad_eval++;
 }
 
-typedef struct __ncm_fit_numdiff_1
-{
-  NcmFit *fit;
-  guint n;
-} __ncm_fit_numdiff_1;
-
-static gdouble
-_ncm_fit_numdiff_1_m2lnL (gdouble x, gpointer userdata)
-{
-  gdouble res;
-  __ncm_fit_numdiff_1 *nd = (__ncm_fit_numdiff_1 *)userdata;
-  ncm_fit_params_set (nd->fit, nd->n, x);
-  ncm_fit_m2lnL_val (nd->fit, &res);
-  return res;
-}
-
 /**
  * ncm_fit_m2lnL_grad_nd_ac:
  * @fit: a #NcmFit
@@ -1611,25 +1711,23 @@ _ncm_fit_numdiff_1_m2lnL (gdouble x, gpointer userdata)
 void
 ncm_fit_m2lnL_grad_nd_ac (NcmFit *fit, NcmVector *grad)
 {
-  gsl_function F;
-  __ncm_fit_numdiff_1 nd;
-  guint fparam_len = ncm_mset_fparam_len (fit->mset);
-  guint i;
+  const guint free_params_len = ncm_mset_fparams_len (fit->mset);
+  GArray *x_a                 = g_array_new (FALSE, FALSE, sizeof (gdouble));
+  NcmVector *x                = NULL;
+  GArray *grad_a              = NULL;
 
-  nd.fit = fit;
-  F.params = &nd;
-  F.function = &_ncm_fit_numdiff_1_m2lnL;
+  g_array_set_size (x_a, free_params_len);
+  x = ncm_vector_new_array (x_a);
 
-  for (i = 0; i < fparam_len; i++)
-  {
-    const gdouble p = ncm_mset_fparam_get (fit->mset, i);
-    const gdouble p_scale = ncm_mset_fparam_get_scale (fit->mset, i);
-    gdouble err, diff;
-    nd.n = i;
-    diff = ncm_numdiff_1 (&F, p, p_scale, &err);
-    ncm_vector_set (grad, i, diff);
-    ncm_fit_params_set (fit, i, p);
-  }
+  ncm_mset_fparams_get_vector (fit->mset, x);
+  grad_a = ncm_diff_rf_d1_N_to_1 (fit->diff, x_a, _ncm_fit_numdiff_m2lnL_val, fit, NULL);
+
+  ncm_vector_set_array (grad, grad_a);
+  ncm_mset_fparams_set_vector (fit->mset, x);
+
+  g_array_unref (x_a);
+  g_array_unref (grad_a);
+  ncm_vector_free (x);
 
   fit->fstate->grad_eval++;
 }
@@ -1798,8 +1896,8 @@ ncm_fit_ls_J_nd_fo (NcmFit *fit, NcmMatrix *J)
 void
 ncm_fit_ls_J_nd_ce (NcmFit *fit, NcmMatrix *J)
 {
-  guint i;
   guint fparam_len = ncm_mset_fparam_len (fit->mset);
+  guint i;
 
   for (i = 0; i < fparam_len; i++)
   {
@@ -1830,6 +1928,39 @@ ncm_fit_ls_J_nd_ce (NcmFit *fit, NcmMatrix *J)
   fit->fstate->grad_eval++;
 }
 
+/**
+ * ncm_fit_ls_J_nd_ac:
+ * @fit: a #NcmFit
+ * @J: a #NcmMatrix
+ *
+ * FIXME
+ *
+ */
+void
+ncm_fit_ls_J_nd_ac (NcmFit *fit, NcmMatrix *J)
+{
+  const guint fparam_len = ncm_mset_fparam_len (fit->mset);
+  const guint data_len   = ncm_fit_state_get_data_len (fit->fstate);
+  GArray *x_a            = g_array_new (FALSE, FALSE, sizeof (gdouble));
+  NcmVector *x           = NULL;
+  GArray *J_a            = NULL;
+
+  g_array_set_size (x_a, fparam_len);
+  x = ncm_vector_new_array (x_a);
+
+  ncm_mset_fparams_get_vector (fit->mset, x);
+  J_a = ncm_diff_rf_d1_N_to_M (fit->diff, x_a, data_len, _ncm_fit_numdiff_ls_f, fit, NULL);
+
+  ncm_matrix_set_from_array (J, J_a);
+  ncm_mset_fparams_set_vector (fit->mset, x);
+
+  g_array_unref (x_a);
+  g_array_unref (J_a);
+  ncm_vector_free (x);
+
+  fit->fstate->grad_eval++;
+}
+
 /**
  * ncm_fit_ls_f_J:
  * @fit: a #NcmFit
@@ -1888,144 +2019,98 @@ ncm_fit_ls_f_J_nd_ce (NcmFit *fit, NcmVector *f, NcmMatrix *J)
   ncm_fit_ls_J_nd_ce (fit, J);
 }
 
-typedef struct __ncm_fit_numdiff_2
+/**
+ * ncm_fit_ls_f_J_nd_ac:
+ * @fit: a #NcmFit
+ * @f: a #NcmVector
+ * @J: a #NcmMatrix
+ *
+ * FIXME
+ */
+void
+ncm_fit_ls_f_J_nd_ac (NcmFit *fit, NcmVector *f, NcmMatrix *J)
 {
-  NcmFit *fit;
-  guint n1;
-  guint n2;
-  gdouble v;
-  gdouble p1_scale;
-  gdouble p2_scale;
-} _ncm_fit_numdiff_2;
+  ncm_fit_ls_f (fit, f);
+  ncm_fit_ls_J_nd_ac (fit, J);
+}
 
-static gdouble
-_ncm_fit_numdiff_2_m2lnL (gdouble u, gpointer userdata)
+static gdouble 
+_ncm_fit_numdiff_m2lnL_val (NcmVector *x, gpointer user_data)
 {
-  gdouble res;
-  _ncm_fit_numdiff_2 *nd = (_ncm_fit_numdiff_2 *)userdata;
-  if (nd->n1 == nd->n2)
-    ncm_mset_fparam_set (nd->fit->mset, nd->n1, u);
-  else
-  {
-    ncm_mset_fparam_set (nd->fit->mset, nd->n1, (u + nd->v) / nd->p1_scale);
-    ncm_mset_fparam_set (nd->fit->mset, nd->n2, (u - nd->v) / nd->p2_scale);
-  }
-  ncm_likelihood_m2lnL_val (nd->fit->lh, nd->fit->mset, &res);
-  /*ncm_fit_m2lnL_val (nd->fit, &res);*/
+  NcmFit *fit = NCM_FIT (user_data);
+  gdouble res = 0.0;
+  ncm_mset_fparams_set_vector (fit->mset, x);
+
+  ncm_likelihood_m2lnL_val (fit->lh, fit->mset, &res);
   return res;
 }
 
+static void
+_ncm_fit_numdiff_ls_f (NcmVector *x, NcmVector *y, gpointer user_data)
+{
+  NcmFit *fit = NCM_FIT (user_data);
+
+  ncm_mset_fparams_set_vector (fit->mset, x);
+
+  ncm_fit_ls_f (fit, y);
+}
+
 /**
  * ncm_fit_numdiff_m2lnL_hessian:
  * @fit: a #NcmFit
  * @H: a #NcmMatrix
  * @reltol: relative tolerance.
  *
- * FIXME
+ * Calculates the Hessian of $-2\ln(L)$ with respect to the free parameters. 
+ * 
  */
 void
 ncm_fit_numdiff_m2lnL_hessian (NcmFit *fit, NcmMatrix *H, gdouble reltol)
 {
-  gsl_function F;
-  _ncm_fit_numdiff_2 nd;
-  guint i, j;
-  gdouble fx;
-  const gdouble target_err = reltol;
-  guint free_params_len = ncm_mset_fparams_len (fit->mset);
+  const guint free_params_len = ncm_mset_fparams_len (fit->mset);
+  GArray *x_a                 = g_array_new (FALSE, FALSE, sizeof (gdouble));
+  NcmVector *x                = NULL;
+  GArray *H_a                 = NULL;
 
-  ncm_likelihood_m2lnL_val (fit->lh, fit->mset, &fx);
-  /*ncm_fit_m2lnL_val (fit, &fx);*/
+  g_array_set_size (x_a, free_params_len);
+  x = ncm_vector_new_array (x_a);
 
-  nd.fit = fit;
-  F.params = &nd;
-  F.function = &_ncm_fit_numdiff_2_m2lnL;
+  ncm_mset_fparams_get_vector (fit->mset, x);
+  H_a = ncm_diff_rf_Hessian_N_to_1 (fit->diff, x_a, _ncm_fit_numdiff_m2lnL_val, fit, NULL);
 
-  /* Diagonal */
-  for (i = 0; i < free_params_len; i++)
-  {
-    const gdouble p = ncm_mset_fparam_get (fit->mset, i);
-    gdouble p_scale = ncm_mset_fparam_get_scale (fit->mset, i);
-    gdouble err, diff;
-    gint tries = 10;
-    nd.n1 = i;
-    nd.n2 = i;
-    diff = ncm_numdiff_2_err (&F, &fx, p, p_scale, target_err, &err);
+  ncm_matrix_set_from_array (H, H_a);
+  ncm_mset_fparams_set_vector (fit->mset, x);
 
-    while (diff == 0.0 && tries > 0)
-    {
-      ncm_fit_params_set (fit, i, p);
-      p_scale *= 1.0e2;
-      diff = ncm_numdiff_2_err (&F, &fx, p, p_scale, target_err, &err);
-      tries--;
-      ncm_mset_fparam_set_scale (fit->mset, i, p_scale);
-    }
-
-    if (fabs(err / diff) > target_err)
-      g_warning ("ncm_fit_numdiff_m2lnL_hessian: effective error on second derivative with respect to parameter %u is (% 20.15e) larger than the required (% 20.15e)", i, fabs(err / diff), target_err);
-    if (diff == 0.0)
-      g_warning ("ncm_fit_numdiff_m2lnL_hessian: the second derivatinve with respect to parameter %u is zero.", i);
-
-    ncm_matrix_set (H, i, i, diff);
-    ncm_fit_params_set (fit, i, p);
-  }
-
-  for (i = 0; i < free_params_len; i++)
-  {
-    for (j = i + 1; j < free_params_len; j++)
-    {
-      const gdouble p1_scale = 1.0 / ncm_mset_fparam_get_scale (fit->mset, i);
-      const gdouble p2_scale = 1.0 / ncm_mset_fparam_get_scale (fit->mset, j);
-      const gdouble p1 = ncm_mset_fparam_get (fit->mset, i);
-      const gdouble p2 = ncm_mset_fparam_get (fit->mset, j);
-      const gdouble u = (p1_scale * p1 + p2_scale * p2) / 2.0;
-      const gdouble v = (p1_scale * p1 - p2_scale * p2) / 2.0;
-      const gdouble u_scale = 1.0;
-      gdouble err, diff;
-      nd.n1 = i;
-      nd.n2 = j;
-      nd.v = v;
-      nd.p1_scale = p1_scale;
-      nd.p2_scale = p2_scale;
-      diff = ncm_numdiff_2_err (&F, &fx, u, u_scale, target_err, &err);
-      if (fabs(err / diff) > target_err)
-        g_warning ("ncm_fit_numdiff_m2lnL_hessian: effective error on the %u-%u derivative is (% 20.15e) larger than the required (% 20.15e)", i, j, fabs(err / diff), target_err);
-      ncm_matrix_set (H, i, j,
-                      0.5 * ( p1_scale * p2_scale * diff -
-                             (p2_scale / p1_scale) * ncm_matrix_get (H, i, i) -
-                             (p1_scale / p2_scale) * ncm_matrix_get (H, j, j)
-                             )
-                      );
-      ncm_matrix_set (H, j, i, ncm_matrix_get (H, i, j));
-      //printf ("d2[%d %d] = % 20.15g\n", i, j, (diff - gsl_matrix_get (H, k, k) - gsl_matrix_get (H, l, l)) / 2.0);
-      ncm_fit_params_set (fit, i, p1);
-      ncm_fit_params_set (fit, j, p2);
-    }
-  }
+  g_array_unref (x_a);
+  g_array_unref (H_a);
+  ncm_vector_free (x);
 }
 
 /**
- * ncm_fit_numdiff_m2lnL_covar:
+ * ncm_fit_fisher_to_covar:
  * @fit: a #NcmFit
- *
- * FIXME
+ * @fisher: a #NcmMatrix
+ * 
+ * Inverts the matrix @fisher and sets as the covariance matrix
+ * of @fit. The Fisher matrix used can be both the Fisher or the
+ * observed Fisher matrices.
+ * 
  */
 void
-ncm_fit_numdiff_m2lnL_covar (NcmFit *fit)
+ncm_fit_fisher_to_covar (NcmFit *fit, NcmMatrix *fisher)
 {
   gint ret;
   if (ncm_mset_fparam_len (fit->mset) == 0)
-    g_error ("ncm_fit_numdiff_m2lnL_covar: mset object has 0 free parameters");
+    g_error ("ncm_fit_fisher_to_covar: mset object has 0 free parameters");
 
-  ncm_fit_numdiff_m2lnL_hessian (fit, fit->fstate->hessian, fit->params_reltol);
-  ncm_matrix_memcpy (fit->fstate->covar, fit->fstate->hessian);
-  ncm_matrix_scale (fit->fstate->covar, 0.5);
+  ncm_matrix_memcpy (fit->fstate->covar, fisher);
 
   ret = ncm_matrix_cholesky_decomp (fit->fstate->covar, 'U');
   if (ret == 0)
   {
     ret = ncm_matrix_cholesky_inverse (fit->fstate->covar, 'U');
     if (ret != 0)
-      g_error ("ncm_fit_numdiff_m2lnL_covar[ncm_matrix_cholesky_decomp]: %d.", ret);
+      g_error ("ncm_fit_fisher_to_covar[ncm_matrix_cholesky_decomp]: %d.", ret);
     ncm_matrix_copy_triangle (fit->fstate->covar, 'U');
   }
   else if (ret > 0)
@@ -2037,23 +2122,79 @@ ncm_fit_numdiff_m2lnL_covar (NcmFit *fit)
 
     ncm_matrix_scale (LU, 0.5);
 
-    g_warning ("ncm_fit_numdiff_m2lnL_covar: covariance matrix not positive definite, errors are not trustworthy.");
+    g_warning ("ncm_fit_fisher_to_covar: covariance matrix not positive definite, errors are not trustworthy.");
 
     ret1 = gsl_linalg_LU_decomp (ncm_matrix_gsl (LU), p, &signum);
-    NCM_TEST_GSL_RESULT ("ncm_fit_numdiff_m2lnL_covar[gsl_linalg_LU_decomp]", ret1);
+    NCM_TEST_GSL_RESULT ("ncm_fit_fisher_to_covar[gsl_linalg_LU_decomp]", ret1);
 
     ret1 = gsl_linalg_LU_invert (ncm_matrix_gsl (LU), p, ncm_matrix_gsl (fit->fstate->covar));
-    NCM_TEST_GSL_RESULT ("ncm_fit_numdiff_m2lnL_covar[gsl_linalg_LU_invert]", ret1);
+    NCM_TEST_GSL_RESULT ("ncm_fit_fisher_to_covar[gsl_linalg_LU_invert]", ret1);
 
     gsl_permutation_free (p);
     ncm_matrix_free (LU);
   }
   else
-    g_error ("ncm_fit_numdiff_m2lnL_covar[ncm_matrix_cholesky_decomp]: %d.", ret);
+    g_error ("ncm_fit_fisher_to_covar[ncm_matrix_cholesky_decomp]: %d.", ret);
 
   fit->fstate->has_covar = TRUE;
 }
 
+/**
+ * ncm_fit_obs_fisher:
+ * @fit: a #NcmFit
+ *
+ * Calculates the covariance from the observed Fisher 
+ * matrix, see ncm_fit_numdiff_m2lnL_covar().
+ * 
+ */
+void
+ncm_fit_obs_fisher (NcmFit *fit)
+{
+  ncm_fit_numdiff_m2lnL_covar (fit);
+}
+
+/**
+ * ncm_fit_fisher:
+ * @fit: a #NcmFit
+ *
+ * Calculates the covariance from the Fisher matrix, see 
+ * ncm_dataset_fisher_matrix().
+ * 
+ */
+void
+ncm_fit_fisher (NcmFit *fit)
+{
+  NcmMatrix *IM = NULL;
+  if ((ncm_likelihood_priors_length_f (fit->lh) > 0) || (ncm_likelihood_priors_length_m2lnL (fit->lh) > 0))
+    g_warning ("ncm_fit_fisher: the analysis contains priors which are ignored in the Fisher matrix calculation.");
+
+  ncm_dataset_fisher_matrix (fit->lh->dset, fit->mset, &IM);
+  ncm_fit_fisher_to_covar (fit, IM);
+
+  ncm_matrix_clear (&IM);
+}
+
+/**
+ * ncm_fit_numdiff_m2lnL_covar:
+ * @fit: a #NcmFit
+ *
+ * Calcualtes the covariance matrix using the inverse of the 
+ * Hessian matrix $\partial_i\partial_j -\ln(L)$, where
+ * the derivatives are taken with respect to the free parameters. 
+ * 
+ */
+void
+ncm_fit_numdiff_m2lnL_covar (NcmFit *fit)
+{
+  if (ncm_mset_fparam_len (fit->mset) == 0)
+    g_error ("ncm_fit_numdiff_m2lnL_covar: mset object has 0 free parameters");
+
+  ncm_fit_numdiff_m2lnL_hessian (fit, fit->fstate->hessian, fit->params_reltol);
+  ncm_matrix_scale (fit->fstate->hessian, 0.5);
+
+  ncm_fit_fisher_to_covar (fit, fit->fstate->hessian);
+}
+
 /**
  * ncm_fit_numdiff_m2lnL_lndet_covar:
  * @fit: a #NcmFit
@@ -2476,11 +2617,14 @@ ncm_fit_function_error (NcmFit *fit, NcmMSetFunc *func, gdouble *x, gboolean pre
   }
   else
   {
-    NcmVector *v = ncm_mset_func_numdiff_fparams (func, fit->mset, x, NULL);
-    NcmVector *tmp1 = ncm_vector_dup (v);
+    NcmVector *v    = NULL;
+    NcmVector *tmp1 = NULL;
     gdouble result;
     gint ret;
 
+    ncm_mset_func_numdiff_fparams (func, fit->mset, x, &v);
+    tmp1 = ncm_vector_dup (v);
+
     if (fit->fstate->covar == NULL)
       g_error ("ncm_fit_function_error: called without any covariance matrix calculated.");
 
