@@ -42,7 +42,9 @@
 #include "math/ncm_func_eval.h"
 #include "ncm_enum_types.h"
 
+#ifndef NUMCOSMO_GIR_SCAN
 #include <gsl/gsl_statistics_double.h>
+#endif /* NUMCOSMO_GIR_SCAN */
 
 enum
 {
@@ -51,6 +53,11 @@ enum
   PROP_NWALKERS,
   PROP_SAMPLER,
   PROP_WALKER,
+  PROP_AUTO_TRIM,
+  PROP_AUTO_TRIM_DIV,
+  PROP_TRIM_TYPE,
+  PROP_MIN_RUNS,
+  PROP_MAX_RUNS_TIME,
   PROP_MTYPE,
   PROP_NTHREADS,
   PROP_DATA_FILE,
@@ -75,6 +82,11 @@ ncm_fit_esmcmc_init (NcmFitESMCMC *esmcmc)
   esmcmc->nt              = ncm_timer_new ();
   esmcmc->ser             = ncm_serialize_new (NCM_SERIALIZE_OPT_CLEAN_DUP);
   esmcmc->walker          = NULL;
+  esmcmc->auto_trim       = FALSE;
+  esmcmc->auto_trim_div   = 0;
+  esmcmc->trim_type       = 0;
+  esmcmc->min_runs        = 0;
+  esmcmc->max_runs_time   = 0.0;
   esmcmc->nadd_vals       = 0;
   esmcmc->fparam_len      = 0;
 
@@ -99,7 +111,7 @@ ncm_fit_esmcmc_init (NcmFitESMCMC *esmcmc)
   esmcmc->nthreads        = 0;
   esmcmc->n               = 0;
   esmcmc->nwalkers        = 0;
-  esmcmc->cur_sample_id   = -1; /* Represents that no samples were calculated yet. */
+  esmcmc->cur_sample_id   = -1; /* Represents that no samples were calculated yet, i.e., id of the last added point. */
   esmcmc->ntotal          = 0;
   esmcmc->naccepted       = 0;
   esmcmc->noffboard       = 0;
@@ -158,6 +170,7 @@ _ncm_fit_esmcmc_constructed (GObject *object)
                                              NCM_MSET_CATALOG_M2LNL_COLNAME, NCM_MSET_CATALOG_M2LNL_SYMBOL, 
                                              NULL);
       }
+      ncm_mset_catalog_set_m2lnp_var (esmcmc->mcat, 0);
       ncm_mset_catalog_set_run_type (esmcmc->mcat, "Ensemble Sampler MCMC");
 
       for (k = 0; k < esmcmc->nwalkers; k++)
@@ -209,6 +222,21 @@ ncm_fit_esmcmc_set_property (GObject *object, guint prop_id, const GValue *value
     case PROP_WALKER:
       esmcmc->walker = g_value_dup_object (value);
       break;
+    case PROP_AUTO_TRIM:
+      esmcmc->auto_trim = g_value_get_boolean (value);
+      break;
+    case PROP_AUTO_TRIM_DIV:
+      esmcmc->auto_trim_div = g_value_get_uint (value);
+      break;
+    case PROP_TRIM_TYPE:
+      esmcmc->trim_type = g_value_get_flags (value);
+      break;
+    case PROP_MIN_RUNS:
+      esmcmc->min_runs = g_value_get_uint (value);
+      break;
+    case PROP_MAX_RUNS_TIME:
+      esmcmc->max_runs_time = g_value_get_double (value);
+      break;
     case PROP_MTYPE:
       ncm_fit_esmcmc_set_mtype (esmcmc, g_value_get_enum (value));
       break;
@@ -260,6 +288,21 @@ ncm_fit_esmcmc_get_property (GObject *object, guint prop_id, GValue *value, GPar
     case PROP_WALKER:
       g_value_set_object (value, esmcmc->walker);
       break;
+    case PROP_AUTO_TRIM:
+      g_value_set_boolean (value, esmcmc->auto_trim);
+      break;
+    case PROP_AUTO_TRIM_DIV:
+      g_value_set_uint (value, esmcmc->auto_trim_div);
+      break;
+    case PROP_TRIM_TYPE:
+      g_value_set_flags (value, esmcmc->trim_type);
+      break;
+    case PROP_MIN_RUNS:
+      g_value_set_uint (value, esmcmc->min_runs);
+      break;
+    case PROP_MAX_RUNS_TIME:
+      g_value_set_double (value, esmcmc->max_runs_time);
+      break;
     case PROP_MTYPE:
       g_value_set_enum (value, esmcmc->mtype);
       break;
@@ -371,6 +414,41 @@ ncm_fit_esmcmc_class_init (NcmFitESMCMCClass *klass)
                                                         "Walker object",
                                                         NCM_TYPE_FIT_ESMCMC_WALKER,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_AUTO_TRIM,
+                                   g_param_spec_boolean ("auto-trim",
+                                                         NULL,
+                                                         "Whether to automatically trim the catalog",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_AUTO_TRIM_DIV,
+                                   g_param_spec_uint ("auto-trim-div",
+                                                      NULL,
+                                                      "Automatically trim divisor",
+                                                      1, G_MAXUINT, 100,
+                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_TRIM_TYPE,
+                                   g_param_spec_flags ("trim-type",
+                                                       NULL,
+                                                       "Trimming tests to apply",
+                                                       NCM_TYPE_MSET_CATALOG_TRIM_TYPE, NCM_MSET_CATALOG_TRIM_TYPE_ESS,
+                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_MIN_RUNS,
+                                   g_param_spec_uint ("min-runs",
+                                                      NULL,
+                                                      "Minumum number of runs",
+                                                      1, G_MAXUINT, 10,
+                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_MAX_RUNS_TIME,
+                                   g_param_spec_double ("max-runs-time",
+                                                        NULL,
+                                                        "Maximum time between runs",
+                                                        1.0, G_MAXDOUBLE, 2.0 * 60.0 * 60.0,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_MTYPE,
                                    g_param_spec_enum ("mtype",
@@ -424,7 +502,7 @@ _ncm_fit_esmcmc_worker_dup (gpointer userdata)
     else
       fw->funcs_array = NULL;
 
-    ncm_serialize_reset (esmcmc->ser);
+    ncm_serialize_reset (esmcmc->ser, TRUE);
     
     G_UNLOCK (dup_thread);
 
@@ -466,12 +544,12 @@ NcmFitESMCMC *
 ncm_fit_esmcmc_new (NcmFit *fit, gint nwalkers, NcmMSetTransKern *sampler, NcmFitESMCMCWalker *walker, NcmFitRunMsgs mtype)
 {
   NcmFitESMCMC *esmcmc = g_object_new (NCM_TYPE_FIT_ESMCMC, 
-                                "fit", fit,
-                                "nwalkers", nwalkers,
-                                "sampler", sampler,
-                                "walker", walker,
-                                "mtype", mtype,
-                                NULL);
+                                       "fit", fit,
+                                       "nwalkers", nwalkers,
+                                       "sampler", sampler,
+                                       "walker", walker,
+                                       "mtype", mtype,
+                                       NULL);
   return esmcmc;
 }
 
@@ -492,17 +570,31 @@ NcmFitESMCMC *
 ncm_fit_esmcmc_new_funcs_array (NcmFit *fit, gint nwalkers, NcmMSetTransKern *sampler, NcmFitESMCMCWalker *walker, NcmFitRunMsgs mtype, NcmObjArray *funcs_array)
 {
   NcmFitESMCMC *esmcmc = g_object_new (NCM_TYPE_FIT_ESMCMC, 
-                                "fit", fit,
-                                "nwalkers",        nwalkers,
-                                "sampler",         sampler,
-                                "walker",          walker,
-                                "mtype",           mtype,
-                                "functions-array", funcs_array,
-                                NULL);
+                                       "fit", fit,
+                                       "nwalkers",        nwalkers,
+                                       "sampler",         sampler,
+                                       "walker",          walker,
+                                       "mtype",           mtype,
+                                       "functions-array", funcs_array,
+                                       NULL);
   return esmcmc;
 }
 
 
+/**
+ * ncm_fit_esmcmc_ref:
+ * @esmcmc: a #NcmFitESMCMC
+ *
+ * FIXME
+ *
+ * Returns: (transfer full): FIXME
+ */
+NcmFitESMCMC *
+ncm_fit_esmcmc_ref (NcmFitESMCMC *esmcmc)
+{
+  return g_object_ref (esmcmc);
+}
+
 /**
  * ncm_fit_esmcmc_free:
  * @esmcmc: a #NcmFitESMCMC
@@ -563,7 +655,7 @@ ncm_fit_esmcmc_set_data_file (NcmFitESMCMC *esmcmc, const gchar *filename)
   }
   
   if (esmcmc->started)
-    g_assert_cmpint (esmcmc->cur_sample_id, ==, esmcmc->mcat->cur_id);
+    g_assert_cmpint (esmcmc->cur_sample_id, ==, ncm_mset_catalog_get_cur_id (esmcmc->mcat));
 }
 
 /**
@@ -634,6 +726,79 @@ ncm_fit_esmcmc_set_rng (NcmFitESMCMC *esmcmc, NcmRNG *rng)
   ncm_mset_catalog_set_rng (esmcmc->mcat, rng);
 }
 
+/**
+ * ncm_fit_esmcmc_set_auto_trim:
+ * @esmcmc: a #NcmFitESMCMC
+ * @enable: a boolean
+ *
+ * If @enable is TRUE turns on the auto-trimming when performing a
+ * run_lre.
+ *
+ */
+void 
+ncm_fit_esmcmc_set_auto_trim (NcmFitESMCMC *esmcmc, gboolean enable)
+{
+  esmcmc->auto_trim = enable;
+}
+
+/**
+ * ncm_fit_esmcmc_set_auto_trim_div:
+ * @esmcmc: a #NcmFitESMCMC
+ * @div: a unsigned integer
+ * 
+ * Sets the divisor for the auto trim tests.
+ *
+ */
+void 
+ncm_fit_esmcmc_set_auto_trim_div (NcmFitESMCMC *esmcmc, guint div)
+{
+  esmcmc->auto_trim_div = div;
+}
+
+/**
+ * ncm_fit_esmcmc_set_min_runs:
+ * @esmcmc: a #NcmFitESMCMC
+ * @min_runs: a unsigned integer
+ * 
+ * Sets the minimum number of runs between tests.
+ *
+ */
+void 
+ncm_fit_esmcmc_set_min_runs (NcmFitESMCMC *esmcmc, guint min_runs)
+{
+  g_assert_cmpuint (min_runs, >, 0);
+  esmcmc->min_runs = min_runs;
+}
+
+/**
+ * ncm_fit_esmcmc_set_max_runs_time:
+ * @esmcmc: a #NcmFitESMCMC
+ * @max_runs_time: a unsigned integer
+ * 
+ * Sets the maximum time for the runs between tests.
+ *
+ */
+void 
+ncm_fit_esmcmc_set_max_runs_time (NcmFitESMCMC *esmcmc, gdouble max_runs_time)
+{
+  g_assert_cmpfloat (max_runs_time, >=, 1.0);
+  esmcmc->max_runs_time = max_runs_time;
+}
+
+/**
+ * ncm_fit_esmcmc_has_rng:
+ * @esmcmc: a #NcmFitESMCMC
+ *
+ * FIXME
+ *
+ * Returns: whether there is a #NcmRNG set.
+ */
+gboolean
+ncm_fit_esmcmc_has_rng (NcmFitESMCMC *esmcmc)
+{
+  return (ncm_mset_catalog_peek_rng (esmcmc->mcat) != NULL);
+}
+
 /**
  * ncm_fit_esmcmc_get_accept_ratio:
  * @esmcmc: a #NcmFitESMCMC
@@ -707,8 +872,12 @@ _ncm_fit_esmcmc_update (NcmFitESMCMC *esmcmc, guint ki, guint kf)
     {
       guint stepi = (esmcmc->cur_sample_id + 1) % step;
       gboolean log_timeout = FALSE;
+
       if ((esmcmc->nt->pos_time - esmcmc->nt->last_log_time) > 60.0)
-        log_timeout = TRUE && ((esmcmc->cur_sample_id + 1) % esmcmc->nwalkers == 0);
+      {
+        log_timeout = ((esmcmc->cur_sample_id + 1) % esmcmc->nwalkers == 0);
+      }
+    
       if (log_timeout || (stepi == 0) || (esmcmc->nt->task_pos == esmcmc->nt->task_len))
       {
         /* guint acc = stepi == 0 ? step : stepi; */
@@ -721,6 +890,7 @@ _ncm_fit_esmcmc_update (NcmFitESMCMC *esmcmc, guint ki, guint kf)
         ncm_timer_task_log_elapsed (esmcmc->nt);
         ncm_timer_task_log_mean_time (esmcmc->nt);
         ncm_timer_task_log_time_left (esmcmc->nt);
+        ncm_timer_task_log_cur_datetime (esmcmc->nt);
         ncm_timer_task_log_end_datetime (esmcmc->nt);
       }
       break;
@@ -749,6 +919,7 @@ _ncm_fit_esmcmc_update (NcmFitESMCMC *esmcmc, guint ki, guint kf)
         ncm_timer_task_log_elapsed (esmcmc->nt);
         ncm_timer_task_log_mean_time (esmcmc->nt);
         ncm_timer_task_log_time_left (esmcmc->nt);
+        ncm_timer_task_log_cur_datetime (esmcmc->nt);
         ncm_timer_task_log_end_datetime (esmcmc->nt);
       }
       break;
@@ -764,6 +935,7 @@ _ncm_fit_esmcmc_gen_init_points_mt_eval (glong i, glong f, gpointer data)
   NcmFitESMCMC *esmcmc        = NCM_FIT_ESMCMC (data);
   NcmFitESMCMCWorker **fk_ptr = ncm_memory_pool_get (esmcmc->walker_pool);
   NcmFit *fit_k               = fk_ptr[0]->fit;
+  NcmRNG *rng                 = ncm_mset_catalog_peek_rng (esmcmc->mcat);
   glong k;
 
   for (k = i; k < f; k++)
@@ -775,7 +947,7 @@ _ncm_fit_esmcmc_gen_init_points_mt_eval (glong i, glong f, gpointer data)
     do
     {
       g_mutex_lock (&esmcmc->resample_lock);
-      ncm_mset_trans_kern_prior_sample (esmcmc->sampler, theta_k, esmcmc->mcat->rng);
+      ncm_mset_trans_kern_prior_sample (esmcmc->sampler, theta_k, rng);
       g_mutex_unlock (&esmcmc->resample_lock);
 
       ncm_mset_fparams_set_vector (fit_k->mset, theta_k);
@@ -834,7 +1006,10 @@ _ncm_fit_esmcmc_gen_init_points (NcmFitESMCMC *esmcmc)
 void 
 ncm_fit_esmcmc_start_run (NcmFitESMCMC *esmcmc)
 {
+  const gint mcat_cur_id = ncm_mset_catalog_get_cur_id (esmcmc->mcat);
   gboolean init_point_task = FALSE;
+	gboolean read_from_cat   = FALSE;
+  
   if (esmcmc->started)
     g_error ("ncm_fit_esmcmc_start_run: run already started, run ncm_fit_esmcmc_end_run() first.");
 
@@ -855,14 +1030,17 @@ ncm_fit_esmcmc_start_run (NcmFitESMCMC *esmcmc)
       break;
   }
 
-  if (esmcmc->mcat->rng == NULL)
+  if (ncm_mset_catalog_peek_rng (esmcmc->mcat) == NULL)
   {
     NcmRNG *rng = ncm_rng_new (NULL);
+    
     ncm_rng_set_random_seed (rng, FALSE);
     ncm_fit_esmcmc_set_rng (esmcmc, rng);
+
     if (esmcmc->mtype > NCM_FIT_RUN_MSGS_NONE)
       g_message ("# NcmFitESMCMC: No RNG was defined, using algorithm: `%s' and seed: %lu.\n",
                  ncm_rng_get_algo (rng), ncm_rng_get_seed (rng));
+
     ncm_rng_free (rng);
   }
 
@@ -879,17 +1057,14 @@ ncm_fit_esmcmc_start_run (NcmFitESMCMC *esmcmc)
   esmcmc->noffboard = 0;
   g_mutex_unlock (&esmcmc->update_lock);
 
-  if (esmcmc->mcat->first_id > 0)
-    g_error ("ncm_fit_esmcmc_start_run: cannot use catalogs with first_id > 0.");
-
-  if (esmcmc->mcat->cur_id > esmcmc->cur_sample_id)
+  if (mcat_cur_id > esmcmc->cur_sample_id)
   {
-    ncm_fit_esmcmc_intern_skip (esmcmc, esmcmc->mcat->cur_id - esmcmc->cur_sample_id);
-    g_assert_cmpint (esmcmc->cur_sample_id, ==, esmcmc->mcat->cur_id);
+    ncm_fit_esmcmc_intern_skip (esmcmc, mcat_cur_id - esmcmc->cur_sample_id);
+    g_assert_cmpint (esmcmc->cur_sample_id, ==, mcat_cur_id);
   }
-  else if (esmcmc->mcat->cur_id < esmcmc->cur_sample_id)
+  else if (mcat_cur_id < esmcmc->cur_sample_id)
     g_error ("ncm_fit_esmcmc_start_run: Unknown error cur_id < cur_sample_id [%d < %d].", 
-             esmcmc->mcat->cur_id, esmcmc->cur_sample_id);
+             mcat_cur_id, esmcmc->cur_sample_id);
 
   if (esmcmc->cur_sample_id + 1 < esmcmc->nwalkers)
   {
@@ -902,12 +1077,16 @@ ncm_fit_esmcmc_start_run (NcmFitESMCMC *esmcmc)
   {
     gint k;
 
+    if (ncm_mset_catalog_get_first_id (esmcmc->mcat) > 0)
+      g_error ("ncm_fit_esmcmc_start_run: cannot use catalogs with first_id > 0 without a complete first ensemble.");
+    
     for (k = 0; k <= esmcmc->cur_sample_id; k++)
     {
       NcmVector *cur_row      = ncm_mset_catalog_peek_row (esmcmc->mcat, k);
       NcmVector *full_theta_k = g_ptr_array_index (esmcmc->full_theta, k);
       g_assert (cur_row != NULL);
-      
+
+			read_from_cat = TRUE;
       ncm_vector_memcpy (full_theta_k, cur_row);
     }
     _ncm_fit_esmcmc_gen_init_points (esmcmc);
@@ -920,30 +1099,56 @@ ncm_fit_esmcmc_start_run (NcmFitESMCMC *esmcmc)
   }
   else
   {
-    guint t = (esmcmc->cur_sample_id + 1) / esmcmc->nwalkers;
-    guint ki = (esmcmc->cur_sample_id + 1) % esmcmc->nwalkers;
-    guint tm1 = t - 1;
+    const guint t   = (esmcmc->cur_sample_id + 1) / esmcmc->nwalkers;
+    const guint ki  = (esmcmc->cur_sample_id + 1) % esmcmc->nwalkers;
+    const guint tm1 = t - 1;
     gint k;
 
     for (k = 0; k < ki; k++)
     {
-      NcmVector *cur_row = ncm_mset_catalog_peek_row (esmcmc->mcat, esmcmc->nwalkers * t + k);
+      const guint row_n       = ncm_mset_catalog_get_row_from_time (esmcmc->mcat, esmcmc->nwalkers * t + k);
+      NcmVector *cur_row      = ncm_mset_catalog_peek_row (esmcmc->mcat, row_n);
       NcmVector *full_theta_k = g_ptr_array_index (esmcmc->full_theta, k);
+
       g_assert (cur_row != NULL);
 
+			read_from_cat = TRUE;
       ncm_vector_memcpy (full_theta_k, cur_row);
     }
 
     for (k = ki; k < esmcmc->nwalkers; k++)
     {
-      NcmVector *cur_row = ncm_mset_catalog_peek_row (esmcmc->mcat, esmcmc->nwalkers * tm1 + k);
+      const guint row_n       = ncm_mset_catalog_get_row_from_time (esmcmc->mcat, esmcmc->nwalkers * tm1 + k);
+      NcmVector *cur_row      = ncm_mset_catalog_peek_row (esmcmc->mcat, row_n);
       NcmVector *full_theta_k = g_ptr_array_index (esmcmc->full_theta, k);
+
       g_assert (cur_row != NULL);
 
+			read_from_cat = TRUE;
       ncm_vector_memcpy (full_theta_k, cur_row);
     }
   }
 
+	if (read_from_cat)
+	{
+		const guint len = ncm_mset_catalog_len (esmcmc->mcat);
+		if (!ncm_fit_esmcmc_validate (esmcmc, len - esmcmc->nwalkers, len))
+		{
+			if (esmcmc->mtype > NCM_FIT_RUN_MSGS_NONE)
+			{
+				ncm_cfg_msg_sepa ();
+				g_message ("# NcmFitESMCMC: Last ensemble failed in the m2lnL check, the catalog may be corrupted, removing last ensemble and retrying...\n");
+			}
+
+			ncm_mset_catalog_remove_last_ensemble (esmcmc->mcat);
+
+			esmcmc->cur_sample_id -= esmcmc->nwalkers;
+			esmcmc->started        = FALSE;
+
+			ncm_fit_esmcmc_start_run (esmcmc);
+		}
+	}
+
   if (init_point_task)
     ncm_timer_task_pause (esmcmc->nt);
 }
@@ -962,6 +1167,8 @@ ncm_fit_esmcmc_end_run (NcmFitESMCMC *esmcmc)
     ncm_timer_task_end (esmcmc->nt);
 
   ncm_mset_catalog_sync (esmcmc->mcat, TRUE);
+  if (esmcmc->mtype > NCM_FIT_RUN_MSGS_NONE)
+    ncm_mset_catalog_log_current_stats (esmcmc->mcat);
   
   esmcmc->started = FALSE;
 }
@@ -1134,11 +1341,12 @@ _ncm_fit_esmcmc_mt_eval (glong i, glong f, gpointer data)
 static void
 _ncm_fit_esmcmc_get_jumps (NcmFitESMCMC *esmcmc, guint ki, guint kf)
 {
+  NcmRNG *rng = ncm_mset_catalog_peek_rng (esmcmc->mcat);
   guint k;
   
   for (k = ki; k < kf; k++)
   {
-    const gdouble jump = gsl_rng_uniform (esmcmc->mcat->rng->r);; 
+    const gdouble jump = gsl_rng_uniform (rng->r);; 
     ncm_vector_set (esmcmc->jumps, k, jump);
   }
 }
@@ -1147,8 +1355,9 @@ _ncm_fit_esmcmc_get_jumps (NcmFitESMCMC *esmcmc, guint ki, guint kf)
 static void
 _ncm_fit_esmcmc_run (NcmFitESMCMC *esmcmc)
 {
-  guint i;
+  NcmRNG *rng      = ncm_mset_catalog_peek_rng (esmcmc->mcat);
   gboolean mthread = (esmcmc->nthreads > 1);
+  guint i;
 
   if (mthread)
   {
@@ -1159,7 +1368,7 @@ _ncm_fit_esmcmc_run (NcmFitESMCMC *esmcmc)
     if (esmcmc->n > 0)
     {
       _ncm_fit_esmcmc_get_jumps (esmcmc, ki, esmcmc->nwalkers);
-      ncm_fit_esmcmc_walker_setup (esmcmc->walker, esmcmc->theta, ki, esmcmc->nwalkers, esmcmc->mcat->rng);
+      ncm_fit_esmcmc_walker_setup (esmcmc->walker, esmcmc->theta, ki, esmcmc->nwalkers, rng);
       
       if (ki < nwalkers_2)
       {
@@ -1179,7 +1388,7 @@ _ncm_fit_esmcmc_run (NcmFitESMCMC *esmcmc)
       for (i = 1; i < esmcmc->n; i++)
       {
         _ncm_fit_esmcmc_get_jumps (esmcmc, 0, esmcmc->nwalkers);
-        ncm_fit_esmcmc_walker_setup (esmcmc->walker, esmcmc->theta, 0, esmcmc->nwalkers, esmcmc->mcat->rng);
+        ncm_fit_esmcmc_walker_setup (esmcmc->walker, esmcmc->theta, 0, esmcmc->nwalkers, rng);
         
         ncm_func_eval_threaded_loop_full (&_ncm_fit_esmcmc_mt_eval, 0, nwalkers_2, esmcmc);
         ncm_func_eval_threaded_loop_full (&_ncm_fit_esmcmc_mt_eval, nwalkers_2, esmcmc->nwalkers, esmcmc);
@@ -1200,7 +1409,7 @@ _ncm_fit_esmcmc_run (NcmFitESMCMC *esmcmc)
     if (esmcmc->n > 0)
     {
       _ncm_fit_esmcmc_get_jumps (esmcmc, ki, esmcmc->nwalkers);
-      ncm_fit_esmcmc_walker_setup (esmcmc->walker, esmcmc->theta, ki, esmcmc->nwalkers, esmcmc->mcat->rng);
+      ncm_fit_esmcmc_walker_setup (esmcmc->walker, esmcmc->theta, ki, esmcmc->nwalkers, rng);
       
       if (ki < nwalkers_2)
       {
@@ -1220,7 +1429,7 @@ _ncm_fit_esmcmc_run (NcmFitESMCMC *esmcmc)
       for (i = 1; i < esmcmc->n; i++)
       {
         _ncm_fit_esmcmc_get_jumps (esmcmc, 0, esmcmc->nwalkers);
-        ncm_fit_esmcmc_walker_setup (esmcmc->walker, esmcmc->theta, 0, esmcmc->nwalkers, esmcmc->mcat->rng);
+        ncm_fit_esmcmc_walker_setup (esmcmc->walker, esmcmc->theta, 0, esmcmc->nwalkers, rng);
 
         _ncm_fit_esmcmc_mt_eval (0, nwalkers_2, esmcmc);
         _ncm_fit_esmcmc_mt_eval (nwalkers_2, esmcmc->nwalkers, esmcmc);
@@ -1257,36 +1466,55 @@ ncm_fit_esmcmc_run_lre (NcmFitESMCMC *esmcmc, guint prerun, gdouble lre)
   if (catlen < prerun)
   {
     guint prerun_left = prerun - catlen;
+    
     if (esmcmc->mtype >= NCM_FIT_RUN_MSGS_SIMPLE)
       g_message ("# NcmFitESMCMC: Running first %u pre-runs...\n", prerun_left);
+
     ncm_fit_esmcmc_run (esmcmc, prerun);
+    if (esmcmc->auto_trim)
+    {
+      ncm_mset_catalog_trim_by_type (esmcmc->mcat, esmcmc->auto_trim_div, esmcmc->trim_type, esmcmc->mtype);
+    }
   }
 
-  ncm_mset_catalog_estimate_autocorrelation_tau (esmcmc->mcat);
+  ncm_mset_catalog_estimate_autocorrelation_tau (esmcmc->mcat, FALSE);
   lerror = ncm_mset_catalog_largest_error (esmcmc->mcat);
 
   while (lerror > lre)
   {
     const gdouble lerror2 = lerror * lerror;
-    gdouble n = ncm_mset_catalog_len (esmcmc->mcat);
-    gdouble m = n * lerror2 / lre2;
-    guint runs = ((m - n) > 1000.0) ? ceil ((m - n) * 1.0e-1) : ceil (m - n);
-    guint ti = (esmcmc->cur_sample_id + 1) / esmcmc->nwalkers;
-    
-    runs = runs / esmcmc->nwalkers + 1;
+    gdouble n             = ncm_mset_catalog_len (esmcmc->mcat);
+    gdouble m             = n * lerror2 / lre2;
+    guint runs            = ((m - n) > 1000.0) ? MIN (ceil ((m - n) * 1.0e-1), 100000) : ceil (m - n);
+    guint ti              = (esmcmc->cur_sample_id + 1) / esmcmc->nwalkers;
 
+    runs = GSL_MIN (ncm_timer_task_estimate_by_time (esmcmc->nt, esmcmc->max_runs_time), runs);
+    runs = GSL_MAX (runs / esmcmc->nwalkers + 1, esmcmc->min_runs);
+    
     if (esmcmc->mtype >= NCM_FIT_RUN_MSGS_SIMPLE)
     {
+      gdouble glnvol;
+      const gdouble lnevol = ncm_mset_catalog_get_post_lnvol (esmcmc->mcat, 0.6827, &glnvol);
       g_message ("# NcmFitESMCMC: Largest relative error %e not attained: %e\n", lre, lerror);
+      g_message ("# NcmFitESMCMC: ln (eVol) = % 22.15g; ln (gVol) = % 22.15g; lnNorm = % 22.15g\n", lnevol, glnvol, ncm_mset_catalog_get_post_lnnorm (esmcmc->mcat));
       g_message ("# NcmFitESMCMC: Running more %u runs...\n", runs);
     }
+
     ncm_fit_esmcmc_run (esmcmc, ti + runs);
-    ncm_mset_catalog_estimate_autocorrelation_tau (esmcmc->mcat);
+    if (esmcmc->auto_trim)
+    {
+      ncm_mset_catalog_trim_by_type (esmcmc->mcat, esmcmc->auto_trim_div, esmcmc->trim_type, esmcmc->mtype);
+    }
+
+    ncm_mset_catalog_estimate_autocorrelation_tau (esmcmc->mcat, FALSE);
     lerror = ncm_mset_catalog_largest_error (esmcmc->mcat);
   }
 
   if (esmcmc->mtype >= NCM_FIT_RUN_MSGS_SIMPLE)
+  {
+    ncm_cfg_msg_sepa ();
     g_message ("# NcmFitESMCMC: Largest relative error %e attained: %e\n", lre, lerror);
+  }
 }
 
 /**
@@ -1298,12 +1526,29 @@ ncm_fit_esmcmc_run_lre (NcmFitESMCMC *esmcmc, guint prerun, gdouble lre)
 void
 ncm_fit_esmcmc_mean_covar (NcmFitESMCMC *esmcmc)
 {
+  NcmMSet *mset = ncm_mset_catalog_peek_mset (esmcmc->mcat);
+  
   ncm_mset_catalog_get_mean (esmcmc->mcat, &esmcmc->fit->fstate->fparams);
   ncm_mset_catalog_get_covar (esmcmc->mcat, &esmcmc->fit->fstate->covar);
-  ncm_mset_fparams_set_vector (esmcmc->mcat->mset, esmcmc->fit->fstate->fparams);
+  ncm_mset_fparams_set_vector (mset, esmcmc->fit->fstate->fparams);
+  
   esmcmc->fit->fstate->has_covar = TRUE;
 }
 
+/**
+ * ncm_fit_esmcmc_peek_ser:
+ * @esmcmc: a #NcmFitESMCMC
+ *
+ * Peeks the internal #NcmSerialize object from @esmcmc.
+ * 
+ * Returns: (transfer none): the internal #NcmSerialize object.
+ */
+NcmSerialize *
+ncm_fit_esmcmc_peek_ser (NcmFitESMCMC *esmcmc)
+{
+  return esmcmc->ser;
+}
+
 /**
  * ncm_fit_esmcmc_get_catalog:
  * @esmcmc: a #NcmFitESMCMC
@@ -1318,6 +1563,62 @@ ncm_fit_esmcmc_get_catalog (NcmFitESMCMC *esmcmc)
   return ncm_mset_catalog_ref (esmcmc->mcat);
 }
 
+static void 
+_ncm_fit_esmcmc_validade_mt_eval (glong i, glong f, gpointer data)
+{
+  NcmFitESMCMC *esmcmc        = NCM_FIT_ESMCMC (data);
+  NcmFitESMCMCWorker **fk_ptr = ncm_memory_pool_get (esmcmc->walker_pool);
+  NcmFit *fit_k               = fk_ptr[0]->fit;
+  glong k;
+
+  for (k = i; k < f; k++)
+  {
+    NcmVector *cur_row      = ncm_mset_catalog_peek_row (esmcmc->mcat, k);
+    const gdouble row_m2lnL = ncm_vector_get (cur_row, NCM_FIT_ESMCMC_M2LNL_ID);
+    gdouble m2lnL           = 0.0;
+    gdouble diff;
+    
+    g_assert (cur_row != NULL);
+
+    ncm_mset_fparams_set_vector_offset (fit_k->mset, cur_row, esmcmc->nadd_vals);
+
+    ncm_fit_m2lnL_val (fit_k, &m2lnL);
+
+    diff = fabs ((row_m2lnL - m2lnL) / row_m2lnL);
+    
+    if (diff > 1.0e-3)
+    {
+      if (esmcmc->mtype >= NCM_FIT_RUN_MSGS_SIMPLE)
+      {
+				g_mutex_lock (&esmcmc->resample_lock);
+        ncm_message ("# NcmFitESMCMC: Catalog row %5lu: m2lnL = %20.15g, recalculated to % 20.15g, diff = %8.5e <====== FAILED.\n",
+                     k, row_m2lnL, m2lnL, diff);
+				ncm_message ("# NcmFitESMCMC: row %5lu values: ", k);
+				ncm_vector_log_vals (cur_row, "", "%22.15g", TRUE);
+				g_mutex_unlock (&esmcmc->resample_lock);
+      }
+
+			g_mutex_lock (&esmcmc->update_lock);
+			g_array_index (esmcmc->accepted, gboolean, 0) = FALSE;
+			g_mutex_unlock (&esmcmc->update_lock);
+		}
+    else if (esmcmc->mtype >= NCM_FIT_RUN_MSGS_SIMPLE)
+    {
+			g_mutex_lock (&esmcmc->resample_lock);
+      ncm_message ("# NcmFitESMCMC: Catalog row %5lu: m2lnL = %20.15g, recalculated to % 20.15g, diff = %8.5e, SUCCEEDED!\n",
+                   k, row_m2lnL, m2lnL, diff);
+			if (esmcmc->mtype >= NCM_FIT_RUN_MSGS_FULL)
+			{
+				ncm_message ("# NcmFitESMCMC: row %5lu values:", k);
+				ncm_vector_log_vals (cur_row, "", "%22.15g", TRUE);
+			}
+			g_mutex_unlock (&esmcmc->resample_lock);
+    }
+  }
+
+  ncm_memory_pool_return (fk_ptr);
+}
+
 /**
  * ncm_fit_esmcmc_validate:
  * @esmcmc: a #NcmFitESMCMC
@@ -1328,59 +1629,39 @@ ncm_fit_esmcmc_get_catalog (NcmFitESMCMC *esmcmc)
  * with the values found in the catalog. This function
  * is particularly useful to check if any problem occured
  * during a multithread evaluation of the likelihood.
+ *
+ * Choosing @pf == 0 performs the validation from  @pi to the end.
  * 
  * Returns: Whether the validation was TRUE or FALSE.
  */
 gboolean
 ncm_fit_esmcmc_validate (NcmFitESMCMC *esmcmc, gulong pi, gulong pf)
 {
-  gulong k;
   gulong len;
-  gboolean valid = TRUE;
   
   ncm_mset_catalog_sync (esmcmc->mcat, TRUE);
 
   len = ncm_mset_catalog_len (esmcmc->mcat);
 
+	if (pf == 0)
+		pf = len;
+
   g_assert_cmpuint (pi, <, pf);
-  g_assert_cmpuint (pi, <, len);
-  g_assert_cmpuint (pf, <, len);
+  g_assert_cmpuint (pf, <=, len);
 
-  if (esmcmc->mtype > NCM_FIT_RUN_MSGS_SIMPLE || TRUE)
+  if (esmcmc->mtype >= NCM_FIT_RUN_MSGS_SIMPLE)
   {
-    ncm_fit_log_info (esmcmc->fit);
-  }
-  
-  for (k = pi; k < pf; k++)
-  {
-    NcmVector *cur_row      = ncm_mset_catalog_peek_row (esmcmc->mcat, k);
-    const gdouble row_m2lnL = ncm_vector_get (cur_row, NCM_FIT_ESMCMC_M2LNL_ID);
-    gdouble m2lnL           = 0.0;
-    gdouble diff;
-    
-    g_assert (cur_row != NULL);
+    ncm_cfg_msg_sepa ();
+    g_message ("# NcmFitESMCMC: validating catalog rows [%lu, %lu)\n", pi, pf);
+		if (esmcmc->mtype >= NCM_FIT_RUN_MSGS_FULL)
+			ncm_fit_log_info (esmcmc->fit);
+	}
 
-    ncm_mset_fparams_set_vector_offset (esmcmc->fit->mset, cur_row, esmcmc->nadd_vals);
+	g_array_index (esmcmc->accepted, gboolean, 0) = TRUE;
+  if (esmcmc->nthreads > 1)
+    ncm_func_eval_threaded_loop_full (&_ncm_fit_esmcmc_validade_mt_eval, pi, pf, esmcmc);
+  else
+    _ncm_fit_esmcmc_validade_mt_eval (pi, pf, esmcmc);
 
-    ncm_fit_m2lnL_val (esmcmc->fit, &m2lnL);
-
-    diff = fabs ((row_m2lnL - m2lnL)/row_m2lnL);
-    
-    if (diff > 1.0e-3)
-    {
-      valid = FALSE;
-      if (esmcmc->mtype > NCM_FIT_RUN_MSGS_SIMPLE)
-      {
-        ncm_message ("# Catalog row %5lu: m2lnL = %20.15g, recalculated to % 20.15g, diff = %8.5e <====== FAIL.\n",
-                     k, row_m2lnL, m2lnL, diff);
-      }
-    }
-    else if (esmcmc->mtype > NCM_FIT_RUN_MSGS_SIMPLE)
-    {
-      ncm_message ("# Catalog row %5lu: m2lnL = %20.15g, recalculated to % 20.15g, diff = %8.5e PASSED.\n",
-                   k, row_m2lnL, m2lnL, diff);
-    }
-  }
-  
-  return valid;
+  return g_array_index (esmcmc->accepted, gboolean, 0);
 }
