@@ -34,270 +34,6 @@
  * models. Second, each child of NcmModel can register itself as a model type.
  * This allows multiples models types to be used simultaneously.
  *
- * For example, in a problem where one must describe some physical model and
- * some model for the measurement tool, lets say FooPhysical model and FooTool
- * both defined in the Foo namespace.
- *
- * The FooPhysical and FooTools define virtual functions called respectively
- * foo_physical_value and foo_tool_value which calculates some quantities needed
- * to compare the model with data. Note that both models do not implement
- * anything thay just define which virtual functions must be implemented in
- * order to define a model of each type.
- *
- * Here is an example of the definition of the FooPhysical model type, note that
- * most of it is just the usual #GObject framework boilerplate code.
- *
- * # Implementing a NcmModel (header: foo_physical.h). #
- *  |[<!-- language="C" -->
- *
- *  #include <glib.h>
- *  #include <numcosmo/numcosmo.h> // Including NumCosmo headers
- *
- *  #define FOO_TYPE_PHYSICAL             (foo_physical_get_type ())
- *  #define FOO_PHYSICAL(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), FOO_TYPE_PHYSICAL, FooPhysical))
- *  #define FOO_PHYSICAL_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), FOO_TYPE_PHYSICAL, FooPhysicalClass))
- *  #define FOO_IS_PHYSICAL(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FOO_TYPE_PHYSICAL))
- *  #define FOO_IS_PHYSICAL_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), FOO_TYPE_PHYSICAL))
- *  #define FOO_PHYSICAL_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), NCM_TYPE_PHYSICAL, FooPhysicalClass))
- *
- *  // The lines above are just the basic GObject definition macros
- *
- *  typedef struct _FooPhysicalClass FooPhysicalClass;
- *  typedef struct _FooPhysical FooPhysical;
- *
- *  // To avoid a gtkdoc bug always declare the instance struct before the class
- *  // struct
- *  struct _FooPhysical
- *  {
- *    NcmModel parent_instance;
- *    ...
- *  };
- *
- *  struct _FooPhysicalClass
- *  {
- *    NcmModelClass parent_class;
- *    gdouble (*value) (NcmModel *model); // Virtual function.
- *    ...
- *  };
- *
- *  ...
- *
- *  GType foo_physical_get_type (void) G_GNUC_CONST;
- *  NCM_MSET_MODEL_DECLARE_ID (foo_physical);
- *  // The last line above is part of the model type registry.
- *
- *  gdouble foo_physical_value (FooPhysical *phys); // The virtual function caller.
- *
- * ]|
- *
- * # Implementing a NcmModel (source: foo_physical.c). #
- * |[<!-- language="C" -->
- *
- *  #include "foo_physical.h"
- *  G_DEFINE_ABSTRACT_TYPE (FooPhysical, foo_physical, NCM_TYPE_MODEL);
- *
- *  static void
- *  foo_physical_init (FooPhysical *object)
- *  {
- *    ... // Some basic initialization for an instance.
- *  }
- *
- *  static void
- *  _foo_physical_finalize (GObject *object)
- *  {
- *
- *    ... // Instance finalization, must deallocate all memory and release all references.
- *
- *    // Chain up : end
- *    G_OBJECT_CLASS (foo_physical_parent_class)->finalize (object);
- *  }
- *
- *  static gboolean _foo_physical_valid (NcmModel *model); // Define if the model is valid
- *  NCM_MSET_MODEL_REGISTER_ID (foo_physical, FOO_TYPE_PHYSICAL); // Second part of the model type registry.
- *
- *  static void
- *  foo_physical_class_init (FooPhysicalClass *klass)
- *  {
- *    GObjectClass* object_class = G_OBJECT_CLASS (klass);
- *    NcmModelClass *model_class = NCM_MODEL_CLASS (klass);
- *
- *    object_class->finalize = &_foo_physical_finalize;
- *
- *    // It is a base model type definition so no parameters.
- *    ncm_model_class_add_params (model_class, 0, 0, 1);
- *
- *    // Define its id short and long description.
- *    ncm_mset_model_register_id (model_class,
- *                                "FooPhysical",
- *                                "Some brief description.",
- *                                "Some long description.");
- *
- *    // Checks if everything is consistent.
- *    ncm_model_class_check_params_info (model_class);
- *
- *    // Sets the "valid" function.
- *    model_class->valid = &_foo_physical_valid;
- *  }
- *
- *  static gboolean
- *  _foo_physical_valid (NcmModel *model)
- *  {
- *    if (!NCM_MODEL_CLASS (foo_physical_parent_class)->valid (model))
- *      return FALSE;
- *    // Chain up : start
- *
- *    ... // returns TRUE if value and FALSE if not.
- *
- *  }
- *
- *  ...
- *
- *  // Virtual function accessor.
- *  G_INLINE_FUNC gdouble foo_physical_value (FooPhysical *phys) \
- *  {
- *    return FOO_PHYSICAL_GET_CLASS (m)->value (NCM_MODEL (phys)); \
- *  }
- *
- *  ...
- *
- * ]|
- *
- * Now it is possible to give different implementations of each model, e.g.,
- * FooPhysicalSimple could be a simple implementation of the physical model
- * taking into account only the main effects and containing only a few parameters.
- * Then, FooPhysicalComplex a more complex implementation including several
- * effects and usually containing several parameters. Suppose that the same
- * applies to FooToolSimple and FooToolComplex.
- *
- * Here an example of a FooPhysical implementation:
- *
- * # Implementing a FooPhysical (header: foo_physical_simple.h). #
- * |[<!-- language="C" -->
- *
- *  #include <glib.h>
- *  #include <numcosmo/numcosmo.h> // Including NumCosmo headers
- *  #include "foo_physical.h"
- *
- *  #define NCM_TYPE_PHYSICAL_SIMPLE             (foo_physical_simple_get_type ())
- *  #define FOO_PHYSICAL_SIMPLE(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), NCM_TYPE_PHYSICAL_SIMPLE, FooPhysicalSimple))
- *  #define FOO_PHYSICAL_SIMPLE_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), NCM_TYPE_PHYSICAL_SIMPLE, FooPhysicalSimpleClass))
- *  #define FOO_IS_PHYSICAL_SIMPLE(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NCM_TYPE_PHYSICAL_SIMPLE))
- *  #define FOO_IS_PHYSICAL_SIMPLE_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), NCM_TYPE_PHYSICAL_SIMPLE))
- *  #define FOO_PHYSICAL_SIMPLE_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), NCM_TYPE_PHYSICAL_SIMPLE, FooPhysicalSimpleClass))
- *
- *  // The lines above are just the basic GObject definition macros
- *
- *  typedef struct _FooPhysicalSimpleClass FooPhysicalSimpleClass;
- *  typedef struct _FooPhysicalSimple FooPhysicalSimple;
- *
- *  // To avoid a gtkdoc bug always declare the instance struct before the class
- *  // struct
- *  struct _FooPhysicalSimple
- *  {
- *    FooPhysical parent_instance; // Note that it is now a child of FooPhysical
- *    ...
- *  };
- *
- *  struct _FooPhysicalSimpleClass
- *  {
- *    FooPhysicalClass parent_class; // Note that it is now a child of FooPhysical
- *    ...
- *  };
- *
- *  ...
- *
- *  GType foo_physical_simple_get_type (void) G_GNUC_CONST;
- *  // Note that we do not registry FooPhysicalSimple as it is of FooPhysical type.
- *
- *  ]|
- *
- * # Implementing a FooPhysical (source: foo_physical_simple.c). #
- * |[<!-- language="C" -->
- *
- *  #include "foo_physical_simple.h"
- *  G_DEFINE_ABSTRACT_TYPE (FooPhysicalSimple, foo_physical_simple, NCM_TYPE_PHYSICAL);
- *
- *  static void
- *  foo_physical_simple_init (FooPhysicalSimple *object)
- *  {
- *    ... // Some basic initialization for an instance.
- *  }
- *
- *  static void
- *  _foo_physical_simple_finalize (GObject *object)
- *  {
- *
- *    ... // Instance finalization, must deallocate all memory and release all references.
- *
- *    // Chain up : end
- *    G_OBJECT_CLASS (foo_physical_simple_parent_class)->finalize (object);
- *  }
- *
- *  // Again, we do not need to register this model.
- *
- *  // Here we need to define what FooPhysicalSimple actually calculates.
- *  gdouble _foo_physical_simple_value (NcmModel *model);
- *
- *  static void
- *  foo_physical_simple_class_init (FooPhysicalSimpleClass *klass)
- *  {
- *    GObjectClass* object_class = G_OBJECT_CLASS (klass);
- *    NcmModelClass *model_class = NCM_MODEL_CLASS (klass);
- *    FooPhysicalClass *physical_class = FOO_PHYSICAL_CLASS (klass);
- *
- *    object_class->finalize = &_foo_physical_finalize;
- *
- *    physical_class->value = &_foo_physical_simple_value;
- *
- *    // It is simple model so lets say is has one (01) scalar parameter and
- *    // zero vector parameters.
- *    ncm_model_class_add_params (model_class, 1, 0, 1);
- *
- *    // Now we set the name and nick for our model.
- *    ncm_model_class_set_name_nick (model_class, "Physical simple model", "PSimple");
- *
- *    // Our parameter number 0 is p0 with its symbol being p_0, it can varies
- *    // from -10.0 to 10.0 and its natural varying scale is 0.1
- *    // its absolute tolerance is 0 (no absolute tolerance) and its default
- *    // value is 2.0 and by default it should be set free in statistical
- *    // analysis.
- *    ncm_model_class_set_sparam (model_class, 0, "p_0", "p0",
- *                                -10.0, 10.0, 0.1,
- *                                0.0, 1.0,
- *                                NCM_PARAM_TYPE_FREE);
- *
- *
- *    // Checks if everything is consistent.
- *    ncm_model_class_check_params_info (model_class);
- *
- *  }
- *
- *  ...
- *
- *  // The virtual function implementation.
- *  gdouble
- *  _foo_physical_simple_value (NcmModel *model)
- *  {
- *    const gdouble p0 = ncm_vector_get (model->params, 0); // Get the value of the parameter p0 to perform calculations.
- *
- *    ...
- *
- *    return ...; // Return the "value".
- *  }
- *
- *  ...
- *
- * ]|
- *
- * Note that, we can now build a #NcmData to compare data with these models
- * (FooPhysical and FooTool), in this way NcmData will only depend on these
- * two classes and not on their implementations, i.e., inside NcmData we only
- * call foo_physical_value and foo_tool_value which are independent of their
- * implementations. We can then build a #NcmMSet using any combination of
- * FooPhysicalSimple or FooPhysicalComplex and FooToolSimple or FooToolComplex,
- * to find their bestfit or make other statistical analysis (see #NcmFit and
- * related objects).
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -511,7 +247,7 @@ _ncm_model_get_property (GObject *object, guint prop_id, GValue *value, GParamSp
       g_value_set_uint (value, model_class->vparam_len);
       break;
     case PROP_IMPLEMENTATION:
-      g_value_set_uint64 (value, model_class->impl);
+      g_value_set_uint64 (value, model_class->impl_flag);
       break;
     case PROP_REPARAM:
       g_value_set_object (value, model->reparam);
@@ -751,7 +487,8 @@ ncm_model_class_set_property (GObject *object, guint prop_id, const GValue *valu
   else if (vparam_len_id < model_class->vparam_len)
   {
     NcmModelClass *model_class = NCM_MODEL_GET_CLASS (model);
-    guint psize = g_value_get_uint (value);
+    guint psize                = g_value_get_uint (value);
+
     if (model->vparam_len->len == 0)
     {
       g_array_set_size (model->vparam_len, model_class->vparam_len);
@@ -772,7 +509,7 @@ ncm_model_class_set_property (GObject *object, guint prop_id, const GValue *valu
   else if (vparam_fit_id < model_class->vparam_len)
   {
     GVariant *var = g_value_get_variant (value);
-    gsize n = g_variant_n_children (var);
+    gsize n  = g_variant_n_children (var);
     gsize nv = g_array_index (model->vparam_len, guint, vparam_fit_id);
     guint i;
 
@@ -796,7 +533,9 @@ ncm_model_class_set_property (GObject *object, guint prop_id, const GValue *valu
       }
     }
     else if (n != nv)
-      g_error ("set_property: cannot set fit type of vector parameter, variant contains %zu childs but vector dimension is %u", n, g_array_index (model->vparam_len, guint, vparam_fit_id));
+    {
+      g_error ("set_property: cannot set fit type of vector parameter, variant contains %zu children but vector dimension is %u", n, g_array_index (model->vparam_len, guint, vparam_fit_id));
+    }
     else
     {
       if (g_variant_is_of_type (var, G_VARIANT_TYPE ("ab")))
@@ -985,10 +724,10 @@ ncm_model_class_set_vparam_obj (NcmModelClass *model_class, guint vparam_id, Ncm
                                                         NCM_TYPE_VECTOR,
                                                         G_PARAM_READWRITE));
   {
-    gchar *param_length_name = g_strdup_printf ("%s-length", ncm_vparam_name (vparam));
+    gchar *param_length_name   = g_strdup_printf ("%s-length", ncm_vparam_name (vparam));
     gchar *param_length_symbol = g_strdup_printf ("%s:length", ncm_vparam_symbol (vparam));
-    gchar *param_fit_name = g_strdup_printf ("%s-fit", ncm_vparam_name (vparam));
-    gchar *param_fit_symbol = g_strdup_printf ("%s:fit", ncm_vparam_symbol (vparam));
+    gchar *param_fit_name      = g_strdup_printf ("%s-fit", ncm_vparam_name (vparam));
+    gchar *param_fit_symbol    = g_strdup_printf ("%s:fit", ncm_vparam_symbol (vparam));
 
     g_object_class_install_property (object_class, prop_len_id,
                                      g_param_spec_uint (param_length_name,
@@ -1115,6 +854,47 @@ ncm_model_class_check_params_info (NcmModelClass *model_class)
   }
 }
 
+/**
+ * ncm_model_class_add_impl_opts: (skip)
+ * @model_class: a #NcmModelClass
+ * @opt1: first option
+ * @...: other options, must end with -1
+ * 
+ * FIXME
+ *
+ */
+void 
+ncm_model_class_add_impl_opts (NcmModelClass *model_class, gint opt1, ...)
+{
+  gint opt_i;
+  va_list ap;
+
+  va_start (ap, opt1);
+
+  model_class->impl_flag = model_class->impl_flag | NCM_MODEL_OPT2IMPL (opt1);
+  
+  while ((opt_i = va_arg (ap, gint)) != -1)
+  {
+    model_class->impl_flag = model_class->impl_flag | NCM_MODEL_OPT2IMPL (opt_i);
+  }
+
+  va_end (ap);
+}
+
+/**
+ * ncm_model_class_add_impl_flag: (skip)
+ * @model_class: a #NcmModelClass
+ * @flag: implementation flag
+ * 
+ * FIXME
+ *
+ */
+void 
+ncm_model_class_add_impl_flag (NcmModelClass *model_class, guint64 flag)
+{
+  model_class->impl_flag = model_class->impl_flag | flag;
+}
+
 /**
  * ncm_model_dup:
  * @model: a #NcmModel
@@ -1522,6 +1302,57 @@ ncm_model_params_valid_bounds (NcmModel *model)
  *
  * Returns: FIXME
  */
+/**
+ * ncm_model_check_impl_flag:
+ * @model: a #NcmModel
+ * @impl: implementation flag
+ *
+ * FIXME
+ *
+ * Returns: FIXME
+ */
+/**
+ * ncm_model_check_impl_opt:
+ * @model: a #NcmModel
+ * @opt: implementation option
+ *
+ * FIXME
+ *
+ * Returns: FIXME
+ */
+
+/**
+ * ncm_model_check_impl_opts:
+ * @model: a #NcmModel
+ * @opt1: first implementation option
+ * @...: implementation options, must end with -1
+ *
+ * FIXME
+ *
+ * Returns: FIXME
+ */
+gboolean 
+ncm_model_check_impl_opts (NcmModel *model, gint opt1, ...)
+{
+  guint64 flag = 0;
+  gint opt_i;
+  
+  va_list ap;
+
+  va_start (ap, opt1);
+
+  flag = flag | NCM_MODEL_OPT2IMPL (opt1);
+  
+  while ((opt_i = va_arg (ap, gint)) != -1)
+  {
+    flag = flag | NCM_MODEL_OPT2IMPL (opt_i);
+  }
+
+  va_end (ap);
+
+  return ncm_model_check_impl_flag (model, flag);
+}
+
 /**
  * ncm_model_len:
  * @model: a #NcmModel
@@ -1545,6 +1376,13 @@ ncm_model_params_valid_bounds (NcmModel *model)
  * FIXME
  *
  */
+/**
+ * ncm_model_state_mark_outdated:
+ * @model: a #NcmModel
+ *
+ * FIXME
+ *
+ */
 /**
  * ncm_model_sparam_len:
  * @model: a #NcmModel
