@@ -31,8 +31,10 @@
 #include <numcosmo/math/ncm_reparam.h>
 #include <numcosmo/math/ncm_model.h>
 
+#ifndef NUMCOSMO_GIR_SCAN
 #include <gsl/gsl_linalg.h>
 #include <gsl/gsl_blas.h>
+#endif /* NUMCOSMO_GIR_SCAN */
 
 G_BEGIN_DECLS
 
