@@ -43,6 +43,7 @@
 #include "math/ncm_vector.h"
 #include "math/ncm_matrix.h"
 #include "ncm_enum_types.h"
+
 #include <gio/gio.h>
 
 enum
@@ -225,37 +226,79 @@ ncm_serialize_clear (NcmSerialize **ser)
   g_clear_object (ser);
 }
 
+static gboolean
+_ncm_serialize_reset_remove_autosaved_name_key (gpointer key, gpointer value, gpointer user_data)
+{
+  gchar *name = (gchar *) key;
+  return g_regex_match_simple ("^S[\\d]+$", name, 0, 0);
+}
+
+static gboolean
+_ncm_serialize_reset_remove_autosaved_name_val (gpointer key, gpointer value, gpointer user_data)
+{
+  gchar *name = (gchar *) value;
+  return g_regex_match_simple ("^S[\\d]+$", name, 0, 0);
+}
+
 /**
  * ncm_serialize_reset:
- * @ser: a #NcmSerialize.
+ * @ser: a #NcmSerialize
+ * @autosave_only: a boolean
  *
  * Releases all objects in @ser and erase all serialized
- * objects.
+ * objects. If @autosave_only is TRUE it will release only
+ * autosaved objects.
  *
  */
 void
-ncm_serialize_reset (NcmSerialize *ser)
+ncm_serialize_reset (NcmSerialize *ser, gboolean autosave_only)
 {
-  ncm_serialize_clear_instances (ser);
+  ncm_serialize_clear_instances (ser, autosave_only);
 
-  g_hash_table_remove_all (ser->saved_ptr_name);
-  g_hash_table_remove_all (ser->saved_name_ser);
+  if (autosave_only)
+  {
+    g_hash_table_foreach_remove (ser->saved_ptr_name, 
+                                 &_ncm_serialize_reset_remove_autosaved_name_val, 
+                                 NULL);    
+    g_hash_table_foreach_remove (ser->saved_name_ser, 
+                                 &_ncm_serialize_reset_remove_autosaved_name_key, 
+                                 NULL);
+  }
+  else
+  {
+    g_hash_table_remove_all (ser->saved_ptr_name);
+    g_hash_table_remove_all (ser->saved_name_ser);
+  }
 
   ser->autosave_count = 0;
 }
 
 /**
  * ncm_serialize_clear_instances:
- * @ser: a #NcmSerialize.
+ * @ser: a #NcmSerialize
+ * @autosave_only: a boolean
  *
- * Releases all objects in @ser.
+ * Releases all objects in @ser. If @autosave_only is TRUE 
+ * it will release only autosaved objects.
  *
  */
 void
-ncm_serialize_clear_instances (NcmSerialize *ser)
+ncm_serialize_clear_instances (NcmSerialize *ser, gboolean autosave_only)
 {
-  g_hash_table_remove_all (ser->name_ptr);
-  g_hash_table_remove_all (ser->ptr_name);
+  if (autosave_only)
+  {
+    g_hash_table_foreach_remove (ser->ptr_name, 
+                                 &_ncm_serialize_reset_remove_autosaved_name_val, 
+                                 NULL);    
+    g_hash_table_foreach_remove (ser->name_ptr, 
+                                 _ncm_serialize_reset_remove_autosaved_name_key, 
+                                 NULL);
+  }
+  else
+  {
+    g_hash_table_remove_all (ser->name_ptr);
+    g_hash_table_remove_all (ser->ptr_name);
+  }
 }
 
 /**
@@ -321,6 +364,20 @@ ncm_serialize_count_instances (NcmSerialize *ser)
   return g_hash_table_size (ser->name_ptr);
 }
 
+/**
+ * ncm_serialize_count_saved_serializations:
+ * @ser: a #NcmSerialize.
+ *
+ * Counts the number of instances registered in @ser.
+ *
+ * Returns: the number of instances in @ser.
+ */
+guint
+ncm_serialize_count_saved_serializations (NcmSerialize *ser)
+{
+  return g_hash_table_size (ser->saved_name_ser);
+}
+
 /**
  * ncm_serialize_peek_by_name:
  * @ser: a #NcmSerialize.
@@ -374,7 +431,10 @@ ncm_serialize_peek_name (NcmSerialize *ser, gpointer obj)
 {
   g_assert (G_IS_OBJECT (obj));
   if (!ncm_serialize_contain_instance (ser, obj))
+  {
     g_error ("ncm_serialize_peek_name: Cannot peek name of object %p, it is not a named instance.", obj);
+    return NULL;
+  }
   else
     return g_hash_table_lookup (ser->ptr_name, obj);
 }
@@ -755,8 +815,8 @@ ncm_serialize_from_string (NcmSerialize *ser, const gchar *obj_ser)
 
 /**
  * ncm_serialize_from_file:
- * @ser: a #NcmSerialize.
- * @filename: File containing the serialized version of the object.
+ * @ser: a #NcmSerialize
+ * @filename: File containing the serialized version of the object
  *
  * Parses the serialized string in @filename and returns the newly created object.
  *
@@ -772,7 +832,7 @@ ncm_serialize_from_file (NcmSerialize *ser, const gchar *filename)
 
   g_assert (filename != NULL);
   if (!g_file_get_contents (filename, &file, &length, &error))
-    g_error ("_nc_data_snia_cov_load_matrix: cannot open file %s: %s",
+    g_error ("ncm_serialize_from_file: cannot open file %s: %s",
              filename, error->message);
 
   g_assert_cmpint (length, >, 0);
@@ -784,6 +844,48 @@ ncm_serialize_from_file (NcmSerialize *ser, const gchar *filename)
   return obj;
 }
 
+/**
+ * ncm_serialize_from_binfile:
+ * @ser: a #NcmSerialize.
+ * @filename: File containing the binary serialized version of the object.
+ *
+ * Parses the serialized binary data in @filename and returns the newly created object.
+ *
+ * Returns: (transfer full): A new #GObject.
+ */
+GObject *
+ncm_serialize_from_binfile (NcmSerialize *ser, const gchar *filename)
+{
+  GError *error = NULL;
+  gchar *file   = NULL;
+  gsize length  = 0;
+  GObject *obj  = NULL;
+
+  g_assert (filename != NULL);
+  
+  if (!g_file_get_contents (filename, &file, &length, &error))
+    g_error ("_nc_data_snia_cov_load_matrix: cannot open file %s: %s",
+             filename, error->message);
+
+  g_assert_cmpint (length, >, 0);
+
+  {
+    GVariant *obj_ser = g_variant_new_from_data (G_VARIANT_TYPE (NCM_SERIALIZE_OBJECT_TYPE),
+                                                 file,
+                                                 length,
+                                                 TRUE,
+                                                 g_free,
+                                                 file
+                                                 );
+
+    obj = ncm_serialize_from_variant (ser, obj_ser);
+
+    g_variant_unref (obj_ser);
+  }
+
+  return obj;
+}
+
 /**
  * ncm_serialize_from_name_params:
  * @ser: a #NcmSerialize.
@@ -817,8 +919,17 @@ ncm_serialize_from_name_params (NcmSerialize *ser, const gchar *obj_name, GVaria
 
   g_match_info_free (match_info);
 
-  if ((name != NULL) && ncm_serialize_contain_name (ser, name))
-    obj = ncm_serialize_get_by_name (ser, name);
+  if (name != NULL)
+  {
+/*
+    if (g_hash_table_lookup (ser->saved_name_ser, name) != NULL)
+      g_error ("ncm_serialize_from_name_params: deserializing object named `%s' but it is present in the list of saved serializations.",
+               name);
+    else 
+*/
+    if (ncm_serialize_contain_name (ser, name))
+      obj = ncm_serialize_get_by_name (ser, name);
+  }
 
   if (obj != NULL)
   {
@@ -912,7 +1023,25 @@ ncm_serialize_from_name_params (NcmSerialize *ser, const gchar *obj_name, GVaria
       g_variant_unref (var);
     }
 
+#if GLIB_CHECK_VERSION(2,54,0)
+    {
+      const gchar **names = g_new (const gchar *, nprop);
+      GValue *values      = g_new (GValue, nprop);
+  
+      for (i = 0; i < nprop; i++)
+      {
+        names[i]  = gprop[i].name;
+        values[i] = gprop[i].value;
+      }
+  
+      obj = g_object_new_with_properties (gtype, nprop, names, values);
+
+      g_free (names);
+      g_free (values);
+    }
+#else
     obj = g_object_newv (gtype, nprop, gprop);
+#endif /* GLIB_CHECK_VERSION(2,54,0) */
     for (i = 0; i < nprop; i++)
       g_value_unset (&gprop[i].value);
 
@@ -1153,12 +1282,11 @@ ncm_serialize_to_variant (NcmSerialize *ser, GObject *obj)
       name = g_strdup_printf (NCM_SERIALIZE_AUTOSAVE_NAME NCM_SERIALIZE_AUTOSAVE_NFORMAT, ser->autosave_count);
 
       obj_name = g_strdup_printf ("%s[%s]", tmp, name);
-
       g_free (tmp);
 
       ser->autosave_count++;
     }
-    
+
     if (n_properties == 0)
     {
       g_free (prop);
@@ -1178,7 +1306,7 @@ ncm_serialize_to_variant (NcmSerialize *ser, GObject *obj)
 
         if ((prop[i]->flags & G_PARAM_READWRITE) != G_PARAM_READWRITE)
           continue;
-
+        
         g_value_init (&val, prop[i]->value_type);
         g_object_get_property (obj, prop[i]->name, &val);
 
@@ -1282,6 +1410,30 @@ ncm_serialize_to_file (NcmSerialize *ser, GObject *obj, const gchar *filename)
   g_free (obj_ser);
 }
 
+/**
+ * ncm_serialize_to_binfile:
+ * @ser: a #NcmSerialize
+ * @obj: a #GObject
+ * @filename: File where to save the serialized version of the object
+ * 
+ * Serializes @obj and saves the binary in @filename.
+ * 
+ */
+void
+ncm_serialize_to_binfile (NcmSerialize *ser, GObject *obj, const gchar *filename)
+{
+  GError *error     = NULL;
+  GVariant *obj_ser = ncm_serialize_to_variant (ser, obj);
+  gsize length      = g_variant_get_size (obj_ser);
+
+  g_assert (filename != NULL);
+
+  if (!g_file_set_contents (filename, g_variant_get_data (obj_ser), length, &error))
+    g_error ("ncm_serialize_to_file: cannot save to file %s: %s",
+             filename, error->message);
+
+  g_variant_unref (obj_ser);
+}
 
 /**
  * ncm_serialize_dup_obj:
@@ -1324,30 +1476,32 @@ ncm_serialize_global (void)
 
 /**
  * ncm_serialize_global_reset:
+ * @autosave_only: a boolean
  *
  * Releases all objects in global #NcmSerialize and erase
  * all serialized objects.
  *
  */
 void
-ncm_serialize_global_reset (void)
+ncm_serialize_global_reset (gboolean autosave_only)
 {
   NcmSerialize *ser = ncm_serialize_global ();
-  ncm_serialize_reset (ser);
+  ncm_serialize_reset (ser, autosave_only);
   ncm_serialize_unref (ser);
 }
 
 /**
  * ncm_serialize_global_clear_instances:
+ * @autosave_only: a boolean
  *
  * Releases all objects in global #NcmSerialize.
  *
  */
 void
-ncm_serialize_global_clear_instances (void)
+ncm_serialize_global_clear_instances (gboolean autosave_only)
 {
   NcmSerialize *ser = ncm_serialize_global ();
-  ncm_serialize_clear_instances (ser);
+  ncm_serialize_clear_instances (ser, autosave_only);
   ncm_serialize_unref (ser);
 }
 
@@ -1415,6 +1569,22 @@ ncm_serialize_global_count_instances (void)
   return ret;
 }
 
+/**
+ * ncm_serialize_global_count_saved_serializations:
+ *
+ * Global version of ncm_serialize_count_saved_serializations().
+ *
+ * Returns: the number of instances in @ser.
+ */
+guint
+ncm_serialize_global_count_saved_serializations (void)
+{
+  NcmSerialize *ser = ncm_serialize_global ();
+  guint ret = ncm_serialize_count_saved_serializations (ser);
+  ncm_serialize_unref (ser);
+  return ret;
+}
+
 /**
  * ncm_serialize_global_get_by_name:
  * @name: an instance name.
@@ -1598,6 +1768,23 @@ ncm_serialize_global_from_file (const gchar *filename)
   return ret;
 }
 
+/**
+ * ncm_serialize_global_from_binfile:
+ * @filename: File containing the serialized version of the object.
+ *
+ * Global version of ncm_serialize_from_binfile().
+ *
+ * Returns: (transfer full): A new #GObject.
+ */
+GObject *
+ncm_serialize_global_from_binfile (const gchar *filename)
+{
+  NcmSerialize *ser = ncm_serialize_global ();
+  GObject *ret = ncm_serialize_from_binfile (ser, filename);
+  ncm_serialize_unref (ser);
+  return ret;
+}
+
 /**
  * ncm_serialize_global_from_name_params:
  * @obj_name: string containing the object name.
@@ -1684,6 +1871,22 @@ ncm_serialize_global_to_file (GObject *obj, const gchar *filename)
   ncm_serialize_unref (ser);
 }
 
+/**
+ * ncm_serialize_global_to_binfile:
+ * @obj: a #GObject.
+ * @filename: File where to save the serialized version of the object
+ * 
+ * Global version of ncm_serialize_to_binfile().
+ * 
+ */
+void
+ncm_serialize_global_to_binfile (GObject *obj, const gchar *filename)
+{
+  NcmSerialize *ser = ncm_serialize_global ();
+  ncm_serialize_to_binfile (ser, obj, filename);
+  ncm_serialize_unref (ser);
+}
+
 /**
  * ncm_serialize_global_dup_obj:
  * @obj: a #GObject.
