@@ -365,6 +365,12 @@ ncm_spline_set (NcmSpline *s, NcmVector *xv, NcmVector *yv, gboolean init)
 	if (init)
 		ncm_spline_prepare (s);
 
+  if (s->acc != NULL)
+  {
+    ncm_spline_acc (s, FALSE);
+    ncm_spline_acc (s, TRUE);
+  }
+
 	return s;
 }
 
@@ -564,7 +570,7 @@ NcmVector *
 ncm_spline_get_xv (NcmSpline *s)
 {
 	if (s->xv != NULL)
-    return g_object_ref (s->xv);
+    return ncm_vector_ref (s->xv);
   else
     return NULL;
 }
@@ -581,7 +587,7 @@ NcmVector *
 ncm_spline_get_yv (NcmSpline *s)
 {
   if (s->yv != NULL)
-    return g_object_ref (s->yv);
+    return ncm_vector_ref (s->yv);
   else
     return NULL;
 }
