@@ -26,9 +26,9 @@
 /**
  * SECTION:ncm_stats_dist1d_spline
  * @title: NcmStatsDist1dSpline
- * @short_description: One dimensional probability distribution based on a spline.
+ * @short_description: One dimensional probability distribution based on a spline
  * 
- * FIXME
+ * Reconstruction of an arbitrary one dimensional probability distribution based on a spline.
  * 
  */
 
@@ -44,6 +44,7 @@ enum
   PROP_0,
   PROP_M2LNP,
   PROP_TAIL_SIGMA,
+	PROP_SIZE,
 };
 
 G_DEFINE_TYPE (NcmStatsDist1dSpline, ncm_stats_dist1d_spline, NCM_TYPE_STATS_DIST1D);
