@@ -1,3 +1,4 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 2 -*-  */
 /***************************************************************************
  *            ncm_util.h
  *
@@ -29,13 +30,13 @@
 #include <glib-object.h>
 #include <numcosmo/build_cfg.h>
 
+#ifndef NUMCOSMO_GIR_SCAN
 #include <gsl/gsl_rng.h>
 #include <gsl/gsl_min.h>
-#ifndef NUMCOSMO_GIR_SCAN
 #include <complex.h>
 #include <gmp.h>
 #include <mpfr.h>
-#endif
+#endif /* NUMCOSMO_GIR_SCAN */
 
 G_BEGIN_DECLS
 
@@ -43,12 +44,16 @@ gdouble *ncm_smoothd (gdouble *in, size_t N, size_t points, size_t pass);
 gdouble ncm_topology_comoving_a0_lss (guint n, gdouble alpha);
 gdouble ncm_topology_sigma_comoving_a0_lss (guint n, gdouble alpha, gdouble sigma_alpha);
 gdouble ncm_sphPlm_x (gint l, gint m, gint order);
-gdouble ncm_sphPlm_test_theta (gdouble theta, gint lmax, gint *lmin_data);
 gdouble ncm_sum (gdouble *d, gulong n);
-gdouble ncm_numdiff_1 (gsl_function *F, const gdouble x, const gdouble ho, gdouble *err);
-gdouble ncm_numdiff_2 (gsl_function *F, gdouble *ofx, const gdouble x, const gdouble ho, gdouble *err);
-gdouble ncm_numdiff_2_err (gsl_function *F, gdouble *ofx, const gdouble x, const gdouble ho, gdouble err, gdouble *ferr);
-gdouble ncm_sqrt1px_m1 (gdouble x);
+
+G_INLINE_FUNC gdouble ncm_util_sqrt1px_m1 (const gdouble x);
+G_INLINE_FUNC gdouble ncm_util_ln1pexpx (const gdouble x);
+G_INLINE_FUNC gdouble ncm_util_1pcosx (const gdouble sinx, const gdouble cosx);
+G_INLINE_FUNC gdouble ncm_util_1mcosx (const gdouble sinx, const gdouble cosx);
+G_INLINE_FUNC gdouble ncm_util_1psinx (const gdouble sinx, const gdouble cosx);
+G_INLINE_FUNC gdouble ncm_util_1msinx (const gdouble sinx, const gdouble cosx);
+G_INLINE_FUNC gdouble ncm_util_cos2x (const gdouble sinx, const gdouble cosx);
+
 gdouble ncm_cmpdbl (const gdouble x, const gdouble y);
 gdouble ncm_exprel (const gdouble x);
 gdouble ncm_d1exprel (const gdouble x);
@@ -62,29 +67,27 @@ gsize ncm_mpq_inp_raw (mpq_t q, FILE *f);
 
 gulong ncm_random_seed (void);
 
-gint ncm_cmp (gdouble x, gdouble y, gdouble reltol);
+gint ncm_cmp (gdouble x, gdouble y, const gdouble reltol, const gdouble abstol);
 
 void ncm_rational_coarce_double (gdouble x, mpq_t q);
 void ncm_mpz_inits (mpz_t z, ...) G_GNUC_NULL_TERMINATED;
 void ncm_mpz_clears (mpz_t z, ...) G_GNUC_NULL_TERMINATED;
-void _ncm_assertion_message_cmpdouble (const gchar *domain, const gchar *file, gint line, const gchar *func, const gchar *expr, gdouble arg1, const gchar *cmp, gdouble arg2);
+void _ncm_assertion_message_cmpdouble (const gchar *domain, const gchar *file, gint line, const gchar *func, const gchar *expr, gdouble arg1, const gchar *cmp, gdouble arg2, const gdouble reltol, const gdouble abstol);
 
 gboolean ncm_util_cvode_check_flag (gpointer flagvalue, const gchar *funcname, gint opt);
 gboolean ncm_util_cvode_print_stats (gpointer cvode);
 
 gchar *ncm_util_basename_fits (const gchar *fits_filename); 
 gchar *ncm_util_function_params (const gchar *func, gdouble **x, guint *len);
+void ncm_util_print_bits (guint64 num);
+
+gulong ncm_util_fact_size (const gulong n);
 
 typedef struct _NcmComplex NcmComplex;
 
-struct _NcmComplex
+struct _NcmComplex 
 {
-  /*< private >*/
-#ifndef NUMCOSMO_GIR_SCAN
-  complex double z;
-#else
-  gdouble Rez, Imz;
-#endif /* NUMCOSMO_GIR_SCAN */
+  gdouble z[2];
 };
 
 GType ncm_complex_get_type (void) G_GNUC_CONST;
@@ -95,8 +98,17 @@ NcmComplex *ncm_complex_dup (NcmComplex *c);
 void ncm_complex_free (NcmComplex *c);
 void ncm_complex_clear (NcmComplex **c);
 
-gdouble ncm_complex_Re (NcmComplex *c);
-gdouble ncm_complex_Im (NcmComplex *c);
+G_INLINE_FUNC void ncm_complex_set (NcmComplex *c, const gdouble a, const gdouble b);
+G_INLINE_FUNC void ncm_complex_set_zero (NcmComplex *c);
+
+G_INLINE_FUNC gdouble ncm_complex_Re (NcmComplex *c);
+G_INLINE_FUNC gdouble ncm_complex_Im (NcmComplex *c);
+
+G_INLINE_FUNC void ncm_complex_res_add_mul_real (NcmComplex * restrict c1, const NcmComplex * restrict c2, const gdouble v);
+G_INLINE_FUNC void ncm_complex_res_add_mul (NcmComplex * restrict c1, const NcmComplex * restrict c2, const NcmComplex * restrict c3);
+
+G_INLINE_FUNC void ncm_complex_mul_real (NcmComplex *c, const gdouble v);
+G_INLINE_FUNC void ncm_complex_res_mul (NcmComplex * restrict c1, const NcmComplex * restrict c2);
 
 G_INLINE_FUNC gdouble ncm_util_smooth_trans (gdouble f0, gdouble f1, gdouble z0, gdouble dz, gdouble z);
 G_INLINE_FUNC void ncm_util_smooth_trans_get_theta (gdouble z0, gdouble dz, gdouble z, gdouble *theta0, gdouble *theta1);
@@ -107,6 +119,12 @@ G_INLINE_FUNC void ncm_util_smooth_trans_get_theta (gdouble z0, gdouble dz, gdou
 #define exp10(x) (exp ((x) * M_LN10))
 #endif /* HAVE_EXP10 */
 
+#ifndef NUMCOSMO_GIR_SCAN
+#ifndef HAVE_SINCOS
+G_INLINE_FUNC void sincos (gdouble x, gdouble *s, gdouble *c);
+#endif
+#endif
+
 #define ncm_acb_get_complex(z) (arf_get_d (arb_midref (acb_realref (z)), ARF_RND_NEAR) + I * arf_get_d (arb_midref (acb_imagref (z)), ARF_RND_NEAR))
 
 #define ncm_util_exp10(x) (exp ((x) * M_LN10))
@@ -127,16 +145,16 @@ memcpy ((dest)->data, (src)->data, (src)->len * g_array_get_element_size (src));
 
 #define ncm_assert_cmpdouble(n1,cmp,n2) \
 do { \
-  if (ncm_cmp ((n1), (n2), GSL_DBL_EPSILON) cmp 0) ; else \
+  if (ncm_cmp ((n1), (n2), GSL_DBL_EPSILON, 0.0) cmp 0) ; else \
     _ncm_assertion_message_cmpdouble (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, \
-                                      #n1 " " #cmp " " #n2, (n1), #cmp, (n2)); \
+                                      #n1 " " #cmp " " #n2, (n1), #cmp, (n2), GSL_DBL_EPSILON, 0.0); \
 } while (0)
 
-#define ncm_assert_cmpdouble_e(n1,cmp,n2,epsilon) \
+#define ncm_assert_cmpdouble_e(n1,cmp,n2,epsilon,abstol) \
 do { \
-  if (ncm_cmp ((n1), (n2), (epsilon)) cmp 0) ; else \
+  if (ncm_cmp ((n1), (n2), (epsilon), (abstol)) cmp 0) ; else \
     _ncm_assertion_message_cmpdouble (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, \
-                                      #n1 " " #cmp " " #n2, (n1), #cmp, (n2)); \
+                                      #n1 " " #cmp " " #n2, (n1), #cmp, (n2), (epsilon), (abstol)); \
 } while (0)
 
 #define NCM_RETURN_IF_INF(a) if (gsl_isinf(a)) return a
@@ -147,52 +165,9 @@ do { \
 
 #define NCM_TEST_GSL_RESULT(func,ret) if (ret != GSL_SUCCESS) g_error ("%s: %s", func, gsl_strerror (ret))
 
-#define NCM_COMPLEX_INC_MUL_REAL_TEST(a,b,c) \
-((fabs(GSL_REAL((b))*(c)/GSL_REAL((a))) < 1e-16) && (fabs(GSL_IMAG((b))*(c)/GSL_IMAG((a))) < 1e-16))
-
-#define NCM_COMPLEX_INC_MUL_REAL(a,b,c) \
-G_STMT_START { \
-  GSL_REAL((a)) += GSL_REAL((b))*(c); \
-  GSL_IMAG((a)) += GSL_IMAG((b))*(c); \
-} G_STMT_END
-
-#define NCM_COMPLEX_INC_MUL(a,b,c) \
-G_STMT_START { \
-  GSL_REAL((a)) += GSL_REAL((b)) * GSL_REAL((c)) - GSL_IMAG((b)) * GSL_IMAG((c)); \
-  GSL_IMAG((a)) += GSL_REAL((b)) * GSL_IMAG((c)) + GSL_IMAG((b)) * GSL_REAL((c)); \
-} G_STMT_END
-
-#define NCM_COMPLEX_INC_MUL_MUL_REAL(a,b,c,d) \
-G_STMT_START { \
-  GSL_REAL((a)) += (GSL_REAL((b)) * GSL_REAL((c)) - GSL_IMAG((b)) * GSL_IMAG((c))) * (d); \
-  GSL_IMAG((a)) += (GSL_REAL((b)) * GSL_IMAG((c)) + GSL_IMAG((b)) * GSL_REAL((c))) * (d); \
-} G_STMT_END
-
-#define NCM_COMPLEX_MUL_REAL(a,b,c) \
-G_STMT_START { \
-  GSL_REAL((a)) = GSL_REAL((b)) * (c); \
-  GSL_IMAG((a)) = GSL_IMAG((b)) * (c); \
-} G_STMT_END
-
-#define NCM_COMPLEX_MUL(a,b) \
-G_STMT_START { \
-  gdouble temp = GSL_REAL((a)) * GSL_REAL((b)) - GSL_IMAG((a)) * GSL_IMAG((b)); \
-  GSL_IMAG(a) = GSL_REAL((a)) * GSL_IMAG((b)) + GSL_IMAG((a)) * GSL_REAL((b)); \
-  GSL_REAL(a) = temp; \
-} G_STMT_END
-
-#define NCM_COMPLEX_ADD(a,b) \
-G_STMT_START { \
-  GSL_REAL(a) += GSL_REAL((b)); \
-  GSL_IMAG(a) += GSL_IMAG((b)); \
-} G_STMT_END
-
-#define NCM_COMPLEX_MUL_CONJUGATE(a,b) \
-G_STMT_START { \
-  gdouble temp = GSL_REAL((a)) * GSL_REAL((b)) + GSL_IMAG((a)) * GSL_IMAG((b)); \
-  GSL_IMAG(a) = - GSL_REAL((a)) * GSL_IMAG((b)) + GSL_IMAG((a)) * GSL_REAL((b)); \
-  GSL_REAL(a) = temp; \
-} G_STMT_END
+#define NCM_COMPLEX_ZERO {{0.0, 0.0}}
+#define NCM_COMPLEX(p) ((NcmComplex *)(p))
+#define NCM_COMPLEX_PTR(p) ((NcmComplex **)(p))
 
 #define NCM_WRITE_INT32(_ff,_ii) G_STMT_START { gint32 _temp_i = GINT32_TO_BE ((_ii)); if (fwrite (&_temp_i, sizeof(gint32), (1), _ff) != 1) g_error ("NCM_WRITE_INT32: io error"); } G_STMT_END
 #define NCM_WRITE_UINT32(_ff,_ii) G_STMT_START { guint32 _temp_i = GUINT32_TO_BE ((_ii)); if (fwrite (&_temp_i, sizeof(guint32), (1), _ff) != 1) g_error ("NCM_WRITE_UINT32: io error"); } G_STMT_END
@@ -225,7 +200,7 @@ G_STMT_START { \
 } G_STMT_END
 
 /* Minumum version here is 2.38 but it segfault during tests so we start at 2.40. */
-#if ((GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 40))
+#if !GLIB_CHECK_VERSION(2,40,0)
 #define NCM_TEST_FAIL(cmd) \
 G_STMT_START { \
   if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDOUT | G_TEST_TRAP_SILENCE_STDERR)) \
@@ -275,7 +250,7 @@ G_STMT_START { \
     g_test_trap_assert_passed (); \
   } \
 } G_STMT_END
-#endif /* !((GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 38)) */
+#endif /* !GLIB_CHECK_VERSION(2,40,0) */
 
 #define NCM_CVODE_CHECK(chk,name,val,ret) \
 G_STMT_START { \
@@ -292,6 +267,68 @@ G_END_DECLS
 
 G_BEGIN_DECLS
 
+G_INLINE_FUNC gdouble
+ncm_util_sqrt1px_m1 (const gdouble x)
+{
+  return x / (sqrt (1.0 + x) + 1.0);
+}
+
+G_INLINE_FUNC gdouble 
+ncm_util_ln1pexpx (const gdouble x)
+{
+	if (x > - GSL_LOG_DBL_EPSILON)
+		return x;
+	else
+	{
+		if (x > 0.0)
+			return x + log1p (exp (-x));
+		else
+			return log1p (exp (x));
+	}
+}
+
+G_INLINE_FUNC gdouble 
+ncm_util_1pcosx (const gdouble sinx, const gdouble cosx)
+{
+  if (cosx > -0.9)
+    return 1.0 + cosx;
+  else
+    return sinx * sinx / (1.0 - cosx);
+}
+
+G_INLINE_FUNC gdouble 
+ncm_util_1mcosx (const gdouble sinx, const gdouble cosx)
+{
+  if (cosx < 0.9)
+    return 1.0 - cosx;
+  else
+    return sinx * sinx / (1.0 + cosx);
+}
+
+G_INLINE_FUNC gdouble 
+ncm_util_1psinx (const gdouble sinx, const gdouble cosx)
+{
+  if (sinx > -0.9)
+    return 1.0 + sinx;
+  else
+    return cosx * cosx / (1.0 - sinx);
+}
+
+G_INLINE_FUNC gdouble 
+ncm_util_1msinx (const gdouble sinx, const gdouble cosx)
+{
+  if (sinx < 0.9)
+    return 1.0 - sinx;
+  else
+    return cosx * cosx / (1.0 + sinx);
+}
+
+G_INLINE_FUNC gdouble 
+ncm_util_cos2x (const gdouble sinx, const gdouble cosx)
+{
+  return (cosx - sinx) * (cosx + sinx);
+}
+
 G_INLINE_FUNC gdouble 
 ncm_util_smooth_trans (gdouble f0, gdouble f1, gdouble z0, gdouble dz, gdouble z)
 {
@@ -319,6 +356,73 @@ ncm_util_smooth_trans_get_theta (gdouble z0, gdouble dz, gdouble z, gdouble *the
   theta1[0] = 1.0 / (1.0 + exp_mgz);
 }
 
+#ifndef NUMCOSMO_GIR_SCAN
+#ifndef HAVE_SINCOS
+G_INLINE_FUNC void 
+sincos (gdouble x, gdouble *s, gdouble *c)
+{
+  s[0] = sin (x);
+  c[0] = cos (x);
+}
+#endif
+#endif
+
+/* NcmComplex methods */
+
+G_INLINE_FUNC void 
+ncm_complex_set (NcmComplex *c, const gdouble a, const gdouble b)
+{
+  c->z[0] = a;
+  c->z[1] = b;
+}
+
+G_INLINE_FUNC void 
+ncm_complex_set_zero (NcmComplex *c)
+{
+  c->z[0] = c->z[1] = 0.0;
+}
+
+G_INLINE_FUNC gdouble
+ncm_complex_Re (NcmComplex *c)
+{
+  return c->z[0];
+}
+
+G_INLINE_FUNC gdouble
+ncm_complex_Im (NcmComplex *c)
+{
+  return c->z[1];
+}
+
+G_INLINE_FUNC void
+ncm_complex_res_add_mul_real (NcmComplex * restrict c1, const NcmComplex * restrict c2, const gdouble v)
+{
+  c1->z[0] += c2->z[0] * v;
+  c1->z[1] += c2->z[1] * v;
+}
+
+G_INLINE_FUNC void
+ncm_complex_res_add_mul (NcmComplex * restrict c1, const NcmComplex * restrict c2, const NcmComplex * restrict c3)
+{
+  c1->z[0] += c2->z[0] * c3->z[0] - c2->z[1] * c3->z[1];
+  c1->z[1] += c2->z[0] * c3->z[1] + c2->z[1] * c3->z[0];
+}
+
+G_INLINE_FUNC void 
+ncm_complex_mul_real (NcmComplex *c, const gdouble v)
+{
+  c->z[0] *= v;
+  c->z[1] *= v;  
+}
+
+G_INLINE_FUNC void 
+ncm_complex_res_mul (NcmComplex * restrict c1, const NcmComplex * restrict c2)
+{
+  const gdouble Re_c1 = c1->z[0] * c2->z[0] - c1->z[1] * c1->z[1];
+  c1->z[1] = c1->z[0] * c2->z[1] + c1->z[1] * c2->z[0];
+  c1->z[0] = Re_c1;
+}
+
 G_END_DECLS
 
 #endif /* NUMCOSMO_HAVE_INLINE */
