@@ -31,7 +31,7 @@
  *
  * This object defines the functions for allocating and accessing vectors.
  * Also includes several vector operations.
- * 
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,6 +43,7 @@
 #include "math/ncm_cfg.h"
 #include "math/ncm_util.h"
 
+#ifndef NUMCOSMO_GIR_SCAN
 #ifdef HAVE_BLAS
 #  ifdef HAVE_MKL_CBLAS_H
 #    include <mkl_cblas.h>
@@ -57,6 +58,7 @@
 #ifdef NUMCOSMO_HAVE_FFTW3
 #include <fftw3.h>
 #endif /* NUMCOSMO_HAVE_FFTW3 */
+#endif /* NUMCOSMO_GIR_SCAN */
 
 enum
 {
@@ -68,7 +70,7 @@ G_DEFINE_TYPE (NcmVector, ncm_vector, G_TYPE_OBJECT);
 
 /**
  * ncm_vector_new:
- * @n: defines the size of the vector.
+ * @n: defines the size of the vector
  *
  * This function allocates memory for a new #NcmVector of double
  * with @n components.
@@ -84,14 +86,14 @@ ncm_vector_new (gsize n)
 
 /**
  * ncm_vector_new_full:
- * @d: (array) (element-type double): pointer to the first double allocated.
- * @size: number of doubles allocated.
- * @stride: the step-size from one element to the next in physical memory, measured in units of double.
- * @pdata: (allow-none): descending data pointer.
- * @pfree: (scope notified) (allow-none): free function to be called when destroying the vector.
+ * @d: (array) (element-type double): pointer to the first double allocated
+ * @size: number of doubles allocated
+ * @stride: the step-size from one element to the next in physical memory, measured in units of double
+ * @pdata: (allow-none): descending data pointer
+ * @pfree: (scope notified) (allow-none): free function to be called when destroying the vector
  *
  * This function returns a #NcmVector of the array @d.
- * This function saves @userdata internally and frees it using @free 
+ * This function saves @userdata internally and frees it using @free
  * when it is no longer necessary.
  *
  * Returns: A new #NcmVector.
@@ -113,8 +115,8 @@ ncm_vector_new_full (gdouble *d, gsize size, gsize stride, gpointer pdata, GDest
 
 /**
  * ncm_vector_new_fftw:
- * @size: number of doubles allocated.
- * 
+ * @size: number of doubles allocated
+ *
  * This function allocates memory for a new #NcmVector of double
  * with @n components. It uses fftw_alloc_real in order to be used
  * by fftw* functions.
@@ -125,48 +127,48 @@ NcmVector *
 ncm_vector_new_fftw (guint size)
 {
   gdouble *d = fftw_alloc_real (size);
-  NcmVector *cv = ncm_vector_new_full (d, size, 1, d, (GDestroyNotify) fftw_free); 
+  NcmVector *cv = ncm_vector_new_full (d, size, 1, d, (GDestroyNotify) fftw_free);
   cv->type = NCM_VECTOR_MALLOC;
   return cv;
 }
 
 /**
  * ncm_vector_new_gsl: (skip)
- * @gv: vector from GNU Scientific Library (GSL) to be converted into a #NcmVector.
- * 
+ * @gv: vector from GNU Scientific Library (GSL) to be converted into a #NcmVector
+ *
  * This function saves @gv internally and frees it when it is no longer necessary.
  * The @gv vector must not be freed.
- * 
+ *
  * Returns: A new #NcmVector.
  */
 NcmVector *
 ncm_vector_new_gsl (gsl_vector *gv)
 {
-  NcmVector *cv = ncm_vector_new_full (gv->data, gv->size, gv->stride, gv, (GDestroyNotify)gsl_vector_free); 
+  NcmVector *cv = ncm_vector_new_full (gv->data, gv->size, gv->stride, gv, (GDestroyNotify)gsl_vector_free);
   cv->type = NCM_VECTOR_GSL_VECTOR;
   return cv;
 }
 
 /**
  * ncm_vector_new_gsl_static: (skip)
- * @gv: vector from GNU Scientific Library (GSL) to be converted into a #NcmVector.
- * 
+ * @gv: vector from GNU Scientific Library (GSL) to be converted into a #NcmVector
+ *
  * This function saves @gv internally and does not frees.
  * The @gv vector must be valid during the life of the created #NcmVector.
- * 
+ *
  * Returns: A new #NcmVector.
  */
 NcmVector *
 ncm_vector_new_gsl_static (gsl_vector *gv)
 {
-  NcmVector *cv = ncm_vector_new_full (gv->data, gv->size, gv->stride, NULL, NULL); 
+  NcmVector *cv = ncm_vector_new_full (gv->data, gv->size, gv->stride, NULL, NULL);
   cv->type = NCM_VECTOR_GSL_VECTOR;
   return cv;
 }
 
 /**
  * ncm_vector_new_array:
- * @a: (array) (element-type double): array of doubles to be converted into a #NcmVector.
+ * @a: (array) (element-type double): array of doubles to be converted into a #NcmVector
  *
  * This function saves @a internally and frees it when it is no longer necessary.
  * The @a array must not be freed.
@@ -176,17 +178,17 @@ ncm_vector_new_gsl_static (gsl_vector *gv)
 NcmVector *
 ncm_vector_new_array (GArray *a)
 {
-  NcmVector *cv = ncm_vector_new_full (&g_array_index (a, gdouble, 0), a->len, 1, 
-                                       g_array_ref (a), (GDestroyNotify) &g_array_unref); 
+  NcmVector *cv = ncm_vector_new_full (&g_array_index (a, gdouble, 0), a->len, 1,
+                                       g_array_ref (a), (GDestroyNotify) &g_array_unref);
   cv->type = NCM_VECTOR_ARRAY;
   return cv;
 }
 
 /**
  * ncm_vector_new_data_slice:
- * @d: (array) (element-type double): pointer to the first double allocated.
- * @size: number of doubles allocated.
- * @stride: the step-size from one element to the next in physical memory, measured in units of double.
+ * @d: (array) (element-type double): pointer to the first double allocated
+ * @size: number of doubles allocated
+ * @stride: the step-size from one element to the next in physical memory, measured in units of double
  *
  * This function returns a #NcmVector of the array @d allocated using g_slice function.
  * This function saves @a internally and frees it when it is no longer necessary.
@@ -197,17 +199,17 @@ ncm_vector_new_array (GArray *a)
 NcmVector *
 ncm_vector_new_data_slice (gdouble *d, gsize size, gsize stride)
 {
-  NcmVector *cv = ncm_vector_new_full (d, size, stride, 
-                                       NULL, NULL); 
+  NcmVector *cv = ncm_vector_new_full (d, size, stride,
+                                       NULL, NULL);
   cv->type  = NCM_VECTOR_SLICE;
   return cv;
 }
 
 /**
  * ncm_vector_new_data_malloc:
- * @d: (array) (element-type double): pointer to the first double allocated.
- * @size: number of doubles allocated.
- * @stride: the step-size from one element to the next in physical memory, measured in units of double.
+ * @d: (array) (element-type double): pointer to the first double allocated
+ * @size: number of doubles allocated
+ * @stride: the step-size from one element to the next in physical memory, measured in units of double
  *
  * This function returns a #NcmVector of the array @d allocated using malloc.
  * It saves @d internally and frees it when it is no longer necessary.
@@ -217,17 +219,17 @@ ncm_vector_new_data_slice (gdouble *d, gsize size, gsize stride)
 NcmVector *
 ncm_vector_new_data_malloc (gdouble *d, gsize size, gsize stride)
 {
-  NcmVector *cv = ncm_vector_new_full (d, size, stride, 
-                                       d, &g_free); 
+  NcmVector *cv = ncm_vector_new_full (d, size, stride,
+                                       d, &g_free);
   cv->type  = NCM_VECTOR_MALLOC;
   return cv;
 }
 
 /**
  * ncm_vector_new_data_static:
- * @d: (array) (element-type double): pointer to the first double allocated.
- * @size: number of doubles allocated.
- * @stride: the step-size from one element to the next in physical memory, measured in units of double.
+ * @d: (array) (element-type double): pointer to the first double allocated
+ * @size: number of doubles allocated
+ * @stride: the step-size from one element to the next in physical memory, measured in units of double
  *
  * This function returns a #NcmVector of the array @d.
  * The memory allocated is kept during all time life of the object and
@@ -238,17 +240,17 @@ ncm_vector_new_data_malloc (gdouble *d, gsize size, gsize stride)
 NcmVector *
 ncm_vector_new_data_static (gdouble *d, gsize size, gsize stride)
 {
-  NcmVector *cv = ncm_vector_new_full (d, size, stride, 
-                                       NULL, NULL); 
+  NcmVector *cv = ncm_vector_new_full (d, size, stride,
+                                       NULL, NULL);
   cv->type  = NCM_VECTOR_DERIVED;
   return cv;
 }
 
 /**
  * ncm_vector_new_data_dup:
- * @d: (array) (element-type double): pointer to the first double allocated.
- * @size: number of doubles allocated.
- * @stride: the step-size from one element to the next in physical memory, measured in units of double.
+ * @d: (array) (element-type double): pointer to the first double allocated
+ * @size: number of doubles allocated
+ * @stride: the step-size from one element to the next in physical memory, measured in units of double
  *
  * This function returns a #NcmVector of the array @d.
  * It allocate a new vector and copy the contents of @d into it.
@@ -266,9 +268,9 @@ ncm_vector_new_data_dup (gdouble *d, const gsize size, const gsize stride)
 
 /**
  * ncm_vector_new_variant:
- * @var: a #GVariant of the type "ad".
+ * @var: a #GVariant of the type "ad"
  *
- * This function convert a #GVariant array to a #NcmVector allocating new 
+ * This function convert a #GVariant array to a #NcmVector allocating new
  * memory for the vector.
  *
  * Returns: A new #NcmVector.
@@ -276,17 +278,17 @@ ncm_vector_new_data_dup (gdouble *d, const gsize size, const gsize stride)
 NcmVector *
 ncm_vector_new_variant (GVariant *var)
 {
-  NcmVector *cv = g_object_new (NCM_TYPE_VECTOR, 
-                                "values", var, 
+  NcmVector *cv = g_object_new (NCM_TYPE_VECTOR,
+                                "values", var,
                                 NULL);
   return cv;
 }
 
 /**
  * ncm_vector_const_new_data:
- * @d: (array) (element-type double): pointer to the first double allocated.
- * @size: number of doubles allocated.
- * @stride: the step-size from one element to the next in physical memory, measured in units of double.
+ * @d: (array) (element-type double): pointer to the first double allocated
+ * @size: number of doubles allocated
+ * @stride: the step-size from one element to the next in physical memory, measured in units of double
  *
  * This function returns a constant #NcmVector of the array @d.
  * The memory allocated is kept during all time life of the object and
@@ -311,7 +313,7 @@ ncm_vector_const_new_data (const gdouble *d, gsize size, gsize stride)
 
 /**
  * ncm_vector_ref:
- * @cv: a NcmVector.
+ * @cv: a #NcmVector
  *
  * This function increses the reference count of the vector @cv.
  *
@@ -325,7 +327,7 @@ ncm_vector_ref (NcmVector *cv)
 
 /**
  * ncm_vector_const_ref:
- * @cv: a NcmVector.
+ * @cv: a #NcmVector
  *
  * This function increses the reference count of the constant vector @cv.
  *
@@ -339,12 +341,12 @@ ncm_vector_const_ref (const NcmVector *cv)
 
 /**
  * ncm_vector_const_new_variant:
- * @var: a #GVariant of the type "ad".
+ * @var: a #GVariant of the type "ad"
  *
- * This function convert a #GVariant array to a #NcmVector. Since it returns 
+ * This function convert a #GVariant array to a #NcmVector. Since it returns
  * a constant #NcmVector it uses the same memory of @var.
  *
- * Returns: (transfer full): A new #NcmVector. 
+ * Returns: (transfer full): A new #NcmVector.
  */
 const NcmVector *
 ncm_vector_const_new_variant (GVariant *var)
@@ -361,7 +363,7 @@ ncm_vector_const_new_variant (GVariant *var)
 
 /**
  * ncm_vector_dup:
- * @cv: a constant #NcmVector.
+ * @cv: a constant #NcmVector
  *
  * This function copies the elements of the constant vector @cv into a new #NcmVector.
  *
@@ -379,14 +381,14 @@ ncm_vector_dup (const NcmVector *cv)
  * ncm_vector_substitute:
  * @cv: a #NcmVector
  * @nv: a #NcmVector
- * @check_size: whether to check vector size.
+ * @check_size: whether to check vector size
  *
  * This function substitute the vector *@cv by @nv, it will unref *@cv first.
- * If @check_size is TRUE then the function asserts that both vectors have the 
+ * If @check_size is TRUE then the function asserts that both vectors have the
  * same size.
  *
  */
-void 
+void
 ncm_vector_substitute (NcmVector **cv, NcmVector *nv, gboolean check_size)
 {
   if (*cv == nv)
@@ -405,23 +407,57 @@ ncm_vector_substitute (NcmVector **cv, NcmVector *nv, gboolean check_size)
 
 /**
  * ncm_vector_get_subvector:
- * @cv: a #NcmVector.
- * @k: component index of the original vector.
- * @size: number of components of the subvector.
+ * @cv: a #NcmVector
+ * @k: component index of the original vector
+ * @size: number of components of the subvector
  *
  * This function returns a #NcmVector which is a subvector of the vector @cv.
- * The start of the new vector is the @k-th component from the original vector @cv.
+ * The start of the new vector is the component @k from the original vector @cv.
  * The new vector has @size elements.
  *
  * Returns: (transfer full): A #NcmVector.
  */
 NcmVector *
-ncm_vector_get_subvector (NcmVector *cv, gsize k, gsize size)
+ncm_vector_get_subvector (NcmVector *cv, const gsize k, const gsize size)
 {
   NcmVector *scv = g_object_new (NCM_TYPE_VECTOR, NULL);
 
-  scv->vv = gsl_vector_subvector (ncm_vector_gsl (cv), k, size);
-  scv->type = NCM_VECTOR_DERIVED;
+  g_assert_cmpuint (size, >, 0);
+  g_assert_cmpuint (size + k, <=, ncm_vector_len (cv));
+
+  scv->vv    = gsl_vector_subvector (ncm_vector_gsl (cv), k, size);
+  scv->type  = NCM_VECTOR_DERIVED;
+  scv->pdata = ncm_vector_ref (cv);
+  scv->pfree = (GDestroyNotify) &ncm_vector_free;
+
+  return scv;
+}
+
+/**
+ * ncm_vector_get_subvector_stride:
+ * @cv: a #NcmVector
+ * @k: component index of the original vector
+ * @size: number of components of the subvector
+ * @stride: the step-size from one element to the next in physical memory, measured in units of double
+ *
+ * This function returns a #NcmVector which is a subvector of the vector @cv.
+ * The start of the new vector is the component @k from the original vector @cv.
+ * The new vector has @size elements.
+ *
+ * Returns: (transfer full): A #NcmVector.
+ */
+NcmVector *
+ncm_vector_get_subvector_stride (NcmVector *cv, const gsize k, const gsize size, const gsize stride)
+{
+  NcmVector *scv  = ncm_vector_new_data_static (ncm_vector_data (cv), size, stride);
+  const gsize len = ncm_vector_len (cv);
+
+  g_assert_cmpuint (size,   >, 0);
+  g_assert_cmpuint (stride, >, 0);
+  
+  g_assert_cmpuint ((size - 1) * stride + k, <, len);
+
+  scv->type  = NCM_VECTOR_DERIVED;
   scv->pdata = ncm_vector_ref (cv);
   scv->pfree = (GDestroyNotify) &ncm_vector_free;
 
@@ -430,7 +466,7 @@ ncm_vector_get_subvector (NcmVector *cv, gsize k, gsize size)
 
 /**
  * ncm_vector_get_variant:
- * @v: a #NcmVector.
+ * @v: a #NcmVector
  *
  * Convert @v to a GVariant of the type "ad" without destroying the
  * original vector @v;
@@ -458,7 +494,7 @@ ncm_vector_get_variant (const NcmVector *v)
 
 /**
  * ncm_vector_peek_variant:
- * @v: a #NcmVector.
+ * @v: a #NcmVector
  *
  * Convert @v to a GVariant of the type "ad" using the same memory space.
  * The vector @v should not be modified during the variant existance.
@@ -487,15 +523,16 @@ ncm_vector_peek_variant (const NcmVector *v)
 
 /**
  * ncm_vector_log_vals:
- * @v: a #NcmVector.
- * @prestr: initial string.
- * @format: float format.
- *
+ * @v: a #NcmVector
+ * @prestr: initial string
+ * @format: float format
+ * @cr: whether to include a cariage return
+ * 
  * Log the vector values using @prestr and @format.
  *
  */
 void 
-ncm_vector_log_vals (const NcmVector *v, const gchar *prestr, const gchar *format)
+ncm_vector_log_vals (const NcmVector *v, const gchar *prestr, const gchar *format, gboolean cr)
 {
   guint i = 0;
   const guint len = ncm_vector_len (v);
@@ -507,21 +544,22 @@ ncm_vector_log_vals (const NcmVector *v, const gchar *prestr, const gchar *forma
     g_message (" ");
     g_message (format, ncm_vector_get (v, i));
   }
-  g_message ("\n");
+  if (cr)
+    g_message ("\n");
 }
 
 /**
  * ncm_vector_log_vals_avpb:
- * @v: a #NcmVector.
- * @prestr: initial string.
- * @format: float format.
- * @a: a double.
- * @b: a double.
+ * @v: a #NcmVector
+ * @prestr: initial string
+ * @format: float format
+ * @a: a double
+ * @b: a double
  *
  * Log the vector values ($a\vec{v}+b$) using @prestr and @format.
  *
  */
-void 
+void
 ncm_vector_log_vals_avpb (const NcmVector *v, const gchar *prestr, const gchar *format, const gdouble a, const gdouble b)
 {
   guint i = 0;
@@ -539,16 +577,16 @@ ncm_vector_log_vals_avpb (const NcmVector *v, const gchar *prestr, const gchar *
 
 /**
  * ncm_vector_log_vals_func:
- * @v: a #NcmVector.
- * @prestr: initial string.
- * @format: float format.
- * @f: (scope notified): a #NcmVectorCompFunc.
- * @user_data: user data used in @f.
+ * @v: a #NcmVector
+ * @prestr: initial string
+ * @format: float format
+ * @f: (scope notified): a #NcmVectorCompFunc
+ * @user_data: user data used in @f
  *
  * Log the vector values (f(\vec{v}_i)$) using @prestr and @format.
  *
  */
-void 
+void
 ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *format, NcmVectorCompFunc f, gpointer user_data)
 {
   guint i = 0;
@@ -566,95 +604,141 @@ ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *
 
 /**
  * ncm_vector_const_new_gsl: (skip)
- * @v: vector from GNU Scientific Library (GSL).
+ * @v: vector from GNU Scientific Library (GSL)
  *
  * This function converts @v into a constant #NcmVector.
  *
- * Returns: A new constant #NcmVector.
+ * Returns: A new constant #NcmVector
  */
 /**
  * ncm_vector_get:
- * @cv: a constant #NcmVector.
- * @i: component index.
+ * @cv: a constant #NcmVector
+ * @i: component index
  *
  * Returns: The @i-th component of the vector @cv.
  */
 /**
  * ncm_vector_fast_get:
- * @cv: a constant #NcmVector.
- * @i: component index.
+ * @cv: a constant #NcmVector
+ * @i: component index
  *
  * Returns: The @i-th component of the vector @cv assuming stride == 1.
  */
 /**
  * ncm_vector_ptr:
- * @cv: a #NcmVector.
- * @i: component index.
+ * @cv: a #NcmVector
+ * @i: component index
  *
  * Returns: A pointer to the @i-th component of the vector @cv.
  */
 /**
  * ncm_vector_fast_ptr:
- * @cv: a #NcmVector.
- * @i: component index.
+ * @cv: a #NcmVector
+ * @i: component index
  *
  * Returns: A pointer to the @i-th component of the vector @cv assuming stride == 1.
  */
 /**
  * ncm_vector_set:
- * @cv: a #NcmVector.
- * @i: component index.
- * @val: a constant double.
+ * @cv: a #NcmVector
+ * @i: component index
+ * @val: a constant double
  *
  * This function sets the value of the @i-th component of the vector @cv to @val.
  */
 /**
  * ncm_vector_fast_set:
- * @cv: a #NcmVector.
- * @i: component index.
- * @val: a constant double.
+ * @cv: a #NcmVector
+ * @i: component index
+ * @val: a constant double
  *
  * This function sets the value of the @i-th component of the vector @cv to @val assuming stride == 1.
  */
 /**
  * ncm_vector_addto:
- * @cv: a #NcmVector.
- * @i: component index.
- * @val: a constant double.
+ * @cv: a #NcmVector
+ * @i: component index
+ * @val: a constant double
  *
  * This function adds @val to the value of the @i-th component of @cv.
  *
  */
 /**
- * ncm_vector_subfrom:
+ * ncm_vector_fast_addto:
  * @cv: a #NcmVector.
  * @i: component index.
- * @val: a cosntant double.
+ * @val: a constant double.
+ *
+ * This function adds @val to the value of the @i-th component of @cv assuming stride == 1.
+ *
+ */
+/**
+ * ncm_vector_subfrom:
+ * @cv: a #NcmVector
+ * @i: component index
+ * @val: a constant double
  *
  * This function subtracts @val from the value of the @i-th component of @cv.
  *
  */
 /**
  * ncm_vector_fast_subfrom:
- * @cv: a #NcmVector.
- * @i: component index.
- * @val: a cosntant double.
- * 
+ * @cv: a #NcmVector
+ * @i: component index
+ * @val: a constant double
+ *
  * This function subtracts @val from the value of the @i-th component of @cv assuming stride == 1.
+ *
+ */
+/**
+ * ncm_vector_mulby:
+ * @cv: a #NcmVector
+ * @i: component index
+ * @val: a constant double
+ *
+ * This function multiplies the @i-th component by @val.
+ *
+ */
+/**
+ * ncm_vector_fast_mulby:
+ * @cv: a #NcmVector
+ * @i: component index
+ * @val: a constant double
+ *
+ * This function multiplies the @i-th component by @val assuming stride == 1.
  * 
  */
 /**
  * ncm_vector_set_all:
- * @cv: a #NcmVector.
- * @val: a constant double.
+ * @cv: a #NcmVector
+ * @val: a constant double
  *
  * This function sets all the components of the vector @cv to the value @val.
  *
  */
 /**
- * ncm_vector_scale:
+ * ncm_vector_set_data:
  * @cv: a #NcmVector.
- * @val: a cosntant double.
+ * @array: (array length=size) (element-type double): a pointer to a double array
+ * @size: data array size
+ * 
+ * This function sets all the components of the vector @cv using the data array @array,
+ * @size must match the vector size.
+ *
+ */
+/**
+ * ncm_vector_set_array:
+ * @cv: a #NcmVector.
+ * @array: (element-type double): a pointer to a double #GArray
+ * 
+ * This function sets all the components of the vector @cv using the data array @array,
+ * @array->len must match the vector size.
+ *
+ */
+/**
+ * ncm_vector_scale:
+ * @cv: a #NcmVector
+ * @val: a constant double
  *
  * This function multiplies the components of the vector @cv by the constant factor @val.
  *
@@ -667,10 +751,19 @@ ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *
  * This function adds the constant @val to all components.
  *
  */
+/**
+ * ncm_vector_mul:
+ * @cv1: a #NcmVector, numerator
+ * @cv2: a #NcmVector, denominator
+ *
+ * This function multiplies the components of the vector @cv1 by the components of the vector @cv2.
+ * The two vectors must have the same length.
+ *
+ */
 /**
  * ncm_vector_div:
- * @cv1: a #NcmVector, numerator.
- * @cv2: a #NcmVector, denominator.
+ * @cv1: a #NcmVector, numerator
+ * @cv2: a #NcmVector, denominator
  *
  * This function divides the components of the vector @cv1 by the components of the vector @cv2.
  * The two vectors must have the same length.
@@ -678,8 +771,8 @@ ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *
  */
 /**
  * ncm_vector_add:
- * @cv1: a #NcmVector.
- * @cv2: a #NcmVector.
+ * @cv1: a #NcmVector
+ * @cv2: a #NcmVector
  *
  * This function adds the components of the vector @cv2 to the components of the vector @cv1.
  * The two vectors must have the same length.
@@ -687,8 +780,8 @@ ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *
  */
 /**
  * ncm_vector_sub:
- * @cv1: a #NcmVector.
- * @cv2: a #NcmVector.
+ * @cv1: a #NcmVector
+ * @cv2: a #NcmVector
  *
  * This function subtracts the components of the vector @cv2 to the components of the vector @cv1.
  * The two vectors must have the same length.
@@ -696,15 +789,15 @@ ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *
  */
 /**
  * ncm_vector_set_zero:
- * @cv: a #NcmVector.
+ * @cv: a #NcmVector
  *
  * This function sets all the components of the vector @cv to zero.
  *
  */
 /**
  * ncm_vector_memcpy:
- * @cv1: a #NcmVector.
- * @cv2: a #NcmVector.
+ * @cv1: a #NcmVector
+ * @cv2: a #NcmVector
  *
  * This function copies the components of the vector @cv2 into the vector @cv1.
  * The two vectors must have the same length.
@@ -712,11 +805,11 @@ ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *
  */
 /**
  * ncm_vector_memcpy2:
- * @cv1: a #NcmVector.
- * @cv2: a #NcmVector.
- * @cv1_start: component of @cv1.
- * @cv2_start: component of @cv2.
- * @size: number of components.
+ * @cv1: a #NcmVector
+ * @cv2: a #NcmVector
+ * @cv1_start: component of @cv1
+ * @cv2_start: component of @cv2
+ * @size: number of components
  *
  * This function copies @size components of @cv2, counting from @cv2_start,
  * to the vector @cv1, starting from the @cv1_start component.
@@ -725,7 +818,7 @@ ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *
  */
 /**
  * ncm_vector_get_array:
- * @cv: a #NcmVector.
+ * @cv: a #NcmVector
  *
  * FIXME
  *
@@ -733,7 +826,7 @@ ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *
  */
 /**
  * ncm_vector_dup_array:
- * @cv: a #NcmVector.
+ * @cv: a #NcmVector
  *
  * FIXME
  *
@@ -741,7 +834,7 @@ ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *
  */
 /**
  * ncm_vector_data:
- * @cv: a #NcmVector.
+ * @cv: a #NcmVector
  *
  * FIXME
  *
@@ -749,7 +842,7 @@ ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *
  */
 /**
  * ncm_vector_const_data:
- * @cv: a #NcmVector.
+ * @cv: a #NcmVector
  *
  * FIXME
  *
@@ -757,8 +850,8 @@ ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *
  */
 /**
  * ncm_vector_ddot:
- * @a: a #NcmVector.
- * @b: a #NcmVector.
+ * @a: a #NcmVector
+ * @b: a #NcmVector
  *
  * FIXME
  *
@@ -766,7 +859,7 @@ ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *
  */
 /**
  * ncm_vector_gsl: (skip)
- * @cv: a #NcmVector.
+ * @cv: a #NcmVector
  *
  * FIXME
  *
@@ -774,7 +867,7 @@ ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *
  */
 /**
  * ncm_vector_const_gsl: (skip)
- * @cv: a #NcmVector.
+ * @cv: a #NcmVector
  *
  * FIXME
  *
@@ -782,7 +875,7 @@ ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *
  */
 /**
  * ncm_vector_len:
- * @cv: a #NcmVector.
+ * @cv: a #NcmVector
  *
  * FIXME
  *
@@ -790,49 +883,64 @@ ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *
  */
 /**
  * ncm_vector_stride:
- * @cv: a #NcmVector.
+ * @cv: a #NcmVector
  *
  * FIXME
  *
  * Returns: FIXME
  */
-
-
 /**
- * ncm_vector_get_minmax: 
+ * ncm_vector_get_max: 
+ * @cv: a @NcmVector.
+ * 
+ * Gets the maximum value of the vector components.
+ * 
+ * Returns: the maximum value of the vector components.
+ */
+/**
+ * ncm_vector_get_min: 
+ * @cv: a @NcmVector.
+ * 
+ * Gets the minimum value of the vector components.
+ * 
+ * Returns: the minimum value of the vector components.
+ */
+/**
+ * ncm_vector_get_max_index: 
+ * @cv: a @NcmVector.
+ * 
+ * Gets the index of the maximal vector component.
+ * 
+ * Returns: the index of the maximal component.
+ */
+/**
+ * ncm_vector_get_min_index: 
+ * @cv: a @NcmVector.
+ * 
+ * Gets the index of the minimal vector component.
+ * 
+ * Returns: the index of the minimal component.
+ */
+/**
+ * ncm_vector_get_minmax:
  * @cv: a @NcmVector.
  * @min: (out): minimum component value of @cv
  * @max: (out): maximum component value of @cv
- * 
+ *
  * Gets the minimum/maximum value of the vector components.
- * 
+ *
  */
-void 
-ncm_vector_get_minmax (const NcmVector *cv, gdouble *min, gdouble *max)
-{
-  guint size = ncm_vector_len (cv);
-  guint i;
-
-  *min = HUGE_VAL;
-  *max = -HUGE_VAL;
-  for (i = 0; i < size; i++)
-  {
-    const gdouble v = ncm_vector_get (cv, i);
-    *min = GSL_MIN (*min, v);
-    *max = GSL_MAX (*max, v);
-  }
-}
 
 /**
- * ncm_vector_get_absminmax: 
+ * ncm_vector_get_absminmax:
  * @cv: a @NcmVector.
  * @absmin: (out): minimum component absolute value of @cv
  * @absmax: (out): maximum component absolute value of @cv
- * 
+ *
  * Gets the minimum/maximum absolute value of the vector components.
- * 
+ *
  */
-void 
+void
 ncm_vector_get_absminmax (const NcmVector *cv, gdouble *absmin, gdouble *absmax)
 {
   guint size = ncm_vector_len (cv);
@@ -849,13 +957,13 @@ ncm_vector_get_absminmax (const NcmVector *cv, gdouble *absmin, gdouble *absmax)
 }
 
 /**
- * ncm_vector_set_from_variant: 
- * @cv: a @NcmVector.
- * @var: a #GVariant of type ad.
- * 
+ * ncm_vector_set_from_variant:
+ * @cv: a #NcmVector
+ * @var: a #GVariant of type ad
+ *
  * Sets the values of @cv using the variant @var. This function fails
  * if @cv and @var differ in size.
- * 
+ *
  */
 void
 ncm_vector_set_from_variant (NcmVector *cv, GVariant *var)
@@ -874,7 +982,7 @@ ncm_vector_set_from_variant (NcmVector *cv, GVariant *var)
     cv->vv = gsl_vector_view_array (d, n);
     cv->pdata = NULL;
     cv->pfree = NULL;
-    cv->type = NCM_VECTOR_SLICE; 
+    cv->type = NCM_VECTOR_SLICE;
   }
   else if (n != ncm_vector_len (cv))
     g_error ("set_property: cannot set vector values, variant contains %zu childs but vector dimension is %u", n, ncm_vector_len (cv));
@@ -884,32 +992,146 @@ ncm_vector_set_from_variant (NcmVector *cv, GVariant *var)
     gdouble val = 0.0;
     g_variant_get_child (var, i, "d", &val);
     ncm_vector_set (cv, i, val);
-  }    
+  }
 }
 
 /**
- * ncm_vector_dnrm2: 
+ * ncm_vector_dnrm2:
  * @cv: a @NcmVector
- * 
+ *
  * Calculates the Euclidean norm of the vector @cv, i.e.,
  * $\vert\text{cv}\vert_2$.
- * 
+ *
  * Returns: $\vert\text{cv}\vert_2$.
  */
-gdouble 
+gdouble
 ncm_vector_dnrm2 (const NcmVector *cv)
 {
-  return cblas_dnrm2 (ncm_vector_len (cv), 
-                      ncm_vector_const_ptr (cv, 0), 
+  return cblas_dnrm2 (ncm_vector_len (cv),
+                      ncm_vector_const_ptr (cv, 0),
                       ncm_vector_stride (cv));
 }
 
 /**
- * ncm_vector_sum_cpts: 
+ * ncm_vector_axpy:
+ * @cv1: a #NcmVector $y$
+ * @alpha: a double $\alpha$
+ * @cv2: a #NcmVector $x$
+ * 
+ * Performs the operation $y = \alpha x + y$.
+ *
+ */
+void 
+ncm_vector_axpy (NcmVector *cv1, const gdouble alpha, const NcmVector *cv2)
+{
+  const guint len = ncm_vector_len (cv1);
+  g_assert_cmpuint (len, ==, ncm_vector_len (cv2));
+
+  cblas_daxpy (len, alpha, ncm_vector_const_data (cv2), ncm_vector_stride (cv2), ncm_vector_data (cv1), ncm_vector_stride (cv1));
+}
+
+/**
+ * ncm_vector_cmp:
+ * @cv1: a #NcmVector $x_1$
+ * @cv2: a #NcmVector $x_2$
+ * 
+ * Performs a comparison, component-wise, of the two vectors and 
+ * puts the weighted difference in @cv1.
+ *
+ */
+void 
+ncm_vector_cmp (NcmVector *cv1, const NcmVector *cv2)
+{
+  const guint len = ncm_vector_len (cv1);
+  guint i;
+
+  for (i = 0; i < len; i++)
+  {
+    const gdouble x1_i = ncm_vector_get (cv1, i);
+    const gdouble x2_i = ncm_vector_get (cv2, i);
+
+    if (G_UNLIKELY (x1_i == 0.0))
+    {
+      if (G_UNLIKELY (x2_i == 0.0))
+        ncm_vector_set (cv1, i, 0.0);
+      else
+        ncm_vector_set (cv1, i, fabs (x2_i));
+    }
+    else if (G_UNLIKELY (x2_i == 0.0))
+    {
+      ncm_vector_set (cv1, i, fabs (x1_i));
+    }
+    else
+    {
+      const gdouble abs_x1_i  = fabs (x1_i);
+      const gdouble abs_x2_i  = fabs (x2_i);
+      const gdouble max_x12_i = GSL_MIN (abs_x1_i, abs_x2_i);
+      ncm_vector_set (cv1, i, fabs ((x1_i - x2_i) / max_x12_i));
+    }
+  }
+}
+
+/**
+ * ncm_vector_sub_round_off:
+ * @cv1: a #NcmVector $x_1$
+ * @cv2: a #NcmVector $x_2$
+ * 
+ * Estimate the round-off error component wise in the
+ * subtraction of @cv1 by @cv2.
+ *
+ */
+void 
+ncm_vector_sub_round_off (NcmVector *cv1, const NcmVector *cv2)
+{
+  const guint len = ncm_vector_len (cv1);
+  guint i;
+
+  for (i = 0; i < len; i++)
+  {
+    const gdouble x1_i        = ncm_vector_get (cv1, i);
+    const gdouble x2_i        = ncm_vector_get (cv2, i);
+    const gdouble abs_x1_i    = fabs (x1_i);
+    const gdouble abs_x2_i    = fabs (x2_i);
+    const gdouble x2_i_m_x1_i = fabs (x2_i - x1_i);
+    
+    if (G_UNLIKELY (x2_i_m_x1_i == 0.0))
+    {
+      ncm_vector_set (cv1, i, 1.0);
+    }
+    else
+    {
+      const gdouble max_x12_i = GSL_MAX (abs_x1_i, abs_x2_i);
+      ncm_vector_set (cv1, i, max_x12_i * GSL_DBL_EPSILON / x2_i_m_x1_i );
+    }
+  }
+}
+
+/**
+ * ncm_vector_reciprocal:
+ * @cv: a #NcmVector
+ * 
+ * Calculates the reciprocal of @cv1.
+ *
+ */
+void 
+ncm_vector_reciprocal (NcmVector *cv)
+{
+  const guint len = ncm_vector_len (cv);
+  guint i;
+
+  for (i = 0; i < len; i++)
+  {
+    const gdouble x_i = ncm_vector_get (cv, i);
+    ncm_vector_set (cv, i, 1.0 / x_i);
+  }
+}
+
+/**
+ * ncm_vector_sum_cpts:
  * @cv: a @NcmVector
- * 
+ *
  * Calculates the sum of the components.
- * 
+ *
  * Returns: the sum of the vector @cv components.
  */
 
@@ -957,7 +1179,7 @@ _ncm_vector_finalize (GObject *object)
 
 /**
  * ncm_vector_free:
- * @cv: a #NcmVector.
+ * @cv: a #NcmVector
  *
  * Atomically decrements the reference count of @cv by one. If the reference count drops to 0,
  * all memory allocated by @cv is released.
@@ -971,21 +1193,21 @@ ncm_vector_free (NcmVector *cv)
 
 /**
  * ncm_vector_clear:
- * @cv: a #NcmVector.
+ * @cv: a #NcmVector
  *
  * Atomically decrements the reference count of @cv by one. If the reference count drops to 0,
  * all memory allocated by @cv is released. The pointer is set to NULL.
  *
  */
-void 
+void
 ncm_vector_clear (NcmVector **cv)
 {
-  g_clear_object (cv);  
+  g_clear_object (cv);
 }
 
 /**
  * ncm_vector_const_free:
- * @cv: a constant #NcmVector.
+ * @cv: a constant #NcmVector
  *
  * Atomically decrements the reference count of @cv by one. If the reference count drops to 0,
  * all memory allocated by @cv is released.
@@ -1001,7 +1223,7 @@ static struct _generic_N_Vector_Ops _ncm_ops;
 
 /**
  * ncm_vector_nvector: (skip)
- * @cv: a #NcmVector.
+ * @cv: a #NcmVector
  *
  * FIXME
  *
@@ -1084,6 +1306,9 @@ _ncm_vector_nvector_free (N_Vector nv)
 
 static struct _generic_N_Vector_Ops _ncm_ops =
 {
+#if defined (HAVE_SUNDIALS_2_7_0) || (HAVE_SUNDIALS_MAJOR == 3)
+  NULL,
+#endif /* HAVE_SUNDIALS_2_7_0 */
   &_ncm_nvclone,
   &_ncm_nvcloneempty,
   &_ncm_vector_nvector_free,
