@@ -31,6 +31,7 @@
 #include <numcosmo/nc_hicosmo.h>
 #include <numcosmo/math/ncm_likelihood.h>
 #include <numcosmo/math/ncm_spline2d.h>
+#include <numcosmo/math/ncm_integral1d_ptr.h>
 
 G_BEGIN_DECLS
 
@@ -43,40 +44,42 @@ G_BEGIN_DECLS
 
 typedef struct _NcHICosmoDEClass NcHICosmoDEClass;
 typedef struct _NcHICosmoDE NcHICosmoDE;
+typedef struct _NcHICosmoDEPrivate NcHICosmoDEPrivate;
 
 /**
  * NcHICosmoDEImpl:
- * @NC_HICOSMO_DE_IMPL_E2Omega_de: FIXME
- * @NC_HICOSMO_DE_IMPL_dE2Omega_de_dz: FIXME
- * @NC_HICOSMO_DE_IMPL_d2E2Omega_de_dz2: FIXME
- * @NC_HICOSMO_DE_IMPL_w_de: FIXME
- * 
+ * @NC_HICOSMO_DE_IMPL_E2Omega_de: dark energy (DE) component of the normalized Hubble function (squared) $E^2(z)$
+ * @NC_HICOSMO_DE_IMPL_dE2Omega_de_dz: DE component of the first derivative of $E^2(z)$ with respect to the redshift $z$ 
+ * @NC_HICOSMO_DE_IMPL_d2E2Omega_de_dz2: DE component of the second derivative of $E^2(z)$ with respect to $z$
+ * @NC_HICOSMO_DE_IMPL_w_de: DE equation of state
+ *
  * FIXME
- * 
+ *
  */
 typedef enum _NcHICosmoDEImpl
 {
-  NC_HICOSMO_DE_IMPL_E2Omega_de       = NC_HICOSMO_IMPL_LAST << 0,
-  NC_HICOSMO_DE_IMPL_dE2Omega_de_dz   = NC_HICOSMO_IMPL_LAST << 1,
-  NC_HICOSMO_DE_IMPL_d2E2Omega_de_dz2 = NC_HICOSMO_IMPL_LAST << 2,
-  NC_HICOSMO_DE_IMPL_w_de             = NC_HICOSMO_IMPL_LAST << 3, /*< private >*/
-  NC_HICOSMO_DE_IMPL_LAST             = NC_HICOSMO_IMPL_LAST << 4, /*< skip >*/
+  NC_HICOSMO_DE_IMPL_E2Omega_de = NC_HICOSMO_IMPL_LAST,
+  NC_HICOSMO_DE_IMPL_dE2Omega_de_dz,
+  NC_HICOSMO_DE_IMPL_d2E2Omega_de_dz2,
+  NC_HICOSMO_DE_IMPL_w_de, 
+  /* < private > */
+  NC_HICOSMO_DE_IMPL_LAST, /*< skip >*/
 } NcHICosmoDEImpl;
 
 typedef gdouble (*NcHICosmoDEFunc1) (NcHICosmoDE *cosmo_de, gdouble z);
 
 /**
  * NcHICosmoDEParams:
- * @NC_HICOSMO_DE_H0: FIXME
- * @NC_HICOSMO_DE_OMEGA_C: FIXME
- * @NC_HICOSMO_DE_OMEGA_X: FIXME
- * @NC_HICOSMO_DE_T_GAMMA0: FIXME
- * @NC_HICOSMO_DE_HE_YP: FIXME
- * @NC_HICOSMO_DE_ENNU: FIXME
- * @NC_HICOSMO_DE_OMEGA_B: FIXME
+ * @NC_HICOSMO_DE_H0: Hubble constant [km/(s Mpc)]
+ * @NC_HICOSMO_DE_OMEGA_C: cold dark matter density parameter
+ * @NC_HICOSMO_DE_OMEGA_X: dark energy density parameter
+ * @NC_HICOSMO_DE_T_GAMMA0: CMB temperature today
+ * @NC_HICOSMO_DE_HE_YP: primordial helium abundance
+ * @NC_HICOSMO_DE_ENNU: effective number of neutrinos
+ * @NC_HICOSMO_DE_OMEGA_B: baryon density parameter
  *
  * FIXME
- * 
+ *
  */
 typedef enum _NcHICosmoDEParams
 {
@@ -86,10 +89,31 @@ typedef enum _NcHICosmoDEParams
   NC_HICOSMO_DE_T_GAMMA0,
   NC_HICOSMO_DE_HE_YP,
   NC_HICOSMO_DE_ENNU,
-  NC_HICOSMO_DE_OMEGA_B,    /*< private >*/
+  NC_HICOSMO_DE_OMEGA_B,
+  /* < private > */
   NC_HICOSMO_DE_SPARAM_LEN, /*< skip >*/
 } NcHICosmoDEParams;
 
+/**
+ * NcHICosmoDEVParams:
+ * @NC_HICOSMO_DE_MASSNU_M: neutrino masses
+ * @NC_HICOSMO_DE_MASSNU_T: massive neutrino temperatures
+ * @NC_HICOSMO_DE_MASSNU_MU: massive neutrino chemical potentials
+ * @NC_HICOSMO_DE_MASSNU_G: massive neutrino degeneracy factors
+ * 
+ * FIXME
+ *
+ */
+typedef enum _NcHICosmoDEVParams
+{
+  NC_HICOSMO_DE_MASSNU_M = 0,
+  NC_HICOSMO_DE_MASSNU_T,
+  NC_HICOSMO_DE_MASSNU_MU,
+  NC_HICOSMO_DE_MASSNU_G,   
+  /* < private > */
+  NC_HICOSMO_DE_VPARAM_LEN, /*< skip >*/
+} NcHICosmoDEVParams;
+
 #define NC_HICOSMO_DE_DEFAULT_H0        ncm_c_hubble_cte_wmap ()
 #define NC_HICOSMO_DE_DEFAULT_OMEGA_C   (0.2568)
 #define NC_HICOSMO_DE_DEFAULT_OMEGA_X   (0.70)
@@ -97,6 +121,10 @@ typedef enum _NcHICosmoDEParams
 #define NC_HICOSMO_DE_DEFAULT_T_GAMMA0  (2.7245)
 #define NC_HICOSMO_DE_DEFAULT_HE_YP     (0.24)
 #define NC_HICOSMO_DE_DEFAULT_ENNU      (3.046)
+#define NC_HICOSMO_DE_DEFAULT_NU_MASS   (1.0e-5)
+#define NC_HICOSMO_DE_DEFAULT_NU_T      (0.71611)
+#define NC_HICOSMO_DE_DEFAULT_NU_MU     (0.0)
+#define NC_HICOSMO_DE_DEFAULT_NU_G      (1.0)
 
 struct _NcHICosmoDEClass
 {
@@ -112,8 +140,7 @@ struct _NcHICosmoDE
 {
   /*< private >*/
   NcHICosmo parent_instance;
-  NcmSpline2d *BBN_spline2d;
-  guint64 HE4_Yp_key;
+  NcHICosmoDEPrivate *priv;
 };
 
 GType nc_hicosmo_de_get_type (void) G_GNUC_CONST;
@@ -149,7 +176,7 @@ NCM_MODEL_FUNC1_IMPL (NC_HICOSMO_DE,NcHICosmoDE,nc_hicosmo_de,dE2Omega_de_dz,z)
 NCM_MODEL_FUNC1_IMPL (NC_HICOSMO_DE,NcHICosmoDE,nc_hicosmo_de,d2E2Omega_de_dz2,z)
 NCM_MODEL_FUNC1_IMPL (NC_HICOSMO_DE,NcHICosmoDE,nc_hicosmo_de,w_de,z)
 
-G_INLINE_FUNC gdouble 
+G_INLINE_FUNC gdouble
 nc_hicosmo_de_E2Omega_de_onepw (NcHICosmoDE *cosmo_de, gdouble z)
 {
   return nc_hicosmo_de_E2Omega_de (cosmo_de, z) * (1.0 + nc_hicosmo_de_w_de (cosmo_de, z));
