@@ -44,14 +44,16 @@ typedef struct _NcHICosmoDEXcdm NcHICosmoDEXcdm;
 
 /**
  * NcHICosmoDEXCDMParams:
- * @NC_HICOSMO_DE_XCDM_W: FIXME
+ * @NC_HICOSMO_DE_XCDM_W: constant parameter
  *
- * FIXME
+ * Dark Energy equation of state: $w(z) = w$.
+ * 
  */
 typedef enum _NcHICosmoDEXCDMParams
 {
-  NC_HICOSMO_DE_XCDM_W = NC_HICOSMO_DE_SPARAM_LEN, /*< private >*/
-  NC_HICOSMO_DE_XCDM_SPARAM_LEN,                 /*< skip >*/
+  NC_HICOSMO_DE_XCDM_W = NC_HICOSMO_DE_SPARAM_LEN, 
+  /* < private > */
+  NC_HICOSMO_DE_XCDM_SPARAM_LEN,                   /*< skip >*/
 } NcHICosmoDEXCDMParams;
 
 #define NC_HICOSMO_DE_XCDM_DEFAULT_W0 (-1.0)
