@@ -28,7 +28,11 @@
  * @title: NcHIPrimPowerLaw
  * @short_description: Power law implementation for primordial spectra.
  *
- * FIXME
+ * Primordial adiabatic scalar power spectrum:
+ * $$ \mathcal{P}_{SA}(k) = \mathcal{A}_\mathrm{s}\left(\frac{k}{k_\star}\right)^{n_s -1 }.$$
+ * 
+ * Primordial tensor power spectrum:
+ * $$ \mathcal{P}_T(k) = r \mathcal{A}_\mathrm{s} \left(\frac{k}{k_\star}\right)^{n_T -1 }.$$
  *
  */
 
@@ -52,7 +56,7 @@ nc_hiprim_power_law_init (NcHIPrimPowerLaw *nc_hiprim_power_law)
 }
 
 static void
-nc_hiprim_power_law_finalize (GObject *object)
+_nc_hiprim_power_law_finalize (GObject *object)
 {
 
   /* Chain up : end */
@@ -69,32 +73,32 @@ nc_hiprim_power_law_class_init (NcHIPrimPowerLawClass *klass)
   NcHIPrimClass *prim_class  = NC_HIPRIM_CLASS (klass);
   NcmModelClass *model_class = NCM_MODEL_CLASS (klass);
 
-  object_class->finalize = nc_hiprim_power_law_finalize;
+  object_class->finalize = &_nc_hiprim_power_law_finalize;
 
   ncm_model_class_set_name_nick (model_class, "Power Law model for primordial spectra", "PowerLaw");
   ncm_model_class_add_params (model_class, NC_HIPRIM_POWER_LAW_SPARAM_LEN, 0, PROP_SIZE);
 
   /* Set ln10e10ASA param info */
   ncm_model_class_set_sparam (model_class, NC_HIPRIM_POWER_LAW_LN10E10ASA, "\\log(10^{10}A_{SA})", "ln10e10ASA",
-                              0.0, 5.0, 1.0e-1,
+                              0.0, 5.0, 1.0e0,
                               NC_HIPRIM_DEFAULT_PARAMS_ABSTOL, NC_HIPRIM_POWER_LAW_DEFAULT_LN10E10ASA,
                               NCM_PARAM_TYPE_FIXED);
 
   /* Set T_SA_ratio param info */
-  ncm_model_class_set_sparam (model_class, NC_HIPRIM_POWER_LAW_T_SA_RATIO, "A_{SA}/A_T", "T_SA_ratio",
-                              0.0, 1.0, 1.0e-4,
+  ncm_model_class_set_sparam (model_class, NC_HIPRIM_POWER_LAW_T_SA_RATIO, "A_T/A_{SA}", "T_SA_ratio",
+                              0.0, 10.0, 1.0e-1,
                               NC_HIPRIM_DEFAULT_PARAMS_ABSTOL, NC_HIPRIM_POWER_LAW_DEFAULT_T_SA_RATIO,
                               NCM_PARAM_TYPE_FIXED);
 
   /* Set N_SA param info */
   ncm_model_class_set_sparam (model_class, NC_HIPRIM_POWER_LAW_N_SA, "n_{SA}", "n_SA",
-                              0.5, 1.5, 1.0e-2,
+                              0.5, 1.5, 1.0e-1,
                               NC_HIPRIM_DEFAULT_PARAMS_ABSTOL, NC_HIPRIM_POWER_LAW_DEFAULT_N_SA,
                               NCM_PARAM_TYPE_FIXED);
 
   /* Set N_T param info */
   ncm_model_class_set_sparam (model_class, NC_HIPRIM_POWER_LAW_N_T, "n_{T}", "n_T",
-                              -0.5, 0.5, 1.0e-3,
+                              -0.5, 0.5, 1.0e-2,
                               NC_HIPRIM_DEFAULT_PARAMS_ABSTOL, NC_HIPRIM_POWER_LAW_DEFAULT_N_T,
                               NCM_PARAM_TYPE_FIXED);
 
@@ -108,9 +112,9 @@ nc_hiprim_power_law_class_init (NcHIPrimPowerLawClass *klass)
 /**
  * nc_hiprim_power_law_new: (constructor)
  *
- * FIXME
+ * This function instantiates a new object of type #NcHIPrimPowerLaw.
  *
- * Returns: (transfer full): FIXME
+ * Returns: (transfer full): A new #NcHIPrimPowerLaw
  */
 NcHIPrimPowerLaw *
 nc_hiprim_power_law_new (void)
@@ -140,6 +144,6 @@ _nc_hiprim_power_law_lnSA_powespec_lnk (NcHIPrim *prim, const gdouble lnk)
 static gdouble
 _nc_hiprim_power_law_lnT_powespec_lnk (NcHIPrim *prim, const gdouble lnk)
 {
-  const gdouble ln_ka = lnk - prim->lnk_pivot;
+  const gdouble ln_ka = lnk - prim->lnk_pivot;  
   return N_T * ln_ka + LN10E10ASA - 10.0 * M_LN10 + log (T_SA_RATIO);
 }
