@@ -1313,7 +1313,7 @@ _nc_cbe_set_pert (NcCBE* cbe, NcHICosmo* cosmo)
 
 	cbe->priv->ppt.has_cl_cmb_temperature              = cbe->target_Cls & NC_DATA_CMB_TYPE_TT ? _TRUE_ : _FALSE_;
 	cbe->priv->ppt.has_cl_cmb_polarization             = cbe->target_Cls & (NC_DATA_CMB_TYPE_EE | NC_DATA_CMB_TYPE_BB | NC_DATA_CMB_TYPE_TE) ? _TRUE_ : _FALSE_;
-	cbe->priv->ppt.has_cl_cmb_lensing_potential        = _TRUE_;
+	cbe->priv->ppt.has_cl_cmb_lensing_potential        = ((cbe->target_Cls & NC_DATA_CMB_TYPE_PHIPHI) || (cbe->use_lensed_Cls)) ? _TRUE_ : _FALSE_;
 	cbe->priv->ppt.has_cl_number_count                 = _FALSE_;
 	cbe->priv->ppt.has_cl_lensing_potential            = _FALSE_;
 	cbe->priv->ppt.has_pk_matter                       = cbe->calc_transfer ? _TRUE_ : _FALSE_;
@@ -1529,7 +1529,7 @@ _nc_cbe_set_spectra (NcCBE* cbe, NcHICosmo* cosmo)
 static void
 _nc_cbe_set_lensing (NcCBE* cbe, NcHICosmo* cosmo)
 {
-	cbe->priv->ple.has_lensed_cls = cbe->use_lensed_Cls ? _TRUE_ : _FALSE_;
+	cbe->priv->ple.has_lensed_cls  = cbe->use_lensed_Cls ? _TRUE_ : _FALSE_;
 	cbe->priv->ple.lensing_verbose = cbe->lensing_verbose;
 }
 
@@ -2054,7 +2054,7 @@ nc_cbe_thermodyn_z_d (NcCBE *cbe)
  *
  * Returns: (transfer full): a #NcmSpline2d for the logarithm base e of the matter power spectrum, $\ln P(\ln k, z)$.
  */
-NcmSpline2d*
+NcmSpline2d *
 nc_cbe_get_matter_ps (NcCBE* cbe)
 {
 	const gint z_size  = cbe->priv->psp.ln_tau_size;
@@ -2163,6 +2163,7 @@ nc_cbe_get_matter_ps (NcCBE* cbe)
 /**
  * nc_cbe_get_all_Cls:
  * @cbe: a #NcCBE
+ * @PHIPHI_Cls: a #NcmVector
  * @TT_Cls: a #NcmVector
  * @EE_Cls: a #NcmVector
  * @BB_Cls: a #NcmVector
@@ -2173,20 +2174,23 @@ nc_cbe_get_matter_ps (NcCBE* cbe)
  * ignored.
  *
  */
-void nc_cbe_get_all_Cls (NcCBE* cbe, NcmVector* TT_Cls, NcmVector* EE_Cls, NcmVector* BB_Cls, NcmVector* TE_Cls)
+void 
+nc_cbe_get_all_Cls (NcCBE* cbe, NcmVector* PHIPHI_Cls, NcmVector* TT_Cls, NcmVector* EE_Cls, NcmVector* BB_Cls, NcmVector* TE_Cls)
 {
-	guint all_Cls_size, index_tt, index_ee, index_bb, index_te;
-	gboolean has_tt, has_ee, has_bb, has_te;
+	guint all_Cls_size, index_pp, index_tt, index_ee, index_bb, index_te;
+	gboolean has_pp, has_tt, has_ee, has_bb, has_te;
 
 	if (cbe->use_lensed_Cls)
 	{
 		struct lensing *ptr = &cbe->priv->ple;
 
     all_Cls_size = ptr->lt_size;
+		index_pp     = ptr->index_lt_pp;
 		index_tt     = ptr->index_lt_tt;
 		index_ee     = ptr->index_lt_ee;
 		index_bb     = ptr->index_lt_bb;
 		index_te     = ptr->index_lt_te;
+		has_pp       = ptr->has_pp;
 		has_tt       = ptr->has_tt;
 		has_ee       = ptr->has_ee;
 		has_bb       = ptr->has_bb;
@@ -2197,10 +2201,12 @@ void nc_cbe_get_all_Cls (NcCBE* cbe, NcmVector* TT_Cls, NcmVector* EE_Cls, NcmVe
 		struct spectra* ptr = &cbe->priv->psp;
 
     all_Cls_size = ptr->ct_size;
+		index_pp     = ptr->index_ct_pp;
 		index_tt     = ptr->index_ct_tt;
 		index_ee     = ptr->index_ct_ee;
 		index_bb     = ptr->index_ct_bb;
 		index_te     = ptr->index_ct_te;
+		has_pp       = ptr->has_pp;
 		has_tt       = ptr->has_tt;
 		has_ee       = ptr->has_ee;
 		has_bb       = ptr->has_bb;
@@ -2211,16 +2217,18 @@ void nc_cbe_get_all_Cls (NcCBE* cbe, NcmVector* TT_Cls, NcmVector* EE_Cls, NcmVe
 		const gdouble T_gamma0 = cbe->priv->pba.T_cmb;
 		const gdouble Cl_fac   = gsl_pow_2 (1.0e6 * T_gamma0);
 		gdouble *all_Cls       = g_new0 (gdouble, all_Cls_size);
-		guint TT_lmax          = has_tt ? (TT_Cls != NULL ? ncm_vector_len (TT_Cls) : 0) : 0;
-		guint EE_lmax          = has_tt ? (EE_Cls != NULL ? ncm_vector_len (EE_Cls) : 0) : 0;
-		guint BB_lmax          = has_tt ? (BB_Cls != NULL ? ncm_vector_len (BB_Cls) : 0) : 0;
-		guint TE_lmax          = has_tt ? (TE_Cls != NULL ? ncm_vector_len (TE_Cls) : 0) : 0;
+		guint PHIPHI_lmax      = has_pp ? (PHIPHI_Cls != NULL ? ncm_vector_len (PHIPHI_Cls) : 0) : 0;
+		guint TT_lmax          = has_tt ? (TT_Cls     != NULL ? ncm_vector_len (TT_Cls)     : 0) : 0;
+		guint EE_lmax          = has_ee ? (EE_Cls     != NULL ? ncm_vector_len (EE_Cls)     : 0) : 0;
+		guint BB_lmax          = has_bb ? (BB_Cls     != NULL ? ncm_vector_len (BB_Cls)     : 0) : 0;
+		guint TE_lmax          = has_te ? (TE_Cls     != NULL ? ncm_vector_len (TE_Cls)     : 0) : 0;
 		guint l;
 
-		g_assert (!(cbe->target_Cls & NC_DATA_CMB_TYPE_TT) || has_tt);
-		g_assert (!(cbe->target_Cls & NC_DATA_CMB_TYPE_EE) || has_ee);
-		g_assert (!(cbe->target_Cls & NC_DATA_CMB_TYPE_BB) || has_bb);
-		g_assert (!(cbe->target_Cls & NC_DATA_CMB_TYPE_TE) || has_te);
+		g_assert (!(cbe->target_Cls & NC_DATA_CMB_TYPE_PHIPHI) || has_pp);
+		g_assert (!(cbe->target_Cls & NC_DATA_CMB_TYPE_TT)     || has_tt);
+		g_assert (!(cbe->target_Cls & NC_DATA_CMB_TYPE_EE)     || has_ee);
+		g_assert (!(cbe->target_Cls & NC_DATA_CMB_TYPE_BB)     || has_bb);
+		g_assert (!(cbe->target_Cls & NC_DATA_CMB_TYPE_TE)     || has_te);
 
 		for (l = 0; l <= cbe->scalar_lmax; l++)
 		{
@@ -2229,6 +2237,12 @@ void nc_cbe_get_all_Cls (NcCBE* cbe, NcmVector* TT_Cls, NcmVector* EE_Cls, NcmVe
 			else
 				spectra_cl_at_l (&cbe->priv->psp, l, all_Cls, NULL, NULL);
 
+			if (PHIPHI_lmax > 0)
+			{
+				const gdouble PHIPHI_Cl = all_Cls[index_pp];
+				ncm_vector_set (PHIPHI_Cls, l, PHIPHI_Cl);
+				PHIPHI_lmax--;
+			}
 			if (TT_lmax > 0)
 			{
 				const gdouble TT_Cl = all_Cls[index_tt];
@@ -2259,7 +2273,6 @@ void nc_cbe_get_all_Cls (NcCBE* cbe, NcmVector* TT_Cls, NcmVector* EE_Cls, NcmVe
 	}
 }
 
-
 /**
  * nc_cbe_debug_test:
  * @cbe: a #NcCBE
@@ -2267,7 +2280,8 @@ void nc_cbe_get_all_Cls (NcCBE* cbe, NcmVector* TT_Cls, NcmVector* EE_Cls, NcmVe
  * Temporary debug function
  *
  */
-void nc_cbe_debug_test (NcCBE *cbe)
+void 
+nc_cbe_debug_test (NcCBE *cbe)
 {
 	NCM_UNUSED (cbe);
 	g_assert_not_reached ();
