@@ -7,7 +7,7 @@
  ****************************************************************************/
 /*
  * numcosmo
- * Copyright (C) Sandro Dias Pinto Vitenti 2012 <sandro@lapsandro>
+ * Copyright (C) Sandro Dias Pinto Vitenti 2012 <sandro@isoftware.com.br>
  * numcosmo is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
@@ -116,13 +116,77 @@ typedef struct _ComovingDistanceArgument{
 enum
 {
   PROP_0,
-  PROP_ZF
+  PROP_ZF,
+  PROP_RECOMB,
+  PROP_SIZE,
 };
 
 G_DEFINE_TYPE (NcDistance, nc_distance, G_TYPE_OBJECT);
 
 static void
-nc_distance_constructed (GObject *object)
+nc_distance_init (NcDistance *dist)
+{
+  dist->use_cache                = TRUE;
+
+  dist->comoving_distance_cache  = ncm_function_cache_new (1, NCM_INTEGRAL_ABS_ERROR, NCM_INTEGRAL_ERROR);
+
+	dist->comoving_infinity        = ncm_function_cache_new (1, NCM_INTEGRAL_ABS_ERROR, NCM_INTEGRAL_ERROR);
+  dist->time_cache               = ncm_function_cache_new (1, NCM_INTEGRAL_ABS_ERROR, NCM_INTEGRAL_ERROR);
+  dist->lookback_time_cache      = ncm_function_cache_new (1, NCM_INTEGRAL_ABS_ERROR, NCM_INTEGRAL_ERROR);
+  dist->conformal_time_cache     = ncm_function_cache_new (1, NCM_INTEGRAL_ABS_ERROR, NCM_INTEGRAL_ERROR);
+
+  dist->sound_horizon_cache      = ncm_function_cache_new (1, NCM_INTEGRAL_ABS_ERROR, NCM_INTEGRAL_ERROR);
+
+  dist->comoving_distance_spline = NULL;
+
+  dist->recomb                   = NULL;
+  
+  dist->ctrl = ncm_model_ctrl_new (NULL);
+}
+
+static void
+_nc_distance_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
+{
+  NcDistance *dist = NC_DISTANCE (object);
+  g_return_if_fail (NC_IS_DISTANCE (object));
+
+  switch (prop_id)
+  {
+    case PROP_ZF:
+      dist->zf = g_value_get_double (value);
+      ncm_model_ctrl_force_update (dist->ctrl);
+      break;
+    case PROP_RECOMB:
+      nc_distance_set_recomb (dist, g_value_get_object (value));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
+static void
+_nc_distance_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
+{
+  NcDistance *dist = NC_DISTANCE (object);
+  g_return_if_fail (NC_IS_DISTANCE (object));
+
+  switch (prop_id)
+  {
+    case PROP_ZF:
+      g_value_set_double (value, dist->zf);
+      break;
+    case PROP_RECOMB:
+      g_value_set_object (value, dist->recomb);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
+static void
+_nc_distance_constructed (GObject *object)
 {
   /* Chain up : start */
   G_OBJECT_CLASS (nc_distance_parent_class)->constructed (object);
@@ -132,11 +196,12 @@ nc_distance_constructed (GObject *object)
 }
 
 static void
-nc_distance_dispose (GObject *object)
+_nc_distance_dispose (GObject *object)
 {
   NcDistance *dist = NC_DISTANCE (object);
 
   ncm_function_cache_clear (&dist->comoving_distance_cache);
+	ncm_function_cache_clear (&dist->comoving_infinity);
   ncm_function_cache_clear (&dist->time_cache);
   ncm_function_cache_clear (&dist->lookback_time_cache);
   ncm_function_cache_clear (&dist->conformal_time_cache);
@@ -151,58 +216,23 @@ nc_distance_dispose (GObject *object)
 }
 
 static void
-nc_distance_finalize (GObject *object)
+_nc_distance_finalize (GObject *object)
 {
 
   /* Chain up : end */
   G_OBJECT_CLASS (nc_distance_parent_class)->finalize (object);
 }
 
-static void
-nc_distance_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
-{
-  NcDistance *dist = NC_DISTANCE (object);
-  g_return_if_fail (NC_IS_DISTANCE (object));
-
-  switch (prop_id)
-  {
-    case PROP_ZF:
-      dist->zf = g_value_get_double (value);
-      ncm_model_ctrl_force_update (dist->ctrl);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-nc_distance_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
-{
-  NcDistance *dist = NC_DISTANCE (object);
-  g_return_if_fail (NC_IS_DISTANCE (object));
-
-  switch (prop_id)
-  {
-    case PROP_ZF:
-      g_value_set_double (value, dist->zf);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
 static void
 nc_distance_class_init (NcDistanceClass *klass)
 {
   GObjectClass* object_class = G_OBJECT_CLASS (klass);
 
-  object_class->constructed  = nc_distance_constructed;
-  object_class->set_property = nc_distance_set_property;
-  object_class->get_property = nc_distance_get_property;
-  object_class->dispose = nc_distance_dispose;
-  object_class->finalize = nc_distance_finalize;
+  object_class->constructed  = &_nc_distance_constructed;
+  object_class->set_property = &_nc_distance_set_property;
+  object_class->get_property = &_nc_distance_get_property;
+  object_class->dispose      = &_nc_distance_dispose;
+  object_class->finalize     = &_nc_distance_finalize;
 
   g_object_class_install_property (object_class,
                                    PROP_ZF,
@@ -213,6 +243,13 @@ nc_distance_class_init (NcDistanceClass *klass)
                                                         G_MAXDOUBLE,
                                                         10.0,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_RECOMB,
+                                   g_param_spec_object ("recomb",
+                                                        NULL,
+                                                        "Recombination object",
+                                                        NC_TYPE_RECOMB,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
 }
 
 /**
@@ -290,6 +327,22 @@ nc_distance_require_zf (NcDistance *dist, const gdouble zf)
   }
 }
 
+void 
+nc_distance_set_recomb (NcDistance *dist, NcRecomb *recomb)
+{
+  if (dist->recomb != recomb)
+  {
+    nc_recomb_clear (&dist->recomb);
+
+    if (recomb != NULL)
+    {
+      dist->recomb = nc_recomb_ref (recomb);
+    }
+
+    ncm_model_ctrl_force_update (dist->ctrl);
+  }
+}
+
 static gdouble dcddz (gdouble y, gdouble x, gpointer userdata);
 
 /**
@@ -304,9 +357,10 @@ void
 nc_distance_prepare (NcDistance *dist, NcHICosmo *cosmo)
 {
   dist->comoving_distance_cache->clear = TRUE;
-  dist->time_cache->clear = TRUE;
-  dist->lookback_time_cache->clear = TRUE;
-  dist->conformal_time_cache->clear = TRUE;
+	dist->comoving_infinity->clear       = TRUE;
+  dist->time_cache->clear              = TRUE;
+  dist->lookback_time_cache->clear     = TRUE;
+  dist->conformal_time_cache->clear    = TRUE;
 
   dist->sound_horizon_cache->clear = TRUE;
 
@@ -321,37 +375,14 @@ nc_distance_prepare (NcDistance *dist, NcHICosmo *cosmo)
 
   ncm_ode_spline_prepare (dist->comoving_distance_spline, cosmo);
 
+  if (dist->recomb != NULL)
+    nc_recomb_prepare_if_needed (dist->recomb, cosmo);
+  
   ncm_model_ctrl_update (dist->ctrl, NCM_MODEL (cosmo));
 
   return;
 }
 
-/**
- * nc_distance_prepare_if_needed:
- * @dist: a #NcDistance
- * @cosmo: a #NcHICosmo
- *
- * FIXME
- *
- */
-static void
-nc_distance_init (NcDistance *dist)
-{
-  dist->use_cache = TRUE;
-
-  dist->comoving_distance_cache = ncm_function_cache_new (1, NCM_INTEGRAL_ABS_ERROR, NCM_INTEGRAL_ERROR);
-
-  dist->time_cache = ncm_function_cache_new (1, NCM_INTEGRAL_ABS_ERROR, NCM_INTEGRAL_ERROR);
-  dist->lookback_time_cache = ncm_function_cache_new (1, NCM_INTEGRAL_ABS_ERROR, NCM_INTEGRAL_ERROR);
-  dist->conformal_time_cache = ncm_function_cache_new (1, NCM_INTEGRAL_ABS_ERROR, NCM_INTEGRAL_ERROR);
-
-  dist->sound_horizon_cache = ncm_function_cache_new (1, NCM_INTEGRAL_ABS_ERROR, NCM_INTEGRAL_ERROR);
-
-  dist->comoving_distance_spline = NULL;
-
-  dist->ctrl = ncm_model_ctrl_new (NULL);
-}
-
 /**
  * nc_distance_hubble:
  * @dist: a #NcDistance
@@ -389,7 +420,7 @@ nc_distance_comoving (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
 
   nc_distance_prepare_if_needed (dist, cosmo);
 
-  if (ncm_model_impl (NCM_MODEL (cosmo)) & NC_HICOSMO_IMPL_Dc)
+  if (ncm_model_check_impl_opt (NCM_MODEL (cosmo), NC_HICOSMO_IMPL_Dc))
     return nc_hicosmo_Dc (cosmo, z);
 
   if (z <= dist->zf)
@@ -422,6 +453,7 @@ dcddz (gdouble cd, gdouble z, gpointer userdata)
   NcHICosmo *cosmo = NC_HICOSMO (userdata);
   const gdouble E2 = nc_hicosmo_E2 (cosmo, z);
   NCM_UNUSED (cd);
+
   return 1.0 / sqrt (E2);
 }
 
@@ -726,7 +758,7 @@ gdouble
 nc_distance_decoupling_redshift (NcDistance *dist, NcHICosmo *cosmo)
 {
   NCM_UNUSED (dist);
-  if (ncm_model_impl (NCM_MODEL (cosmo)) & NC_HICOSMO_IMPL_z_lss)
+  if (ncm_model_check_impl_opt (NCM_MODEL (cosmo), NC_HICOSMO_IMPL_z_lss))
     return nc_hicosmo_z_lss (cosmo);
   else
   {
@@ -762,7 +794,7 @@ nc_distance_sound_horizon (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
   gdouble result, error;
   gsl_function F;
 
-  g_assert (ncm_model_impl (NCM_MODEL (cosmo)) & NC_HICOSMO_IMPL_bgp_cs2);
+  g_assert (ncm_model_check_impl_opt (NCM_MODEL (cosmo), NC_HICOSMO_IMPL_bgp_cs2));
 
   F.function = &sound_horizon_integral_argument;
   F.params = cosmo;
@@ -862,9 +894,10 @@ nc_distance_theta100CMB (NcDistance *dist, NcHICosmo *cosmo)
  * @cosmo: a #NcHICosmo
  *
  * Drag redshift is the epoch at which baryons were released from photons.
- *
- * This function computes $z_d$ using the fitting formula given in
- * [Eisenstein & Hu (1998)][XEisenstein1998],
+ * 
+ * If the @dist object constains a NcRecomb object, it calculates the drag
+ * redshift through the recombination history. Otherwise, it computes $z_d$ 
+ * using the fitting formula given in [Eisenstein & Hu (1998)][XEisenstein1998],
  * $$z_d = \frac{1291 (\Omega_{m0} h^2)^{0.251}}{(1 + 0.659 (\Omega_{m0} h^2)^{0.828})}
  * \left(1 + b_1 (\Omega_{b0} h^2)^{b_2}\right),$$
  * where $\Omega_{b0} h^2$ [nc_hicosmo_Omega_b0h2()] and $\Omega_{m0} h^2$ [nc_hicosmo_Omega_m0h2()]
@@ -879,15 +912,22 @@ nc_distance_theta100CMB (NcDistance *dist, NcHICosmo *cosmo)
 gdouble
 nc_distance_drag_redshift (NcDistance *dist, NcHICosmo *cosmo)
 {
-  gdouble omega_m_h2 = nc_hicosmo_Omega_m0h2 (cosmo);
-  gdouble omega_b_h2 = nc_hicosmo_Omega_b0h2 (cosmo);
-  gdouble b1 = 0.313 * pow (omega_m_h2, -0.419) * (1.0 + 0.607 * pow (omega_m_h2, 0.674));
-  gdouble b2 = 0.238 * pow (omega_m_h2, 0.223);
+  if (dist->recomb != NULL)
+  {
+    return nc_recomb_get_tau_drag_z (dist->recomb, cosmo);
+  }
+  else
+  {
+    gdouble omega_m_h2 = nc_hicosmo_Omega_m0h2 (cosmo);
+    gdouble omega_b_h2 = nc_hicosmo_Omega_b0h2 (cosmo);
+    gdouble b1 = 0.313 * pow (omega_m_h2, -0.419) * (1.0 + 0.607 * pow (omega_m_h2, 0.674));
+    gdouble b2 = 0.238 * pow (omega_m_h2, 0.223);
 
-  NCM_UNUSED (dist);
+    NCM_UNUSED (dist);
 
-  return 1291.0 * pow (omega_m_h2, 0.251) / (1.0 + 0.659 * pow (omega_m_h2, 0.828)) *
-    (1.0 + b1 * pow (omega_b_h2, b2));
+    return 1291.0 * pow (omega_m_h2, 0.251) / (1.0 + 0.659 * pow (omega_m_h2, 0.828)) *
+      (1.0 + b1 * pow (omega_b_h2, b2));
+  }
 }
 
 /**
@@ -955,7 +995,7 @@ nc_distance_r_zd (NcDistance *dist, NcHICosmo *cosmo)
 {
   gdouble r_zd;
 
-  if (ncm_model_impl (NCM_MODEL (cosmo)) & NC_HICOSMO_IMPL_as_drag)
+  if (ncm_model_check_impl_opt (NCM_MODEL (cosmo), NC_HICOSMO_IMPL_as_drag))
     r_zd = nc_hicosmo_as_drag (cosmo);
   else
   {
@@ -968,6 +1008,21 @@ nc_distance_r_zd (NcDistance *dist, NcHICosmo *cosmo)
   return r_zd;
 }
 
+/**
+ * nc_distance_r_zd_Mpc:
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
+ *
+ * $r (z_d) R_H\, [\mathrm{Mpc}]$.
+ *
+ * Returns: FIXME
+ */
+gdouble
+nc_distance_r_zd_Mpc (NcDistance *dist, NcHICosmo *cosmo)
+{
+  return nc_distance_r_zd (dist, cosmo) * nc_hicosmo_RH_Mpc (cosmo);
+}
+
 /**
  * nc_distance_bao_r_Dv:
  * @dist: a #NcDistance
@@ -992,9 +1047,10 @@ nc_distance_bao_r_Dv (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
  * @cosmo: a #NcHICosmo
  * @z: the redshift $z$
  *
- * $H(z) / (c r_{z_d})$.
+ * Computes the ratio between the Hubble (distance) radius and the sound horizon at the drag epoch, 
+ * $$\frac{R_H}{r_s(z_d)} = c / (H(z) r_d).$$  
  *
- * Returns: FIXME
+ * Returns: $R_H/r_d = c / (H(z) r_d)$
  */
 gdouble
 nc_distance_DH_r (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
@@ -1010,9 +1066,9 @@ nc_distance_DH_r (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
  * @cosmo: a #NcHICosmo
  * @z: the redshift $z$
  *
- * $D_A(z) / (c r_{z_d})$.
+ * Computes the ratio between the angular-diameter distance and the sound horizon at the drag epoch, $$D_A(z) / (c r_s(z_d)).$$
  *
- * Returns: FIXME
+ * Returns: $D_A(z) / (c r_d)$
  */
 gdouble
 nc_distance_DA_r (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
@@ -1022,6 +1078,96 @@ nc_distance_DA_r (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
   return DA / r_zd;
 }
 
+/* Distances from z to infinity */
+
+static gdouble
+_nc_distance_comoving_infinity_integrand (gdouble logx, gpointer p)
+{
+  if (logx > GSL_LOG_DBL_MAX)
+    return 0.0;
+  else
+  {
+    NcHICosmo *cosmo = NC_HICOSMO (p);
+    const gdouble z = expm1 (logx);
+    const gdouble E = sqrt (nc_hicosmo_E2 (cosmo, z));
+
+    if (gsl_finite (E))
+      return (1.0 + z) / E;
+    else
+      return 0.0;
+  }
+}
+
+/**
+ * nc_distance_comoving_z_to_infinity:
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
+ * @z: redshift
+ * 
+ * Computes the comoving distance from @z to infinity.
+ * 
+ * Returns: $D_c$ from $z$ to $\infity$
+ */
+gdouble
+nc_distance_comoving_z_to_infinity (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
+{
+  gdouble result, error;
+  gsl_function F;
+  F.function = &_nc_distance_comoving_infinity_integrand;
+  F.params = cosmo;
+
+  if (dist->use_cache)
+    ncm_integral_cached_x_inf (dist->comoving_infinity, &F, log1p (z), &result, &error);
+  else
+    ncm_integral_locked_a_inf (&F, log1p (z), NCM_INTEGRAL_ABS_ERROR, NCM_INTEGRAL_ERROR, &result, &error);
+
+  return result;
+}
+
+
+/**
+ * nc_distance_transverse_z_to_infinity:
+ * @dist: a #NcDistance
+ * @cosmo: a #NcHICosmo
+ * @z: redshift $z$
+ *
+ * Compute the transverse comoving distance $D_t (z)$ (defined in Eq. $\eqref{eq:def:Dt}$) 
+ * but from @z to infinity.
+ *
+ * Returns: $D_t(z)$ from $z$ to $\infity$
+ */
+gdouble
+nc_distance_transverse_z_to_infinity (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
+{
+  const gdouble Omega_k0 = nc_hicosmo_Omega_k0 (cosmo);
+  const gdouble sqrt_Omega_k0 = sqrt (fabs (Omega_k0));
+  const gdouble comoving_dist = nc_distance_comoving_z_to_infinity (dist, cosmo, z);
+  const gint k = fabs (Omega_k0) < NCM_ZERO_LIMIT ? 0 : (Omega_k0 > 0.0 ? -1 : 1);
+  gdouble Dt;
+
+  if (gsl_isinf (comoving_dist))
+    return comoving_dist;
+
+  switch (k)
+  {
+    case 0:
+      Dt = comoving_dist;
+      break;
+    case -1:
+      Dt = sinh (sqrt_Omega_k0 * comoving_dist) / sqrt_Omega_k0;
+      break;
+    case 1:
+      Dt = fabs (sin (sqrt_Omega_k0 * comoving_dist) / sqrt_Omega_k0);
+      break;
+    default:
+      g_assert_not_reached();
+      Dt = 0.0;
+      break;
+  }
+
+  return Dt;
+}
+
 /***************************************************************************
  *            cosmic_time.c
  *
@@ -1031,12 +1177,21 @@ nc_distance_DA_r (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
  ****************************************************************************/
 
 static gdouble
-_nc_time_integrand (gdouble z, gpointer p)
+_nc_distance_cosmic_time_integrand (gdouble logx, gpointer p)
 {
-  NcHICosmo *cosmo = NC_HICOSMO (p);
-  const gdouble x = 1.0 + z;
-  const gdouble E = sqrt (nc_hicosmo_E2 (cosmo, z));
-  return 1.0 / (x * E);
+  if (logx > GSL_LOG_DBL_MAX)
+    return 0.0;
+  else
+  {
+    NcHICosmo *cosmo = NC_HICOSMO (p);
+    const gdouble z = expm1 (logx);
+    const gdouble E = sqrt (nc_hicosmo_E2 (cosmo, z));
+
+    if (gsl_finite (E))
+      return 1.0 / E;
+    else
+      return 0.0;
+  }
 }
 
 /**
@@ -1053,13 +1208,13 @@ nc_distance_cosmic_time (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
 {
   gdouble result, error;
   gsl_function F;
-  F.function = &_nc_time_integrand;
+  F.function = &_nc_distance_cosmic_time_integrand;
   F.params = cosmo;
 
   if (dist->use_cache)
-    ncm_integral_cached_x_inf (dist->time_cache, &F, z, &result, &error);
+    ncm_integral_cached_x_inf (dist->time_cache, &F, log1p (z), &result, &error);
   else
-    ncm_integral_locked_a_inf (&F, z, NCM_INTEGRAL_ABS_ERROR, NCM_INTEGRAL_ERROR, &result, &error);
+    ncm_integral_locked_a_inf (&F, log1p (z), NCM_INTEGRAL_ABS_ERROR, NCM_INTEGRAL_ERROR, &result, &error);
 
   return result;
 }
@@ -1078,13 +1233,13 @@ nc_distance_lookback_time (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
 {
   gdouble result, error;
   gsl_function F;
-  F.function = &_nc_time_integrand;
+  F.function = &_nc_distance_cosmic_time_integrand;
   F.params = cosmo;
 
   if (dist->use_cache)
-    ncm_integral_cached_0_x (dist->lookback_time_cache, &F, z, &result, &error);
+    ncm_integral_cached_0_x (dist->lookback_time_cache, &F, log1p (z), &result, &error);
   else
-    ncm_integral_locked_a_b (&F, 0.0, z, 0.0, NCM_INTEGRAL_ERROR, &result, &error);
+    ncm_integral_locked_a_b (&F, 0.0, log1p (z), 0.0, NCM_INTEGRAL_ERROR, &result, &error);
 
   return result;
 }
@@ -1105,7 +1260,7 @@ nc_distance_conformal_lookback_time (NcDistance *dist, NcHICosmo *cosmo, gdouble
 }
 
 static gdouble
-nc_conformal_time_integrand (gdouble logx, gpointer p)
+_nc_distance_conformal_time_integrand (gdouble logx, gpointer p)
 {
   if (logx > GSL_LOG_DBL_MAX)
     return 0.0;
@@ -1116,10 +1271,10 @@ nc_conformal_time_integrand (gdouble logx, gpointer p)
     const gdouble x = 1.0 + z;
     const gdouble E = sqrt (nc_hicosmo_E2 (cosmo, z));
 
-    if (!gsl_finite (E))
-      return 0.0;
-    else
+    if (gsl_finite (E))    
       return x / E;
+    else
+      return 0.0;
   }
 }
 
@@ -1138,7 +1293,7 @@ nc_distance_conformal_time (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
   gdouble result, error;
   gsl_function F;
 
-  F.function = &nc_conformal_time_integrand;
+  F.function = &_nc_distance_conformal_time_integrand;
   F.params = cosmo;
 
   if (dist->use_cache)
@@ -1166,6 +1321,7 @@ _NC_DISTANCE_FUNC0_TO_FLIST (acoustic_scale)
 _NC_DISTANCE_FUNC0_TO_FLIST (theta100CMB)
 _NC_DISTANCE_FUNC0_TO_FLIST (angular_diameter_curvature_scale)
 _NC_DISTANCE_FUNC0_TO_FLIST (r_zd)
+_NC_DISTANCE_FUNC0_TO_FLIST (r_zd_Mpc)
 
 #define _NC_DISTANCE_FUNC1_TO_FLIST(fname) \
 static void _nc_distance_flist_##fname (NcmMSetFuncList *flist, NcmMSet *mset, const gdouble *x, gdouble *res) \
@@ -1198,7 +1354,8 @@ _nc_distance_register_functions (void)
   ncm_mset_func_list_register ("acoustic_scale",                   "r_\\mathrm{ac}",           "NcDistance", "Acoustic scale at lss",            NC_TYPE_DISTANCE, _nc_distance_flist_acoustic_scale,                   0, 1);
   ncm_mset_func_list_register ("theta100CMB",                      "100\\theta_\\mathrm{CMB}", "NcDistance", "CMB angular scale times 100",      NC_TYPE_DISTANCE, _nc_distance_flist_theta100CMB,                      0, 1);
   ncm_mset_func_list_register ("angular_diameter_curvature_scale", "z_\\mathrm{dec}",          "NcDistance", "Angular diameter curvature scale", NC_TYPE_DISTANCE, _nc_distance_flist_angular_diameter_curvature_scale, 0, 1);
-  ncm_mset_func_list_register ("r_zd",                             "z_\\mathrm{dec}",          "NcDistance", "Sound horizon at drag redshift",   NC_TYPE_DISTANCE, _nc_distance_flist_r_zd,                             0, 1);
+  ncm_mset_func_list_register ("r_zd",                             "r_\\mathrm{dec}",          "NcDistance", "Sound horizon at drag redshift",   NC_TYPE_DISTANCE, _nc_distance_flist_r_zd,                             0, 1);
+  ncm_mset_func_list_register ("r_zd_Mpc",                         "r_\\mathrm{dec}R_H",       "NcDistance", "Sound horizon at drag redshift in Mpc", NC_TYPE_DISTANCE, _nc_distance_flist_r_zd_Mpc,                    0, 1);
 
   ncm_mset_func_list_register ("comoving",         "d_\\mathrm{c}",               "NcDistance", "Comoving distance",          NC_TYPE_DISTANCE, _nc_distance_flist_comoving,         1, 1);
   ncm_mset_func_list_register ("transverse",       "d_\\mathrm{t}",               "NcDistance", "Transverse distance",        NC_TYPE_DISTANCE, _nc_distance_flist_transverse,       1, 1);
