@@ -8,12 +8,12 @@
 /*
  * numcosmo
  * Copyright (C) Sandro Dias Pinto Vitenti 2012 <sandro@isoftware.com.br>
-   *
+ *
  * numcosmo is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
-   *
+ *
  * numcosmo is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
@@ -35,6 +35,7 @@ typedef struct _TestNcmSerialize
 } TestNcmSerialize;
 
 static void test_ncm_serialize_new (TestNcmSerialize *test, gconstpointer pdata);
+void test_ncm_serialize_new_noclean_dup (TestNcmSerialize *test, gconstpointer pdata);
 static void test_ncm_serialize_free (TestNcmSerialize *test, gconstpointer pdata);
 
 static void test_ncm_serialize_global_from_string_plain (TestNcmSerialize *test, gconstpointer pdata);
@@ -46,6 +47,11 @@ static void test_ncm_serialize_from_string_params_named (TestNcmSerialize *test,
 static void test_ncm_serialize_from_string_nest_named (TestNcmSerialize *test, gconstpointer pdata);
 static void test_ncm_serialize_from_string_nest_samename (TestNcmSerialize *test, gconstpointer pdata);
 
+static void test_ncm_serialize_to_file_from_file (TestNcmSerialize *test, gconstpointer pdata);
+static void test_ncm_serialize_to_binfile_from_binfile (TestNcmSerialize *test, gconstpointer pdata);
+
+static void test_ncm_serialize_reset_autosave_only (TestNcmSerialize *test, gconstpointer pdata);
+
 static void test_ncm_serialize_traps (TestNcmSerialize *test, gconstpointer pdata);
 static void test_ncm_serialize_global_invalid_from_string_syntax (TestNcmSerialize *test, gconstpointer pdata);
 static void test_ncm_serialize_global_invalid_from_string_nonexist (TestNcmSerialize *test, gconstpointer pdata);
@@ -89,12 +95,24 @@ main (gint argc, gchar *argv[])
               &test_ncm_serialize_new,
               &test_ncm_serialize_from_string_nest_samename,
               &test_ncm_serialize_free);
+  g_test_add ("/ncm/serialize/from_string/plain/reset/autosave_only", TestNcmSerialize, NULL,
+              &test_ncm_serialize_new,
+              &test_ncm_serialize_reset_autosave_only,
+              &test_ncm_serialize_free);
+  g_test_add ("/ncm/serialize/to_file/from_file", TestNcmSerialize, NULL,
+              &test_ncm_serialize_new_noclean_dup,
+              &test_ncm_serialize_to_file_from_file,
+              &test_ncm_serialize_free);
+  g_test_add ("/ncm/serialize/to_binfile/from_binfile", TestNcmSerialize, NULL,
+              &test_ncm_serialize_new_noclean_dup,
+              &test_ncm_serialize_to_binfile_from_binfile,
+              &test_ncm_serialize_free);
 
   g_test_add ("/ncm/serialize/traps", TestNcmSerialize, NULL,
               &test_ncm_serialize_new,
               &test_ncm_serialize_traps,
               &test_ncm_serialize_free);
-#if !((GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 38))
+#if GLIB_CHECK_VERSION(2,38,0)
   g_test_add ("/ncm/serialize/invalid/from_string/syntax/subprocess", TestNcmSerialize, NULL,
               &test_ncm_serialize_new,
               &test_ncm_serialize_global_invalid_from_string_syntax,
@@ -130,6 +148,12 @@ test_ncm_serialize_new (TestNcmSerialize *test, gconstpointer pdata)
   test->ser = ncm_serialize_new (NCM_SERIALIZE_OPT_CLEAN_DUP);
 }
 
+void 
+test_ncm_serialize_new_noclean_dup (TestNcmSerialize *test, gconstpointer pdata)
+{
+  test->ser = ncm_serialize_new (0);
+}
+
 void
 test_ncm_serialize_free (TestNcmSerialize *test, gconstpointer pdata)
 {
@@ -229,11 +253,58 @@ test_ncm_serialize_from_string_plain_named (TestNcmSerialize *test, gconstpointe
   g_free (obj_new_ser);
   g_object_unref (obj_new);
 
-  ncm_serialize_clear_instances (test->ser);
+  ncm_serialize_clear_instances (test->ser, FALSE);
   
   NCM_TEST_FREE (nc_hicosmo_free, hic);
 }
 
+static void 
+test_ncm_serialize_reset_autosave_only (TestNcmSerialize *test, gconstpointer pdata)
+{
+  /* Deserialized from string, since it is named it will be automatically added to the saved instances. */
+  GObject *obj1   = ncm_serialize_from_string (test->ser, "NcHICosmoLCDM[myname:is]");
+  /* Deserialized from string, since it is *not* named it will *not* be automatically added to the saved instances.*/
+  GObject *obj2   = ncm_serialize_from_string (test->ser, "NcPowspecMLTransfer{'transfer':<{'NcTransferFuncEH',@a{sv} {}}>}");
+
+  /* Serializing from object */
+  gchar *obj_ser1 = ncm_serialize_to_string (test->ser, obj1, FALSE);
+  /* Serializing from object, all objects will receive a name */
+  gchar *obj_ser2 = ncm_serialize_to_string (test->ser, obj2, FALSE);
+
+  g_assert_cmpuint (ncm_serialize_count_instances (test->ser), ==, 1);
+  g_assert_cmpuint (ncm_serialize_count_saved_serializations (test->ser), >=, 2);
+
+  {
+    GObject *obj1_ref2 = ncm_serialize_from_string (test->ser, obj_ser1);
+    GObject *obj2_ref2 = ncm_serialize_from_string (test->ser, obj_ser2);
+
+    g_assert (obj1_ref2 == obj1);
+    g_assert (obj2_ref2 != obj2);
+
+    g_object_unref (obj1_ref2);
+    g_object_unref (obj2_ref2);
+    
+    g_free (obj_ser1);
+    g_free (obj_ser2);
+  }
+
+  g_assert_cmpuint (ncm_serialize_count_instances (test->ser), >, 1);
+  g_assert_cmpuint (ncm_serialize_count_saved_serializations (test->ser), >=, 2);
+  
+  ncm_serialize_reset (test->ser, TRUE);
+
+  g_assert_cmpuint (ncm_serialize_count_instances (test->ser), ==, 1);
+  g_assert_cmpuint (ncm_serialize_count_saved_serializations (test->ser), ==, 0);
+
+  ncm_serialize_reset (test->ser, FALSE);
+
+  g_assert_cmpuint (ncm_serialize_count_instances (test->ser), ==, 0);
+  g_assert_cmpuint (ncm_serialize_count_saved_serializations (test->ser), ==, 0);
+
+  NCM_TEST_FREE (g_object_unref, obj1);
+  NCM_TEST_FREE (g_object_unref, obj2);
+}
+
 void
 test_ncm_serialize_from_string_params_named (TestNcmSerialize *test, gconstpointer pdata)
 {
@@ -241,14 +312,14 @@ test_ncm_serialize_from_string_params_named (TestNcmSerialize *test, gconstpoint
   ncm_assert_cmpdouble (ncm_model_param_get (m, NC_HICOSMO_DE_H0), ==, 12.3);
   ncm_assert_cmpdouble (ncm_model_param_get (m, NC_HICOSMO_DE_OMEGA_C), ==, 0.2);
 
-  ncm_serialize_clear_instances (test->ser);
+  ncm_serialize_clear_instances (test->ser, FALSE);
   NCM_TEST_FREE (ncm_model_free, m);
 
   m = NCM_MODEL (ncm_serialize_from_string (test->ser, "{'NcHICosmoLCDM[T1]', {'H0':<12.3>,'Omegac':<0.2>}}"));
   ncm_assert_cmpdouble (ncm_model_param_get (m, NC_HICOSMO_DE_H0), ==, 12.3);
   ncm_assert_cmpdouble (ncm_model_param_get (m, NC_HICOSMO_DE_OMEGA_C), ==, 0.2);
 
-  ncm_serialize_clear_instances (test->ser);
+  ncm_serialize_clear_instances (test->ser, FALSE);
   NCM_TEST_FREE (ncm_model_free, m);
 }
 
@@ -284,7 +355,7 @@ test_ncm_serialize_from_string_nest_named (TestNcmSerialize *test, gconstpointer
   g_assert_cmpstr (ncm_serialize_peek_name (test->ser, NC_POWSPEC_ML_TRANSFER (ps_ml)->tf), ==, "T1");
   g_assert_cmpstr (ncm_serialize_peek_name (test->ser, ps_ml), ==, "T0");
 
-  ncm_serialize_clear_instances (test->ser);
+  ncm_serialize_clear_instances (test->ser, FALSE);
   NCM_TEST_FREE (nc_powspec_ml_free, ps_ml);
 }
 
@@ -300,12 +371,12 @@ test_ncm_serialize_from_string_nest_samename (TestNcmSerialize *test, gconstpoin
   gchar *obj_ser, *obj_new_ser;
   GObject *obj_new;
 
-  ncm_serialize_clear_instances (test->ser);
+  ncm_serialize_clear_instances (test->ser, FALSE);
 
   obj_ser     = ncm_serialize_to_string (test->ser, obj, TRUE);
   obj_new     = ncm_serialize_from_string (test->ser, obj_ser);
 
-  ncm_serialize_reset (test->ser);
+  ncm_serialize_reset (test->ser, FALSE);
 
   obj_new_ser = ncm_serialize_to_string (test->ser, obj_new, TRUE);
 
@@ -315,10 +386,10 @@ test_ncm_serialize_from_string_nest_samename (TestNcmSerialize *test, gconstpoin
   g_free (obj_ser);
   g_free (obj_new_ser);
 
-  ncm_serialize_reset (test->ser);
+  ncm_serialize_reset (test->ser, FALSE);
   obj_ser     = ncm_serialize_to_string (test->ser, obj, FALSE);
 
-  ncm_serialize_reset (test->ser);
+  ncm_serialize_reset (test->ser, FALSE);
   obj_new_ser = ncm_serialize_to_string (test->ser, obj_new, FALSE);
   
   g_assert_cmpstr (obj_ser, ==, obj_new_ser);
@@ -326,14 +397,80 @@ test_ncm_serialize_from_string_nest_samename (TestNcmSerialize *test, gconstpoin
   g_free (obj_new_ser);
   g_object_unref (obj_new);
 
-  ncm_serialize_clear_instances (test->ser);
+  ncm_serialize_clear_instances (test->ser, FALSE);
   NCM_TEST_FREE (ncm_spline_free, s);
 }
 
+static void 
+test_ncm_serialize_to_file_from_file (TestNcmSerialize *test, gconstpointer pdata)
+{
+  GObject *obj       = ncm_serialize_from_string (test->ser, "NcHICosmoDEXcdm{'w':<-2.0>}");
+  NcHICosmo *hic     = NC_HICOSMO (obj);
+  gchar *obj_ser     = ncm_serialize_to_string (test->ser, obj, TRUE);
+
+  ncm_serialize_to_file (test->ser, obj, "test-serialize-file.obj");
+  
+  {
+    GObject *obj_new   = ncm_serialize_from_file (test->ser, "test-serialize-file.obj");
+    gchar *obj_new_ser = ncm_serialize_to_string (test->ser, obj_new, TRUE);
+
+    g_assert (G_OBJECT_TYPE (obj) == G_OBJECT_TYPE (obj_new));
+    g_assert_cmpstr (obj_ser, ==, obj_new_ser);
+    g_free (obj_ser);
+    g_free (obj_new_ser);
+
+    obj_ser     = ncm_serialize_to_string (test->ser, obj, FALSE);
+    obj_new_ser = ncm_serialize_to_string (test->ser, obj_new, FALSE);
+
+    g_assert_cmpstr (obj_ser, ==, obj_new_ser);
+
+    g_free (obj_ser);
+    g_free (obj_new_ser);
+    g_object_unref (obj_new);
+
+    ncm_serialize_clear_instances (test->ser, FALSE);
+
+    NCM_TEST_FREE (nc_hicosmo_free, hic);
+  }
+}
+
+static void 
+test_ncm_serialize_to_binfile_from_binfile (TestNcmSerialize *test, gconstpointer pdata)
+{
+  GObject *obj       = ncm_serialize_from_string (test->ser, "NcHICosmoDEXcdm{'w':<-2.0>}");
+  NcHICosmo *hic     = NC_HICOSMO (obj);
+  gchar *obj_ser     = ncm_serialize_to_string (test->ser, obj, TRUE);
+
+  ncm_serialize_to_binfile (test->ser, obj, "test-serialize-binfile.obj");
+  
+  {
+    GObject *obj_new   = ncm_serialize_from_binfile (test->ser, "test-serialize-binfile.obj");
+    gchar *obj_new_ser = ncm_serialize_to_string (test->ser, obj_new, TRUE);
+
+    g_assert (G_OBJECT_TYPE (obj) == G_OBJECT_TYPE (obj_new));
+    g_assert_cmpstr (obj_ser, ==, obj_new_ser);
+    g_free (obj_ser);
+    g_free (obj_new_ser);
+
+    obj_ser     = ncm_serialize_to_string (test->ser, obj, FALSE);
+    obj_new_ser = ncm_serialize_to_string (test->ser, obj_new, FALSE);
+
+    g_assert_cmpstr (obj_ser, ==, obj_new_ser);
+
+    g_free (obj_ser);
+    g_free (obj_new_ser);
+    g_object_unref (obj_new);
+
+    ncm_serialize_clear_instances (test->ser, FALSE);
+
+    NCM_TEST_FREE (nc_hicosmo_free, hic);
+  }
+}
+
 void
 test_ncm_serialize_traps (TestNcmSerialize *test, gconstpointer pdata)
 {
-#if !((GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 38))
+#if GLIB_CHECK_VERSION(2,38,0)
   g_test_trap_subprocess ("/ncm/serialize/invalid/from_string/syntax/subprocess", 0, 0);
   g_test_trap_assert_failed ();
 
