@@ -34,7 +34,7 @@
 #include <gsl/gsl_randist.h>
 #include <gsl/gsl_statistics_double.h>
 
-#define _TEST_NCM_VECTOR_STATIC_SIZE 100
+#define _TEST_NCM_VECTOR_STATIC_SIZE 20
 #define _TEST_NCM_VECTOR_MIN_SIZE 5
 #define _TEST_NCM_STATS_VEC_PREC (1.0e-9)
 #define _TEST_NCM_STATS_VEC_COV_PREC (1.0e-9)
@@ -96,7 +96,7 @@ main (gint argc, gchar *argv[])
               &test_ncm_stats_vec_subsample_autocorr_test, 
               &test_ncm_stats_vec_free);
   
-#if !((GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 38))
+#if GLIB_CHECK_VERSION(2,38,0)
   g_test_add ("/ncm/stats_vec/mean/get_var/subprocess", TestNcmStatsVec, NULL, 
               &test_ncm_stats_vec_mean_new, 
               &test_ncm_stats_vec_invalid_get_var, 
@@ -217,7 +217,7 @@ test_ncm_stats_vec_mean_test (TestNcmStatsVec *test, gconstpointer pdata)
                                             ncm_matrix_ptr (test->xs, 0, k), test->v_size, 
                                             i + 1);
         gdouble svec_mean = ncm_stats_vec_get_mean (test->svec, k);
-        ncm_assert_cmpdouble_e (gsl_mean, ==, svec_mean, _TEST_NCM_STATS_VEC_PREC);
+        ncm_assert_cmpdouble_e (gsl_mean, ==, svec_mean, _TEST_NCM_STATS_VEC_PREC, 0.0);
       }
       
     }
@@ -264,8 +264,8 @@ test_ncm_stats_vec_var_test (TestNcmStatsVec *test, gconstpointer pdata)
                                                i + 1);
         gdouble svec_mean = ncm_stats_vec_get_mean (test->svec, k);
         gdouble svec_var = ncm_stats_vec_get_var (test->svec, k);
-        ncm_assert_cmpdouble_e (gsl_mean, ==, svec_mean, _TEST_NCM_STATS_VEC_PREC);
-        ncm_assert_cmpdouble_e (gsl_var, ==, svec_var, _TEST_NCM_STATS_VEC_PREC);
+        ncm_assert_cmpdouble_e (gsl_mean, ==, svec_mean, _TEST_NCM_STATS_VEC_PREC, 0.0);
+        ncm_assert_cmpdouble_e (gsl_var, ==, svec_var, _TEST_NCM_STATS_VEC_PREC, 0.0);
       }
     }
   }
@@ -316,8 +316,8 @@ test_ncm_stats_vec_cov_test (TestNcmStatsVec *test, gconstpointer pdata)
         gdouble svec_mean = ncm_stats_vec_get_mean (test->svec, k);
         gdouble svec_var = ncm_stats_vec_get_var (test->svec, k);
         guint l;
-        ncm_assert_cmpdouble_e (gsl_mean, ==, svec_mean, _TEST_NCM_STATS_VEC_PREC);
-        ncm_assert_cmpdouble_e (gsl_var, ==, svec_var, _TEST_NCM_STATS_VEC_PREC);
+        ncm_assert_cmpdouble_e (gsl_mean, ==, svec_mean, _TEST_NCM_STATS_VEC_PREC, 0.0);
+        ncm_assert_cmpdouble_e (gsl_var, ==, svec_var, _TEST_NCM_STATS_VEC_PREC, 0.0);
         for (l = k + 1; l < test->v_size; l++)
         {
           const gdouble gsl_cov_kl = gsl_stats_covariance (ncm_matrix_ptr (test->xs, 0, k), test->v_size, 
@@ -326,7 +326,7 @@ test_ncm_stats_vec_cov_test (TestNcmStatsVec *test, gconstpointer pdata)
           const gdouble svec_cov_kl = ncm_stats_vec_get_cov (test->svec, k, l);
           const gdouble svec_cov_lk = ncm_stats_vec_get_cov (test->svec, l, k);
           ncm_assert_cmpdouble (svec_cov_kl, ==, svec_cov_lk);
-          ncm_assert_cmpdouble_e (gsl_cov_kl + 1.0, ==, svec_cov_kl + 1.0, _TEST_NCM_STATS_VEC_COV_PREC);
+          ncm_assert_cmpdouble_e (gsl_cov_kl + 1.0, ==, svec_cov_kl + 1.0, _TEST_NCM_STATS_VEC_COV_PREC, 0.0);
         }
       }
     }
@@ -373,8 +373,8 @@ test_ncm_stats_vec_autocorr_test (TestNcmStatsVec *test, gconstpointer pdata)
     const gdouble svec_mean = ncm_stats_vec_get_mean (test->svec, i);
     const gdouble svec_var = ncm_stats_vec_get_var (test->svec, i);
 
-    ncm_assert_cmpdouble_e (gsl_mean, ==, svec_mean, _TEST_NCM_STATS_VEC_PREC);
-    ncm_assert_cmpdouble_e (gsl_var, ==, svec_var, _TEST_NCM_STATS_VEC_PREC);
+    ncm_assert_cmpdouble_e (gsl_mean, ==, svec_mean, _TEST_NCM_STATS_VEC_PREC, 0.0);
+    ncm_assert_cmpdouble_e (gsl_var, ==, svec_var, _TEST_NCM_STATS_VEC_PREC, 0.0);
 
     {
       NcmVector *ac = ncm_stats_vec_get_autocorr (test->svec, i);
@@ -384,7 +384,7 @@ test_ncm_stats_vec_autocorr_test (TestNcmStatsVec *test, gconstpointer pdata)
       {
         if (ncm_vector_get (ac, j) < 0.0)
           break;
-        ncm_assert_cmpdouble_e (ncm_vector_get (ac, j), ==, pow (a, j), 1.0e-1);
+        ncm_assert_cmpdouble_e (ncm_vector_get (ac, j), ==, pow (a, j), 1.0e-1, 0.0);
       }
       ncm_vector_free (ac);
     }
@@ -435,8 +435,8 @@ test_ncm_stats_vec_subsample_autocorr_test (TestNcmStatsVec *test, gconstpointer
     const gdouble svec_mean = ncm_stats_vec_get_mean (test->svec, i);
     const gdouble svec_var = ncm_stats_vec_get_var (test->svec, i);
 
-    ncm_assert_cmpdouble_e (gsl_mean, ==, svec_mean, _TEST_NCM_STATS_VEC_PREC);
-    ncm_assert_cmpdouble_e (gsl_var, ==, svec_var, _TEST_NCM_STATS_VEC_PREC);
+    ncm_assert_cmpdouble_e (gsl_mean, ==, svec_mean, _TEST_NCM_STATS_VEC_PREC, 0.0);
+    ncm_assert_cmpdouble_e (gsl_var, ==, svec_var, _TEST_NCM_STATS_VEC_PREC, 0.0);
 
     {
       NcmVector *ac = ncm_stats_vec_get_subsample_autocorr (test->svec, i, nchains);
@@ -446,7 +446,7 @@ test_ncm_stats_vec_subsample_autocorr_test (TestNcmStatsVec *test, gconstpointer
       {
         if (ncm_vector_get (ac, j) < 0.0)
           break;
-        ncm_assert_cmpdouble_e (ncm_vector_get (ac, j), ==, pow (a, j), 1.0e-1);
+        ncm_assert_cmpdouble_e (ncm_vector_get (ac, j), ==, pow (a, j), 1.0e-1, 0.0);
       }
       ncm_vector_free (ac);
     }
@@ -470,7 +470,7 @@ test_ncm_stats_vec_invalid_get_cov (TestNcmStatsVec *test, gconstpointer pdata)
 void 
 test_ncm_stats_vec_traps (TestNcmStatsVec *test, gconstpointer pdata)
 {
-#if !((GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 38))
+#if GLIB_CHECK_VERSION(2,38,0)
   g_test_trap_subprocess ("/ncm/stats_vec/mean/get_var/subprocess", 0, 0);
   g_test_trap_assert_failed ();
   
